@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: lamclean.c,v 6.6.2.1 2002/10/09 19:48:37 brbarret Exp $
+ * $Id: lamclean.c,v 6.14 2003/02/27 03:01:11 jsquyres Exp $
  *
  *	Function:	- command to clean out several daemons on all nodes
  */
@@ -22,11 +20,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <sys/param.h>
+#include <netdb.h>
 #include <unistd.h>
 
 #include <args.h>
+#include <etc_misc.h>
 #include <bfreq.h>
-#include <debug.h>
+#include <lamdebug.h>
 #include <freq.h>
 #include <ksignal.h>
 #include <laminternal.h>
@@ -40,14 +41,10 @@
 /*
  * local functions
  */
-static void		help();
+static void		help(void);
 
 int
-main(argc, argv)
-
-int			argc;
-char			*argv[];
-
+main(int argc, char *argv[])
 {
 	int4		*all;		/* array of all node IDs */
 	int4		i;
@@ -56,10 +53,12 @@ char			*argv[];
 
 	/* Ensure that we are not root */
 
+#ifndef LAM_ALLOW_RUN_AS_ROOT
 	if (getuid() == 0 || geteuid() == 0) {
 	  show_help(NULL, "deny-root", NULL);
 	  exit(EACCES);
 	}
+#endif
 
 /*
  * Parse command-line arguments.
@@ -81,7 +80,10 @@ char			*argv[];
  * Initialize and attach to Trollius kernel.
  */
 	if (kinit(PRCMD)) {
-	  show_help(NULL, "no-lamd", "lamclean", NULL);
+	  char hostname[MAXHOSTNAMELEN];
+	  gethostname(hostname, MAXHOSTNAMELEN);
+
+	  show_help(NULL, "no-lamd", "lamclean", hostname, NULL);
 	  exit(LAM_EEXIT);
 	}
 /*
@@ -184,8 +186,7 @@ char			*argv[];
  *	Function:	- prints helpful information on this command
  */
 static void
-help()
-
+help(void)
 {
 	printf("\nSynopsis:	lamclean [options]\n");
 	printf("\nDescription:	Clean up resources on all nodes.\n");
