@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: lamexec.c,v 1.12.2.2 2003/01/20 05:53:44 brbarret Exp $
+ * $Id: lamexec.c,v 1.23 2003/02/26 02:58:19 jsquyres Exp $
  *
  *	Function:	- run commands on a LAM
  */
@@ -28,6 +26,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/param.h>
+#include <netdb.h>
 #include <unistd.h>
 
 #include <all_list.h>
@@ -47,6 +47,7 @@
 #include <sfh.h>
 #include <terror.h>
 #include <typical.h>
+#include <etc_misc.h>
 
 /*
  * private functions
@@ -55,7 +56,7 @@ static int pwait(int4 nwait, int *childstat);
 static int set_mode(void);
 static int set_stdio(void);
 static void reset_stdio(void);
-static void sigint_handler(int sig);
+static void signal_handler(int sig);
 static LIST *build_job(int argc, char **argv, char **env);
 static char *locate_aschema(char *filename);
 static void node_ranges(char *buf, int4 initnode, int4 initindex);
@@ -113,7 +114,7 @@ main(int argc, char **argv)
     ao_setopt(ad, "pty", 0, 0, 0);
     ao_setopt(ad, "npty", "pty", 0, 0);
 
-    if (asc_compat(&argc, &argv)) {
+    if (asc_compat(&argc, &argv, ad)) {
 	perror("lamexec");
 	exit(errno);
     }
@@ -140,14 +141,17 @@ main(int argc, char **argv)
  * Attach to kernel.
  */
     if (kinit(PRCMD)) {
-      show_help(NULL, "no-lamd", "lamexec", NULL);
+      char hostname[MAXHOSTNAMELEN];
+      gethostname(hostname, MAXHOSTNAMELEN);
+
+      show_help(NULL, "no-lamd", "lamexec", hostname, NULL);
       exit(errno);
     }
 /*
  * Set job identifier to be inherited by the programs.
  */
     _kio.ki_jobid.jid_node = getnodeid();
-    _kio.ki_jobid.jid_pid = getpid();
+    _kio.ki_jobid.jid_pid = lam_getpid();
 /*
  * Get the job environment.
  */
@@ -180,7 +184,8 @@ main(int argc, char **argv)
  * Set signal handler to terminate the job.
  */
     if (fl_wait) {
-	_lam_signal(SIGINT, sigint_handler);
+	_lam_signal(SIGINT, signal_handler);
+	_lam_signal(SIGTERM, signal_handler);
     }
 
     reset_stdio();
@@ -367,11 +372,19 @@ pwait(int4 nwait, int *childstat)
 			return(LAMERROR);
 		}
 
+		status &= ~(_LAM_WIFNODETACH);
 		if (status) {
+		    if (LAM_WIFSIGNALED(status)) {
+			printf("%d (n%d) exited due to signal %d\n",
+				pid, nodeid, LAM_WTERMSIG(status));
+			*childstat = -1;
+		    } else {
+			status = LAM_WEXITSTATUS(status);
 			printf("%d (n%d) exited with status %d\n",
 				pid, nodeid, status);
 			*childstat = status;
-			return(LAMERROR);
+		    }
+		    return(LAMERROR);
 		}
 	}
 
@@ -444,7 +457,7 @@ set_stdio(void)
 /*
  * Pass stdin, stdout and stderr to filed.
  */
-	if (lam_mktmpid((int) getpid(), server, sizeof(server))) {
+	if (lam_mktmpid((int) lam_getpid(), server, sizeof(server))) {
 		return(LAMERROR);
 	}
 
@@ -574,18 +587,19 @@ node_ranges(char *buf, int4 initnode, int4 initindex)
 }
 
 /*
- *	sigint_handler
+ *	signal_handler
  *
  *	Function:	- dooms application started by lamexec
  *	Accepts:	- signal
  */
 static void
-sigint_handler(int sig)
+signal_handler(int sig)
 {
 	static int	numsigs = 0;		/* # times signal caught */
 	int		pid;
 
-	if (sig != SIGINT) return;
+	if (sig != SIGINT && sig != SIGTERM) 
+	  return;
 /*
  * Second time around just exit.
  */
@@ -597,7 +611,7 @@ sigint_handler(int sig)
 		return;
 	}
 	else if (pid == 0) {
-
+		lam_reset_pid_cache();
 		if (kinit(PRCMD)) {
 			exit(errno);
 		}
@@ -623,7 +637,7 @@ error_cleanup(void)
     sleep(1);
 
     LAM_ZERO_ME(msg);
-    msg.nh_event = (-getpid()) & 0xBFFFFFFF;
+    msg.nh_event = (-lam_getpid()) & 0xBFFFFFFF;
     msg.nh_length = 0;
     msg.nh_flags = 0;
 /*
@@ -635,4 +649,3 @@ error_cleanup(void)
 	    break;
     }
 }
-
