@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: filed.c,v 6.4.2.1 2002/10/09 19:48:42 brbarret Exp $
+ *	$Id: filed.c,v 6.6 2003/03/10 03:22:05 jsquyres Exp $
  * 
  *	Function:	- filed mainline 
  *			- remote daemon
@@ -70,6 +68,7 @@
 #include <terror.h>
 #include <typical.h>
 
+
 /*
  * external functions
  */
@@ -93,12 +92,14 @@ extern void		fqsystem();
 extern void		fqrmdir();
 extern void		fqunlink();
 extern void		fqincr();
+extern void		fqpathfind();
 extern void		fqf77read();
 extern void		fqf77back();
 extern void		fqopenfd();
 extern void		fdstdio();	/* set up 0, 1 and 2 */
 extern int		fmyinit();	/* machine dependent initialization */
 
+
 /*
  * local variables
  */
@@ -126,6 +127,7 @@ static void		(*(fqfunc[FQMAX]))() = {
 			fqtrunc,
 			fqftrunc,
 			fqopenfd,
+			fqpathfind,
 			fqf77read,
 			fqf77back,
 			};
@@ -134,20 +136,21 @@ static void		(*(fqfunc[FQMAX]))() = {
  */
 char			fbuf[MAXNMSGLEN];
 
+
 /*
  * global functions
  */
 void			(*(f_init()))();
 void			(*(filed()))();
 
+
 /*
  *	f_init
  *
  *	Function:	- filed initialization
  */
-void (*(
-f_init()))()
-
+void 
+(*(f_init()))()
 {
         memset((void*) fbuf, -1, MAXNMSGLEN);
 	if (fmyinit()) lampanic("filed (fmyinit)");
@@ -181,7 +184,6 @@ f_init()))()
  */
 void
 (*(filed()))()
-
 {
 	struct freq	*fq;		/* filed request details */
 
@@ -207,6 +209,7 @@ void
 	return((void (*)()) filed);
 }
 
+
 /*
  *	fsendr
  *
@@ -222,15 +225,7 @@ void
  *			- message flags
  */
 void
-fsendr(node, event, err, ret, length, flags)
-
-int4			node;
-int4			event;
-int			err;
-int4			ret;
-int4			length;
-int4			flags;
-
+fsendr(int4 node, int4 event, int err, int4 ret, int4 length, int4 flags)
 {
 	struct nmsg	outgoing;	/* outgoing filed reply */
 	struct freply	*fr;		/* filed reply details */
