@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: fqopenfd.c,v 6.3 1999/06/12 17:11:29 kmeyer1 Exp $
+ * $Id: fqopenfd.c,v 6.6 2003/06/01 14:03:52 jsquyres Exp $
  *
  *	Function:	- FQOPENFD request
  *			- opens passed file descriptors
@@ -57,6 +37,7 @@
 #include <portable.h>
 #include <terror.h>
 #include <typical.h>
+#include <etc_misc.h>
 
 /*
  * public functions
@@ -153,14 +134,14 @@ struct freq		*fq;
 	}
 	
 	if (isatty(fd[1])) {
-		strncpy(fname, ttyname(fd[1]), FNAMELEN);
+		lam_strncpy(fname, ttyname(fd[1]), FNAMELEN);
 	} else {
 		sprintf(fname, "stdout.%d", -fq->fq_src_event);
 	}
 	f[1] = passed_fd_new(fd[1], fname, fq);
 	
 	if (isatty(fd[2])) {
-		strncpy(fname, ttyname(fd[2]), FNAMELEN);
+		lam_strncpy(fname, ttyname(fd[2]), FNAMELEN);
 	} else {
 		sprintf(fname, "stderr.%d", -fq->fq_src_event);
 	}
