@@ -1,36 +1,21 @@
 /*
- *
- * Copyright 1998-1999, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Kinis L. Meyer with M. D. McNally 
- *          and Andrew Lumsdaine
- *
- * This file is part of the Notre Dame LAM implementation of MPI.
- *
- * You should have received a copy of the License Agreement for the
- * Notre Dame LAM implementation of MPI along with the software; see
- * the file LICENSE.  If not, contact Office of Research, University
- * of Notre Dame, Notre Dame, IN 46556.
- *
- * Permission to modify the code and to distribute modified code is
- * granted, provided the text of this NOTICE is retained, a notice that
- * the code was modified is included with the above COPYRIGHT NOTICE and
- * with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
- * file is distributed with the modified code.
- *
- * LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
- * By way of example, but not limitation, Licensor MAKES NO
- * REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
- * PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
- * OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
- * OR OTHER RIGHTS.  
- *
- * Additional copyrights may follow.
- *
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
+ * 
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
+ * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1997 The Ohio State University
  *	NJN
  *
- *	$Id: iod.ext.c,v 6.4 1999/07/28 00:31:51 jsquyres Exp $
+ *	$Id: iod.ext.c,v 6.7.2.2 2003/10/11 19:04:00 brbarret Exp $
  *
  *	Function:	- external IO daemon
  */
@@ -41,8 +26,9 @@
 #include <string.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <unistd.h>
 
-#if LAM_NEED_SYS_SELECT_H
+#if defined(HAVE_SYS_SELECT_H) && HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
