@@ -10,11 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1997 The Ohio State University
  *	GDB/NJN
  *
- *	$Id: pqcreate.c,v 6.16.2.1 2002/10/09 19:48:45 brbarret Exp $
+ *	$Id: pqcreate.c,v 6.20 2003/04/09 03:37:46 brbarret Exp $
  *
  *	Function:	- OTB kenyad
  *			- creates a process
@@ -42,7 +43,7 @@
 #include "t_types.h"
 #include "terror.h"
 #include "typical.h"
-#include "lamlog.h"
+#include "lamdebug.h"
 
 /*
  * private functions
@@ -62,6 +63,7 @@ extern struct pprocess	ptable[];		/* kenya process table */
 extern struct preq	pq;			/* kenya request */
 extern struct preply	pr;			/* kenya reply */
 extern char		**pathv;		/* argv of search paths */
+extern mode_t original_umask;
 
 /*
  * external functions
@@ -231,13 +233,14 @@ pcreate(char *loadname, int4 argc, char **argv, char **envv)
   
   pnew = ptable + inew;
   pnew->p_loadpt = loadname;
-  pnew->p_rtf = pq.pq_rtf | RTF_KENYA;
+  pnew->p_rtf = pq.pq_rtf | RTF_KENYA_CHILD;
   pnew->p_argc = argc;
   pnew->p_argv = argv;
   pnew->p_jobid.jid_node = pq.pq_jobid.jid_node;
   pnew->p_jobid.jid_pid = pq.pq_jobid.jid_pid;
   pnew->p_nodeid = pq.pq_src_node;
   pnew->p_event = pq.pq_src_event;
+  pnew->p_status = NOTSTATUS;
 /*
  * Receive stdio file descriptors. The process about to be created
  * inherits them.
@@ -323,7 +326,8 @@ pcreate(char *loadname, int4 argc, char **argv, char **envv)
   sigprocmask(SIG_BLOCK, &newset, &oldset);
   
   pnew->p_pid = kcreatev(loadname, argv, envv,
-			 wrkdir, (pq.pq_rtf & RTF_PFDIO) ? fds : 0);
+			 wrkdir, (pq.pq_rtf & RTF_PFDIO) ? fds : 0,
+			 original_umask);
   
   sigprocmask(SIG_SETMASK, &oldset, (sigset_t *) 0);
 /*
