@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: kernelio.c,v 6.13.2.3 2002/10/09 19:48:45 brbarret Exp $
+ *	$Id: kernelio.c,v 6.29.2.3 2003/10/11 19:04:01 brbarret Exp $
  * 
  *	Function:	- receives kernel requests and send replies
  *			- moves messages between clients
@@ -29,7 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if HAVE_STRINGS_H
+#if defined(HAVE_STRINGS_H) && HAVE_STRINGS_H
 #include <strings.h>
 #endif
 #include <sys/types.h>
@@ -38,17 +36,21 @@
 #include <sys/uio.h>
 #include <sys/un.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <unistd.h>
 
-#if LAM_NEED_SYS_SELECT_H
+#if defined(HAVE_SYS_SELECT_H) && HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
 #include <dl_inet.h>
+#include <lamdebug.h>
+#include <laminternal.h>
 #include <kreq.h>
 #include <preq.h>
 #include <terror.h>
 #include <typical.h>
+#include <sfh.h>
 
 #define KIMAX		(4 + 2 * PMAX)
 #define localaddr_t	struct sockaddr *
@@ -79,8 +81,6 @@ extern struct kproc	*kpfindport();		/* find client by port */
 extern void		kkillall();		/* kill all clients */
 extern void		knuke();		/* remove client */
 extern void		(*_lam_signal())();	/* portable signal() */
-extern char		*killname();		/* get kill filename */
-extern char		*sockname();		/* get socket filename */
 extern int		kkillopen();		/* open kill file */
 extern int		mread();
 extern int		mreadv();
@@ -91,6 +91,7 @@ extern int		mwritev();
  * external variables
  */
 extern struct kproc	*prun;			/* current running client */
+extern char **environ;
 
 /*
  * local variables
@@ -110,9 +111,6 @@ static int		to_count;		/* timeout estimater */
 
 static char		fwdbuf[KPKTLEN];	/* forwarding buffer */
 
-static struct sockaddr_un
-			kernel_un;		/* kernel address */
-
 static struct {
 	void		(*kn_func)();		/* interrupt function */
 	int		kn_fd;			/* interrupt file desc. */
@@ -136,8 +134,12 @@ kio_init()
 {
 	char		*f_kill;		/* kill filename */
 	char		*f_sock;		/* socket filename */
-	int		i;
+	int		i, len;
 	mode_t          mode;
+        struct sockaddr_un kernel_un;
+        char *orig_dir;
+        char *addr_dir;
+        char *addr_file;
 /*
  * Initialize the interrupt fd->func table.
  */
@@ -162,9 +164,9 @@ kio_init()
  * Set up local address.
  */
 	mode = umask(0177);
-	kernel_un.sun_family = AF_UNIX;
-	f_sock = sockname();
-	memcpy((char *) kernel_un.sun_path, f_sock, strlen(f_sock));
+	if (lam_tmpdir_init(NULL, NULL) != 0)
+	  lampanic("lamd kernel: problem making tmp directory");
+
 /*
  * Create the socket.
  */
@@ -172,11 +174,52 @@ kio_init()
 	  lampanic("lamd kernel: problem with socket()");
 
 	fd_max = sd_kernel;
+
+        /* In some cases, we may be trying to open a *really* long
+           socket name.  And that may be longer that the size allowed
+           by sockaddr_un.  So instead of trying to open the socket
+           directly, chdir to the basename of the address and open
+           just the filename of the socket.  Then chdr back to the
+           original directory.  If the filename of the socket is
+           longer than sockaddr_un allows, well, then you're out of
+           luck.  :-) */
+
+        orig_dir = getworkdir();
+
+        /* Get the socket name.  Ensure to malloc a big enough
+           kernel_un to hold the entire path (the socket name may be
+           longer than the default socket_un struct allows) */
+
+	f_sock = lam_get_sockname();
+        addr_dir = f_sock;
+        addr_file = strrchr(addr_dir, '/');
+        if (addr_file != NULL) {
+          *addr_file = '\0';
+          chdir(addr_dir);
+          ++addr_file;
+        } else {
+          addr_file = addr_dir;
+        }
+        if (strlen(addr_file) > sizeof(kernel_un.sun_path)) {
+          lampanic("lamd kernel: kernelio unix socket filename too long");
+        }
+
+	memset(&kernel_un, 0, sizeof(kernel_un));
+	kernel_un.sun_family = AF_UNIX;
+	memcpy((char *) kernel_un.sun_path, addr_file, strlen(addr_file) + 1);
+
+#if LAM_HAVE_SA_LEN
+	len = sizeof(kernel_un.sun_len) + sizeof(kernel_un.sun_family) +
+		strlen(kernel_un.sun_path) + 1;
+	kernel_un.sun_len = len;
+#else
+	len = strlen(kernel_un.sun_path) + sizeof(kernel_un.sun_family);
+#endif
+
 /*
  * Bind the kernel's address to the socket.
  */
-	if (bind(sd_kernel, (localaddr_t) &kernel_un, sizeof(kernel_un) -
-		 sizeof(kernel_un.sun_path) + strlen(f_sock))) {
+	if (bind(sd_kernel, (localaddr_t) &kernel_un, len) != 0) {
 	  if (errno == EACCES) {
 	    fprintf(stderr, "lamd kernel: permission denied trying to bind to the named socket:\n");
 	    fprintf(stderr, "lamd kernel: \"%s\"\n", kernel_un.sun_path);
@@ -185,16 +228,21 @@ kio_init()
 	  } else
 	    lampanic("lamd kernel: problem with bind()");
 	}
-	free(f_sock);
 /*
  * Establish backlog queue.
  */
 	if (listen(sd_kernel, 5)) 
 	  lampanic("lamd kernel: problem with listen()");
+
+        /* Go back to the original directory and free temp memory */
+
+        chdir(orig_dir);
+        free(orig_dir);
+        free(addr_dir);
 /*
  * Open the kill record file.
  */
-	if ((f_kill = killname()) == 0) 
+	if ((f_kill = lam_get_killfile()) == 0) 
 	  lampanic("lamd kernel: problem with internal call killname()");
 
 	if (kkillopen(f_kill)) {
@@ -296,6 +344,7 @@ kio_req()
 			    &length);
 		    if (sd_new < 0) 
 		      lampanic("lamd kernel: problem with accept()");
+		    lamlog("kio_req: new client on fd=%d", sd_new);
 		    nfd_ready--;
 		    FD_SET(sd_new, &allfds);
 		    FD_SET(sd_new, &clientfds);
@@ -607,9 +656,35 @@ kio_fd_ready()
 static void
 kio_shutdown()
 {
+  pid_t pid;
+  int status = 0;
+
+  pid = fork();
+ 
+  if (pid == 0) {
+    /* child */
+
+    /* GRRR....  Can't look at options in here - just use #define */
+#if LAM_WANT_DEBUG
+    execle(LAM_BINDIR "/tkill", "tkill", "-setsid", "-d", "-debug-output",
+	   "syslog", NULL, environ);
+#else
+    execle(LAM_BINDIR "/tkill", "tkill", "-setsid", NULL, environ);
+#endif
+
+    /* Well, that didn't go well.  Parent already trying to clean up... */
+    exit(1);
+  }
+
 	kkillall();
 	shutdown(sd_kernel, 2);
 	kio_cleanup();
+
+	if (pid >= 0) {
+	  /* Wait for tkill to get us, so that tkill must get scheduled... */
+	  waitpid(pid, &status, 0);
+	}
+
 	exit(0);
 }
 
