@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/GDB/AKP
+ * $HEADER$
  *
- *	$Id: tping.c,v 6.13.2.1 2002/10/09 19:48:48 brbarret Exp $
+ * $Id: tping.c,v 6.21 2003/02/27 03:01:15 jsquyres Exp $
  *
  *	Function:	- bounces a message off the echo daemon
  */
@@ -25,16 +23,19 @@
 #include <signal.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/param.h>
+#include <netdb.h>
 #include <unistd.h>
 
 #include <args.h>
-#include <debug.h>
+#include <lamdebug.h>
 #include <ereq.h>
 #include <net.h>
 #include <priority.h>
 #include <terror.h>
 #include <typical.h>
 #include <dl_inet.h>
+#include <etc_misc.h>
 
 #define plural(n)	((n != 1) ? "s" : "")
 
@@ -84,10 +85,12 @@ main(int argc, char *argv[])
 
 	/* Ensure that we are not root */
 
+#ifndef LAM_ALLOW_RUN_AS_ROOT
 	if (getuid() == 0 || geteuid() == 0) {
 	  show_help(NULL, "deny-root", NULL);
 	  exit(EACCES);
 	}
+#endif
 
 /*
  * Parse the command line.
@@ -148,7 +151,10 @@ main(int argc, char *argv[])
  * Attach to the Trollius kernel.
  */
 	if (kinit(PRCMD)) {
-	  show_help(NULL, "no-lamd", "tping", NULL);
+	  char hostname[MAXHOSTNAMELEN];
+	  gethostname(hostname, MAXHOSTNAMELEN);
+
+	  show_help(NULL, "no-lamd", "tping", hostname, NULL);
 	  exit(errno);
 	}
 
