@@ -1,43 +1,37 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: deletef.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: deletef.c,v 1.4.2.2 2003/06/24 13:28:40 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#if _UNICOS
+#ifdef _UNICOS
 #include <fortran.h>
 #endif
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
+
+/* LAM lam_strncpy(), because some platforms have a broken strncpy.
+   <sigh> */
+char *lam_strncpy(char *dest, const char *src, int len);
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_delete_ PMPI_FILE_DELETE
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_delete_ pmpi_file_delete__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_delete pmpi_file_delete_
-#endif
-#define mpi_file_delete_ pmpi_file_delete
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_delete_ pmpi_file_delete
-#endif
-#define mpi_file_delete_ pmpi_file_delete_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL mpi_file_delete_(char *filename FORT_MIXED_LEN(str_len), MPI_Fint *info, int *ierr FORT_END_LEN(str_len));
 #pragma weak MPI_FILE_DELETE = PMPI_FILE_DELETE
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_delete__(char *filename FORT_MIXED_LEN(str_len), MPI_Fint *info, int *ierr FORT_END_LEN(str_len));
 #pragma weak mpi_file_delete__ = pmpi_file_delete__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_delete(char *filename FORT_MIXED_LEN(str_len), MPI_Fint *info, int *ierr FORT_END_LEN(str_len));
 #pragma weak mpi_file_delete = pmpi_file_delete
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_delete_(char *filename FORT_MIXED_LEN(str_len), MPI_Fint *info, int *ierr FORT_END_LEN(str_len));
 #pragma weak mpi_file_delete_ = pmpi_file_delete_
 #endif
 
@@ -69,6 +63,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_delete_ PMPI_FILE_DELETE
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_delete_ pmpi_file_delete__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_delete pmpi_file_delete_
+#endif
+#define mpi_file_delete_ pmpi_file_delete
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_delete_ pmpi_file_delete
+#endif
+#define mpi_file_delete_ pmpi_file_delete_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -88,15 +98,27 @@
 #endif
 
 /* Prototype to keep compiler happy */
-void mpi_file_delete_(char *filename, MPI_Fint *info, int *ierr, int str_len);
+/*
+FORTRAN_API void FORT_CALL mpi_file_delete_(char *filename, MPI_Fint *info, int *ierr, int str_len);
 
-#if _UNICOS
+#ifdef _UNICOS
 void mpi_file_delete_(_fcd filename_fcd, MPI_Fint *info, int *ierr)
 {
     char *filename = _fcdtocp(filename_fcd);
     int str_len = _fcdlen(filename_fcd);
 #else
-void mpi_file_delete_(char *filename, MPI_Fint *info, int *ierr, int str_len)
+FORTRAN_API void FORT_CALL mpi_file_delete_(char *filename, MPI_Fint *info, int *ierr, int str_len)
+*/
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_delete_(char *filename FORT_MIXED_LEN_DECL, MPI_Fint *info, int *ierr FORT_END_LEN_DECL);
+
+#ifdef _UNICOS
+void mpi_file_delete_(_fcd filename_fcd, MPI_Fint *info, int *ierr)
+{
+    char *filename = _fcdtocp(filename_fcd);
+    int str_len = _fcdlen(filename_fcd);
+#else
+FORTRAN_API void FORT_CALL mpi_file_delete_(char *filename FORT_MIXED_LEN(str_len), MPI_Fint *info, int *ierr FORT_END_LEN(str_len))
 {
 #endif
     char *newfname;
@@ -118,7 +140,7 @@ void mpi_file_delete_(char *filename, MPI_Fint *info, int *ierr, int str_len)
     real_len = i + 1;
 
     newfname = (char *) ADIOI_Malloc((real_len+1)*sizeof(char));
-    strncpy(newfname, filename, real_len);
+    lam_strncpy(newfname, filename, real_len);
     newfname[real_len] = '\0';
 
     *ierr = MPI_File_delete(newfname, info_c);
