@@ -1,40 +1,30 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: get_bytofff.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: get_bytofff.c,v 1.3.2.1 2003/06/23 18:56:17 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_get_byte_offset_ PMPI_FILE_GET_BYTE_OFFSET
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_get_byte_offset_ pmpi_file_get_byte_offset__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_byte_offset pmpi_file_get_byte_offset_
-#endif
-#define mpi_file_get_byte_offset_ pmpi_file_get_byte_offset
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_byte_offset_ pmpi_file_get_byte_offset
-#endif
-#define mpi_file_get_byte_offset_ pmpi_file_get_byte_offset_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_GET_BYTE_OFFSET(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr );
 #pragma weak MPI_FILE_GET_BYTE_OFFSET = PMPI_FILE_GET_BYTE_OFFSET
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_byte_offset__(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr );
 #pragma weak mpi_file_get_byte_offset__ = pmpi_file_get_byte_offset__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_byte_offset(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr );
 #pragma weak mpi_file_get_byte_offset = pmpi_file_get_byte_offset
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_get_byte_offset_(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr );
 #pragma weak mpi_file_get_byte_offset_ = pmpi_file_get_byte_offset_
 #endif
 
@@ -66,6 +56,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_get_byte_offset_ PMPI_FILE_GET_BYTE_OFFSET
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_get_byte_offset_ pmpi_file_get_byte_offset__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_byte_offset pmpi_file_get_byte_offset_
+#endif
+#define mpi_file_get_byte_offset_ pmpi_file_get_byte_offset
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_byte_offset_ pmpi_file_get_byte_offset
+#endif
+#define mpi_file_get_byte_offset_ pmpi_file_get_byte_offset_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -85,9 +91,9 @@
 #endif
 
 /* Prototype to keep compiler happy */
-void mpi_file_get_byte_offset_(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr );
+FORTRAN_API void FORT_CALL mpi_file_get_byte_offset_(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr );
 
-void mpi_file_get_byte_offset_(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr )
+FORTRAN_API void FORT_CALL mpi_file_get_byte_offset_(MPI_Fint *fh,MPI_Offset *offset,MPI_Offset *disp, int *ierr )
 {
     MPI_File fh_c;
     
