@@ -1,40 +1,34 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: get_extentf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: get_extentf.c,v 1.3.2.2 2003/06/24 20:11:43 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_get_type_extent_ PMPI_FILE_GET_TYPE_EXTENT
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_get_type_extent_ pmpi_file_get_type_extent__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_type_extent pmpi_file_get_type_extent_
-#endif
-#define mpi_file_get_type_extent_ pmpi_file_get_type_extent
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_type_extent_ pmpi_file_get_type_extent
-#endif
-#define mpi_file_get_type_extent_ pmpi_file_get_type_extent_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_GET_TYPE_EXTENT(MPI_Fint *fh,MPI_Fint *datatype,
+                                                            MPI_Fint *extent, int *ierr );
 #pragma weak MPI_FILE_GET_TYPE_EXTENT = PMPI_FILE_GET_TYPE_EXTENT
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_type_extent__(MPI_Fint *fh,MPI_Fint *datatype,
+                                                            MPI_Fint *extent, int *ierr );
 #pragma weak mpi_file_get_type_extent__ = pmpi_file_get_type_extent__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_type_extent(MPI_Fint *fh,MPI_Fint *datatype,
+                                                            MPI_Fint *extent, int *ierr );
 #pragma weak mpi_file_get_type_extent = pmpi_file_get_type_extent
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Fint *datatype,
+                                                            MPI_Fint *extent, int *ierr );
 #pragma weak mpi_file_get_type_extent_ = pmpi_file_get_type_extent_
 #endif
 
@@ -66,6 +60,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_get_type_extent_ PMPI_FILE_GET_TYPE_EXTENT
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_get_type_extent_ pmpi_file_get_type_extent__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_type_extent pmpi_file_get_type_extent_
+#endif
+#define mpi_file_get_type_extent_ pmpi_file_get_type_extent
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_type_extent_ pmpi_file_get_type_extent
+#endif
+#define mpi_file_get_type_extent_ pmpi_file_get_type_extent_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -84,16 +94,11 @@
 #endif
 #endif
 
-/* Prototype to keep compiler happy */
 #if defined(MPIHP) || defined(MPILAM)
+/* Prototype to keep compiler happy */
 void mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Fint *datatype,
 			       MPI_Fint *extent, int *ierr );
-#else
-void mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Datatype *datatype,
-			       MPI_Fint *extent, int *ierr );
-#endif
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Fint *datatype,
                              MPI_Fint *extent, int *ierr )
 {
@@ -109,8 +114,11 @@ void mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Fint *datatype,
 }
 
 #else
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Datatype *datatype,
+			       MPI_Fint *extent, int *ierr );
 
-void mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Datatype *datatype,
+FORTRAN_API void FORT_CALL mpi_file_get_type_extent_(MPI_Fint *fh,MPI_Datatype *datatype,
                              MPI_Fint *extent, int *ierr )
 {
     MPI_File fh_c;
