@@ -1,40 +1,30 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: get_groupf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: get_groupf.c,v 1.3.2.2 2003/06/24 20:11:43 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_get_group_ PMPI_FILE_GET_GROUP
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_get_group_ pmpi_file_get_group__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_group pmpi_file_get_group_
-#endif
-#define mpi_file_get_group_ pmpi_file_get_group
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_group_ pmpi_file_get_group
-#endif
-#define mpi_file_get_group_ pmpi_file_get_group_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_GET_GROUP(MPI_Fint *fh,MPI_Fint *group, int *ierr );
 #pragma weak MPI_FILE_GET_GROUP = PMPI_FILE_GET_GROUP
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_group__(MPI_Fint *fh,MPI_Fint *group, int *ierr );
 #pragma weak mpi_file_get_group__ = pmpi_file_get_group__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_group(MPI_Fint *fh,MPI_Fint *group, int *ierr );
 #pragma weak mpi_file_get_group = pmpi_file_get_group
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_get_group_(MPI_Fint *fh,MPI_Fint *group, int *ierr );
 #pragma weak mpi_file_get_group_ = pmpi_file_get_group_
 #endif
 
@@ -66,6 +56,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_get_group_ PMPI_FILE_GET_GROUP
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_get_group_ pmpi_file_get_group__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_group pmpi_file_get_group_
+#endif
+#define mpi_file_get_group_ pmpi_file_get_group
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_group_ pmpi_file_get_group
+#endif
+#define mpi_file_get_group_ pmpi_file_get_group_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -84,14 +90,10 @@
 #endif
 #endif
 
+#if defined(MPIHP) || defined(MPILAM)
 /* Prototype to keep compiler happy */
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_get_group_(MPI_Fint *fh,MPI_Fint *group, int *ierr );
-#else
-void mpi_file_get_group_(MPI_Fint *fh,MPI_Group *group, int *ierr );
-#endif
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_get_group_(MPI_Fint *fh,MPI_Fint *group, int *ierr )
 {
     MPI_File fh_c;
@@ -102,7 +104,10 @@ void mpi_file_get_group_(MPI_Fint *fh,MPI_Fint *group, int *ierr )
     *group = MPI_Group_c2f(group_c);
 }
 #else
-void mpi_file_get_group_(MPI_Fint *fh,MPI_Group *group, int *ierr )
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_get_group_(MPI_Fint *fh,MPI_Group *group, int *ierr );
+
+FORTRAN_API void FORT_CALL mpi_file_get_group_(MPI_Fint *fh,MPI_Group *group, int *ierr )
 {
     MPI_File fh_c;
     
