@@ -1,43 +1,40 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: get_viewf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: get_viewf.c,v 1.4.2.3 2003/06/24 20:11:43 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#if _UNICOS
+#ifdef _UNICOS
 #include <fortran.h>
 #endif
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
+/* LAM lam_strncpy(), because some platforms have a broken strncpy.
+   <sigh> */
+char *lam_strncpy(char *dest, const char *src, int len);
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_get_view_ PMPI_FILE_GET_VIEW
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_get_view_ pmpi_file_get_view__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_view pmpi_file_get_view_
-#endif
-#define mpi_file_get_view_ pmpi_file_get_view
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_get_view_ pmpi_file_get_view
-#endif
-#define mpi_file_get_view_ pmpi_file_get_view_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_GET_VIEW(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+                                                     MPI_Fint *filetype,char *datarep, int *ierr, int str_len );
 #pragma weak MPI_FILE_GET_VIEW = PMPI_FILE_GET_VIEW
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_view__(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+                                                     MPI_Fint *filetype,char *datarep, int *ierr, int str_len );
 #pragma weak mpi_file_get_view__ = pmpi_file_get_view__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_get_view(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+                                                     MPI_Fint *filetype,char *datarep, int *ierr, int str_len );
 #pragma weak mpi_file_get_view = pmpi_file_get_view
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+                                                     MPI_Fint *filetype,char *datarep, int *ierr, int str_len );
 #pragma weak mpi_file_get_view_ = pmpi_file_get_view_
 #endif
 
@@ -69,6 +66,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_get_view_ PMPI_FILE_GET_VIEW
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_get_view_ pmpi_file_get_view__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_view pmpi_file_get_view_
+#endif
+#define mpi_file_get_view_ pmpi_file_get_view
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_get_view_ pmpi_file_get_view
+#endif
+#define mpi_file_get_view_ pmpi_file_get_view_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -87,16 +100,11 @@
 #endif
 #endif
 
+#if defined(MPIHP) || defined(MPILAM)
 /* Prototype to keep compiler happy */
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
-			MPI_Fint *filetype,char *datarep, int *ierr, int str_len );
-#else
-void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
-		MPI_Datatype *filetype,char *datarep, int *ierr, int str_len );
-#endif
+		MPI_Fint *filetype,char *datarep, int *ierr, int str_len );
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
    MPI_Fint *filetype,char *datarep, int *ierr, int str_len )
 {
@@ -116,14 +124,14 @@ void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
 
     tmpreplen = strlen(tmprep);
     if (tmpreplen <= str_len) {
-        strncpy(datarep, tmprep, tmpreplen);
+        lam_strncpy(datarep, tmprep, tmpreplen);
 
         /* blank pad the remaining space */
         for (i=tmpreplen; i<str_len; i++) datarep[i] = ' ';
     }
     else {
         /* not enough space */
-        strncpy(datarep, tmprep, str_len);
+        lam_strncpy(datarep, tmprep, str_len);
         /* this should be flagged as an error. */
         *ierr = MPI_ERR_UNKNOWN;
     }
@@ -135,14 +143,18 @@ void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
 
 #else
 
-#if _UNICOS
+#ifdef _UNICOS
 void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
    MPI_Datatype *filetype, _fcd datarep_fcd, int *ierr)
 {
     char *datarep = _fcdtocp(datarep_fcd);
     int str_len = _fcdlen(datarep_fcd);
 #else
-void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
+		MPI_Datatype *filetype,char *datarep, int *ierr, int str_len );
+
+FORTRAN_API void FORT_CALL mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
    MPI_Datatype *filetype,char *datarep, int *ierr, int str_len )
 {
 #endif
@@ -162,14 +174,14 @@ void mpi_file_get_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
 
     tmpreplen = strlen(tmprep);
     if (tmpreplen <= str_len) {
-        strncpy(datarep, tmprep, tmpreplen);
+        lam_strncpy(datarep, tmprep, tmpreplen);
 
         /* blank pad the remaining space */
         for (i=tmpreplen; i<str_len; i++) datarep[i] = ' ';
     }
     else {
         /* not enough space */
-        strncpy(datarep, tmprep, str_len);
+        lam_strncpy(datarep, tmprep, str_len);
         /* this should be flagged as an error. */
         *ierr = MPI_ERR_UNKNOWN;
     }
