@@ -1,40 +1,34 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: iwritef.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: iwritef.c,v 1.3.2.2 2003/06/24 20:11:43 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_iwrite_ PMPI_FILE_IWRITE
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_iwrite_ pmpi_file_iwrite__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_iwrite pmpi_file_iwrite_
-#endif
-#define mpi_file_iwrite_ pmpi_file_iwrite
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_iwrite_ pmpi_file_iwrite
-#endif
-#define mpi_file_iwrite_ pmpi_file_iwrite_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_IWRITE(MPI_Fint *fh,void *buf,int *count,
+                                                   MPI_Fint *datatype,MPI_Fint *request, int *ierr );
 #pragma weak MPI_FILE_IWRITE = PMPI_FILE_IWRITE
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_iwrite__(MPI_Fint *fh,void *buf,int *count,
+                                                   MPI_Fint *datatype,MPI_Fint *request, int *ierr );
 #pragma weak mpi_file_iwrite__ = pmpi_file_iwrite__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_iwrite(MPI_Fint *fh,void *buf,int *count,
+                                                   MPI_Fint *datatype,MPI_Fint *request, int *ierr );
 #pragma weak mpi_file_iwrite = pmpi_file_iwrite
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
+                                                   MPI_Fint *datatype,MPI_Fint *request, int *ierr );
 #pragma weak mpi_file_iwrite_ = pmpi_file_iwrite_
 #endif
 
@@ -66,6 +60,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_iwrite_ PMPI_FILE_IWRITE
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_iwrite_ pmpi_file_iwrite__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_iwrite pmpi_file_iwrite_
+#endif
+#define mpi_file_iwrite_ pmpi_file_iwrite
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_iwrite_ pmpi_file_iwrite
+#endif
+#define mpi_file_iwrite_ pmpi_file_iwrite_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -84,16 +94,11 @@
 #endif
 #endif
 
-/* Prototype to keep compiler happy */
 #if defined(MPIHP) || defined(MPILAM)
+/* Prototype to keep compiler happy */
 void mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
 		      MPI_Fint *datatype,MPI_Fint *request, int *ierr );
-#else
-void mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
-		      MPI_Datatype *datatype,MPI_Fint *request, int *ierr );
-#endif
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
                     MPI_Fint *datatype,MPI_Fint *request, int *ierr )
 {
@@ -108,7 +113,11 @@ void mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
     *request = MPIO_Request_c2f(req_c);
 }
 #else
-void mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
+		      MPI_Datatype *datatype,MPI_Fint *request, int *ierr );
+
+FORTRAN_API void FORT_CALL mpi_file_iwrite_(MPI_Fint *fh,void *buf,int *count,
                     MPI_Datatype *datatype,MPI_Fint *request, int *ierr )
 {
     MPI_File fh_c;
