@@ -1,43 +1,41 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: openf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: openf.c,v 1.4.2.3 2003/06/24 20:11:43 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#if _UNICOS
+#ifdef _UNICOS
 #include <fortran.h>
 #endif
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
+
+/* LAM lam_strncpy(), because some platforms have a broken strncpy.
+   <sigh> */
+char *lam_strncpy(char *dest, const char *src, int len);
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_open_ PMPI_FILE_OPEN
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_open_ pmpi_file_open__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_open pmpi_file_open_
-#endif
-#define mpi_file_open_ pmpi_file_open
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_open_ pmpi_file_open
-#endif
-#define mpi_file_open_ pmpi_file_open_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_OPEN(MPI_Fint *comm,char *filename,int *amode,
+                          MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
 #pragma weak MPI_FILE_OPEN = PMPI_FILE_OPEN
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_open__(MPI_Fint *comm,char *filename,int *amode,
+                            MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
 #pragma weak mpi_file_open__ = pmpi_file_open__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_open(MPI_Fint *comm,char *filename,int *amode,
+                          MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
 #pragma weak mpi_file_open = pmpi_file_open
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_open_(MPI_Fint *comm,char *filename,int *amode,
+                           MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
 #pragma weak mpi_file_open_ = pmpi_file_open_
 #endif
 
@@ -69,6 +67,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_open_ PMPI_FILE_OPEN
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_open_ pmpi_file_open__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_open pmpi_file_open_
+#endif
+#define mpi_file_open_ pmpi_file_open
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_open_ pmpi_file_open
+#endif
+#define mpi_file_open_ pmpi_file_open_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -87,16 +101,11 @@
 #endif
 #endif
 
-/* Prototype to keep compiler happy */
 #if defined(MPIHP) || defined(MPILAM)
+/* Prototype to keep compiler happy */
 void mpi_file_open_(MPI_Fint *comm,char *filename,int *amode,
 		    MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
-#else
-void mpi_file_open_(MPI_Comm *comm,char *filename,int *amode,
-		    MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
-#endif
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_open_(MPI_Fint *comm,char *filename,int *amode,
                   MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len )
 {
@@ -122,7 +131,7 @@ void mpi_file_open_(MPI_Fint *comm,char *filename,int *amode,
     real_len = i + 1;
 
     newfname = (char *) ADIOI_Malloc((real_len+1)*sizeof(char));
-    strncpy(newfname, filename, real_len);
+    lam_strncpy(newfname, filename, real_len);
     newfname[real_len] = '\0';
 
     *ierr = MPI_File_open(comm_c, newfname, *amode, info_c, &fh_c);
@@ -133,15 +142,27 @@ void mpi_file_open_(MPI_Fint *comm,char *filename,int *amode,
 
 #else
 
-#if _UNICOS
+#ifdef _UNICOS
 void mpi_file_open_(MPI_Comm *comm,_fcd filename_fcd,int *amode,
                   MPI_Fint *info, MPI_Fint *fh, int *ierr)
 {
     char *filename = _fcdtocp(filename_fcd);
     int str_len = _fcdlen(filename_fcd);
 #else
-void mpi_file_open_(MPI_Comm *comm,char *filename,int *amode,
+/* Prototype to keep compiler happy */
+/*
+FORTRAN_API void FORT_CALL mpi_file_open_(MPI_Comm *comm,char *filename,int *amode,
+		    MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len );
+
+FORTRAN_API void FORT_CALL mpi_file_open_(MPI_Comm *comm,char *filename,int *amode,
                   MPI_Fint *info, MPI_Fint *fh, int *ierr, int str_len )
+*/
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_open_(MPI_Comm *comm,char *filename FORT_MIXED_LEN_DECL,int *amode,
+		    MPI_Fint *info, MPI_Fint *fh, int *ierr FORT_END_LEN_DECL);
+
+FORTRAN_API void FORT_CALL mpi_file_open_(MPI_Comm *comm,char *filename FORT_MIXED_LEN(str_len),int *amode,
+                  MPI_Fint *info, MPI_Fint *fh, int *ierr FORT_END_LEN(str_len))
 {
 #endif
     char *newfname;
@@ -164,7 +185,7 @@ void mpi_file_open_(MPI_Comm *comm,char *filename,int *amode,
     real_len = i + 1;
 
     newfname = (char *) ADIOI_Malloc((real_len+1)*sizeof(char));
-    strncpy(newfname, filename, real_len);
+    lam_strncpy(newfname, filename, real_len);
     newfname[real_len] = '\0';
 
     *ierr = MPI_File_open(*comm, newfname, *amode, info_c, &fh_c);
