@@ -1,40 +1,30 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: set_atomf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: set_atomf.c,v 1.3.2.1 2003/06/23 18:56:18 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_set_atomicity_ PMPI_FILE_SET_ATOMICITY
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_set_atomicity_ pmpi_file_set_atomicity__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_set_atomicity pmpi_file_set_atomicity_
-#endif
-#define mpi_file_set_atomicity_ pmpi_file_set_atomicity
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_set_atomicity_ pmpi_file_set_atomicity
-#endif
-#define mpi_file_set_atomicity_ pmpi_file_set_atomicity_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_SET_ATOMICITY(MPI_Fint *fh,int *flag, int *ierr );
 #pragma weak MPI_FILE_SET_ATOMICITY = PMPI_FILE_SET_ATOMICITY
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_set_atomicity__(MPI_Fint *fh,int *flag, int *ierr );
 #pragma weak mpi_file_set_atomicity__ = pmpi_file_set_atomicity__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_set_atomicity(MPI_Fint *fh,int *flag, int *ierr );
 #pragma weak mpi_file_set_atomicity = pmpi_file_set_atomicity
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_set_atomicity_(MPI_Fint *fh,int *flag, int *ierr );
 #pragma weak mpi_file_set_atomicity_ = pmpi_file_set_atomicity_
 #endif
 
@@ -66,6 +56,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_set_atomicity_ PMPI_FILE_SET_ATOMICITY
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_set_atomicity_ pmpi_file_set_atomicity__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_set_atomicity pmpi_file_set_atomicity_
+#endif
+#define mpi_file_set_atomicity_ pmpi_file_set_atomicity
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_set_atomicity_ pmpi_file_set_atomicity
+#endif
+#define mpi_file_set_atomicity_ pmpi_file_set_atomicity_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -85,9 +91,9 @@
 #endif
 
 /* Prototype to keep compiler happy */
-void mpi_file_set_atomicity_(MPI_Fint *fh,int *flag, int *ierr );
+FORTRAN_API void FORT_CALL mpi_file_set_atomicity_(MPI_Fint *fh,int *flag, int *ierr );
 
-void mpi_file_set_atomicity_(MPI_Fint *fh,int *flag, int *ierr )
+FORTRAN_API void FORT_CALL mpi_file_set_atomicity_(MPI_Fint *fh,int *flag, int *ierr )
 {
     MPI_File fh_c;
     
