@@ -1,43 +1,44 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: set_viewf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: set_viewf.c,v 1.4.2.3 2003/06/24 20:11:44 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#if _UNICOS
+#ifdef _UNICOS
 #include <fortran.h>
 #endif
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
+/* LAM lam_strncpy(), because some platforms have a broken strncpy.
+   <sigh> */
+char *lam_strncpy(char *dest, const char *src, int len);
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_set_view_ PMPI_FILE_SET_VIEW
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_set_view_ pmpi_file_set_view__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_set_view pmpi_file_set_view_
-#endif
-#define mpi_file_set_view_ pmpi_file_set_view
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_set_view_ pmpi_file_set_view
-#endif
-#define mpi_file_set_view_ pmpi_file_set_view_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_SET_VIEW(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+   MPI_Fint *filetype,char *datarep,MPI_Fint *info, int *ierr,
+                                                     int str_len );
 #pragma weak MPI_FILE_SET_VIEW = PMPI_FILE_SET_VIEW
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_set_view__(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+   MPI_Fint *filetype,char *datarep,MPI_Fint *info, int *ierr,
+                                                     int str_len );
 #pragma weak mpi_file_set_view__ = pmpi_file_set_view__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_set_view(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+   MPI_Fint *filetype,char *datarep,MPI_Fint *info, int *ierr,
+                                                     int str_len );
 #pragma weak mpi_file_set_view = pmpi_file_set_view
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
+   MPI_Fint *filetype,char *datarep,MPI_Fint *info, int *ierr,
+                                                     int str_len );
 #pragma weak mpi_file_set_view_ = pmpi_file_set_view_
 #endif
 
@@ -69,6 +70,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_set_view_ PMPI_FILE_SET_VIEW
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_set_view_ pmpi_file_set_view__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_set_view pmpi_file_set_view_
+#endif
+#define mpi_file_set_view_ pmpi_file_set_view
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_set_view_ pmpi_file_set_view
+#endif
+#define mpi_file_set_view_ pmpi_file_set_view_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -87,18 +104,12 @@
 #endif
 #endif
 
-/* Prototype to keep compiler happy */
 #if defined(MPIHP) || defined(MPILAM)
+/* Prototype to keep compiler happy */
 void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
    MPI_Fint *filetype,char *datarep,MPI_Fint *info, int *ierr,
 			int str_len );
-#else
-void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
-   MPI_Datatype *filetype,char *datarep,MPI_Fint *info, int *ierr,
-			int str_len );
-#endif
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
    MPI_Fint *filetype,char *datarep,MPI_Fint *info, int *ierr,
    int str_len )
@@ -126,7 +137,7 @@ void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
     real_len = i + 1;
 
     newstr = (char *) ADIOI_Malloc((real_len+1)*sizeof(char));
-    strncpy(newstr, datarep, real_len);
+    lam_strncpy(newstr, datarep, real_len);
     newstr[real_len] = '\0';
     
     fh_c = MPI_File_f2c(*fh);
@@ -138,14 +149,19 @@ void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Fint *etype,
 
 #else
 
-#if _UNICOS
+#ifdef _UNICOS
 void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
    MPI_Datatype *filetype,_fcd datarep_fcd,MPI_Fint *info, int *ierr)
 {
    char *datarep = _fcdtocp(datarep_fcd);
    int str_len = _fcdlen(datarep_fcd);
 #else
-void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
+   MPI_Datatype *filetype,char *datarep,MPI_Fint *info, int *ierr,
+			int str_len );
+
+FORTRAN_API void FORT_CALL mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
    MPI_Datatype *filetype,char *datarep,MPI_Fint *info, int *ierr,
    int str_len )
 {
@@ -170,7 +186,7 @@ void mpi_file_set_view_(MPI_Fint *fh,MPI_Offset *disp,MPI_Datatype *etype,
     real_len = i + 1;
 
     newstr = (char *) ADIOI_Malloc((real_len+1)*sizeof(char));
-    strncpy(newstr, datarep, real_len);
+    lam_strncpy(newstr, datarep, real_len);
     newstr[real_len] = '\0';
     
     fh_c = MPI_File_f2c(*fh);
