@@ -1,40 +1,34 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /* 
- *   $Id: write_allf.c,v 1.1.1.1 2000/09/06 20:01:54 jsquyres Exp $    
+ *   $Id: write_allf.c,v 1.3.2.2 2003/06/24 20:11:44 jsquyres Exp $    
  *
  *   Copyright (C) 1997 University of Chicago. 
  *   See COPYRIGHT notice in top-level directory.
  */
 
-#include "mpio.h"
 #include "adio.h"
+#include "mpio.h"
 
 
 #if defined(MPIO_BUILD_PROFILING) || defined(HAVE_WEAK_SYMBOLS)
-#ifdef FORTRANCAPS
-#define mpi_file_write_all_ PMPI_FILE_WRITE_ALL
-#elif defined(FORTRANDOUBLEUNDERSCORE)
-#define mpi_file_write_all_ pmpi_file_write_all__
-#elif !defined(FORTRANUNDERSCORE)
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_write_all pmpi_file_write_all_
-#endif
-#define mpi_file_write_all_ pmpi_file_write_all
-#else
-#if defined(HPUX) || defined(SPPUX)
-#pragma _HP_SECONDARY_DEF pmpi_file_write_all_ pmpi_file_write_all
-#endif
-#define mpi_file_write_all_ pmpi_file_write_all_
-#endif
 
 #if defined(HAVE_WEAK_SYMBOLS)
 #if defined(HAVE_PRAGMA_WEAK)
 #if defined(FORTRANCAPS)
+extern FORTRAN_API void FORT_CALL MPI_FILE_WRITE_ALL(MPI_Fint *fh,void *buf,int *count,
+                  MPI_Fint *datatype,MPI_Status *status, int *ierr );
 #pragma weak MPI_FILE_WRITE_ALL = PMPI_FILE_WRITE_ALL
 #elif defined(FORTRANDOUBLEUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_write_all__(MPI_Fint *fh,void *buf,int *count,
+                  MPI_Fint *datatype,MPI_Status *status, int *ierr );
 #pragma weak mpi_file_write_all__ = pmpi_file_write_all__
 #elif !defined(FORTRANUNDERSCORE)
+extern FORTRAN_API void FORT_CALL mpi_file_write_all(MPI_Fint *fh,void *buf,int *count,
+                  MPI_Fint *datatype,MPI_Status *status, int *ierr );
 #pragma weak mpi_file_write_all = pmpi_file_write_all
 #else
+extern FORTRAN_API void FORT_CALL mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
+                  MPI_Fint *datatype,MPI_Status *status, int *ierr );
 #pragma weak mpi_file_write_all_ = pmpi_file_write_all_
 #endif
 
@@ -66,6 +60,22 @@
 #include "mpioprof.h"
 #endif
 
+#ifdef FORTRANCAPS
+#define mpi_file_write_all_ PMPI_FILE_WRITE_ALL
+#elif defined(FORTRANDOUBLEUNDERSCORE)
+#define mpi_file_write_all_ pmpi_file_write_all__
+#elif !defined(FORTRANUNDERSCORE)
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_write_all pmpi_file_write_all_
+#endif
+#define mpi_file_write_all_ pmpi_file_write_all
+#else
+#if defined(HPUX) || defined(SPPUX)
+#pragma _HP_SECONDARY_DEF pmpi_file_write_all_ pmpi_file_write_all
+#endif
+#define mpi_file_write_all_ pmpi_file_write_all_
+#endif
+
 #else
 
 #ifdef FORTRANCAPS
@@ -84,19 +94,13 @@
 #endif
 #endif
 
+#if defined(MPIHP) || defined(MPILAM)
 /* Prototype to keep compiler happy */
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
-			 MPI_Fint *datatype,MPI_Status *status, int *ierr );
-#else
-void mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
-		 MPI_Datatype *datatype,MPI_Status *status, int *ierr );
-#endif
+		 MPI_Fint *datatype,MPI_Status *status, int *ierr );
 
-#if defined(MPIHP) || defined(MPILAM)
 void mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
-                       MPI_Fint *datatype,MPI_Status *status, int *ierr )
-{
+                       MPI_Fint *datatype,MPI_Status *status, int *ierr ){
     MPI_File fh_c;
     MPI_Datatype datatype_c;
     
@@ -106,8 +110,12 @@ void mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
     *ierr = MPI_File_write_all(fh_c,buf,*count,datatype_c,status);
 }
 #else
-void mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
-                       MPI_Datatype *datatype,MPI_Status *status, int *ierr ){
+/* Prototype to keep compiler happy */
+FORTRAN_API void FORT_CALL mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
+		 MPI_Datatype *datatype,MPI_Status *status, int *ierr );
+
+FORTRAN_API void FORT_CALL mpi_file_write_all_(MPI_Fint *fh,void *buf,int *count,
+                  MPI_Datatype *datatype,MPI_Status *status, int *ierr ){
     MPI_File fh_c;
     
     fh_c = MPI_File_f2c(*fh);
