@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,6 +10,7 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Software for Humanity
  *	GDB/RBD
  *
@@ -17,7 +18,7 @@
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- *	$Id: all_argv.c,v 6.7.2.1 2002/10/09 19:48:49 brbarret Exp $
+ *	$Id: all_argv.c,v 6.9 2003/05/31 22:28:49 jsquyres Exp $
  *
  *	Function:	- argument vector library
  */
@@ -25,7 +26,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "lam.h"
+#include <lam.h>
+#include <etc_misc.h>
 
 #define ARGSIZE		128
 
@@ -137,7 +139,7 @@ sfh_argv_break(char *args, int delim)
 			argtemp = (char*) malloc((unsigned) (arglen + 1));
 			if (argtemp == 0) return(0);
 
-			strncpy(argtemp, args, arglen);
+			lam_strncpy(argtemp, args, arglen);
 			argtemp[arglen] = '\0';
 
 			if (sfh_argv_add(&argc, &argv, argtemp)) {
@@ -151,7 +153,7 @@ sfh_argv_break(char *args, int delim)
  * short argument, copy to buffer and add
  */
 		else {
-			strncpy(arg, args, arglen);
+			lam_strncpy(arg, args, arglen);
 			arg[arglen] = '\0';
 
 			if (sfh_argv_add(&argc, &argv, arg)) return(0);
