@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1995 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: all_opt.c,v 6.4.2.1 2002/10/09 19:48:49 brbarret Exp $
+ * $Id: all_opt.c,v 6.10 2003/03/22 15:31:55 jsquyres Exp $
  * 
  *	Function:	- generic option parsing package
  *			- uses malloc()
@@ -27,6 +25,7 @@
 #include <portable.h>
 #include <terror.h>
 #include <typical.h>
+#include <sfh.h>
 
 /*
  * local functions
@@ -34,8 +33,8 @@
 static int addinst(struct ao_opt *opt, int nparam, char **params);
 static int countparam(OPT *aod, struct ao_opt *opt, char **argv);
 static int mutexchk(struct ao_opt *opt);
-static int strtonum(char *str, int *pnum);
-static struct ao_opt *findopt(OPT *desc, char *optstr);
+static int strtonum(const char *str, int *pnum);
+static struct ao_opt *findopt(OPT *desc, const char *optstr);
 
 
 /*
@@ -45,7 +44,7 @@ static struct ao_opt *findopt(OPT *desc, char *optstr);
  *	Returns:	- ptr option desc. or NULL
  */
 OPT *
-ao_init()
+ao_init(void)
 {
 	OPT		*aod;			/* ptr option desc. */
 
@@ -56,6 +55,17 @@ ao_init()
 		aod->ao_opttop = 0;
 		aod->ao_optend = 0;
 		aod->ao_optcur = 0;
+
+		aod->ao_flags = 0;
+
+		aod->ao_argv0 = 0;
+
+		aod->ao_unusedc = 0;
+		aod->ao_unusedv = 0;
+
+		aod->ao_argv_dup = 0;
+                aod->ao_string_dups = 0;
+
 		aod->ao_tailc = 0;
 		aod->ao_tailv = 0;
 	}
@@ -63,6 +73,7 @@ ao_init()
 	return(aod);
 }
 
+
 /*
  *	ao_free
  *
@@ -95,9 +106,25 @@ ao_free(OPT *aod)
 		free((char *) p);
 	}
 
+	/* Free the argv0 copy */
+
+	if (aod->ao_argv0)
+	  free(aod->ao_argv0);
+
+	/* If AOPRESERVE_ARGV was set, we need to free the argv and
+           corresponding strings that were allocated in ao_parse(). */
+
+	if (aod->ao_argv_dup != NULL) {
+          for (i = 0; aod->ao_string_dups[i] != NULL; ++i)
+            free(aod->ao_string_dups[i]);
+          free(aod->ao_string_dups);
+	  free(aod->ao_argv_dup);
+        }
+
 	free((char *) aod);
 }
 
+
 /*
  *	ao_ntaken
  *
@@ -111,6 +138,7 @@ ao_ntaken(OPT *aod)
 	return((aod) ? aod->ao_ntaken : 0);
 }
 
+
 /*
  *	ao_taken
  *
@@ -120,7 +148,7 @@ ao_ntaken(OPT *aod)
  *	Returns:	- TRUE or FALSE
  */
 int
-ao_taken(OPT *aod, char *opt)
+ao_taken(OPT *aod, const char *opt)
 {
 	struct ao_opt	*p;			/* favourite pointer */
 
@@ -132,6 +160,7 @@ ao_taken(OPT *aod, char *opt)
 	return(FALSE);
 }
 
+
 /*
  *	ao_chosen
  *
@@ -142,7 +171,7 @@ ao_taken(OPT *aod, char *opt)
  *	Returns:	- chosen/taken option or NULL
  */
 char *
-ao_chosen(OPT *aod, char *opt)
+ao_chosen(OPT *aod, const char *opt)
 {
 	struct ao_opt	*p;			/* favourite pointer */
 
@@ -167,6 +196,7 @@ ao_chosen(OPT *aod, char *opt)
 	return((p) ? p->aoo_string : (char *) 0);
 }
 
+
 /*
  *	ao_ninsts
  *
@@ -176,7 +206,7 @@ ao_chosen(OPT *aod, char *opt)
  *	Returns:	- # instances
  */
 int
-ao_ninsts(OPT *aod, char *opt)
+ao_ninsts(OPT *aod, const char *opt)
 {
 	struct ao_opt	*p;			/* favourite pointer */
 
@@ -185,6 +215,7 @@ ao_ninsts(OPT *aod, char *opt)
 	return((p && (p->aoo_ninst > 0)) ? p->aoo_ninst : 0);
 }
 
+
 /*
  *	ao_nparams
  *
@@ -195,7 +226,7 @@ ao_ninsts(OPT *aod, char *opt)
  *	Returns:	- # parameters
  */
 int
-ao_nparams(OPT *aod, char *opt, int inst)
+ao_nparams(OPT *aod, const char *opt, int inst)
 {
 	struct ao_opt	*p;			/* favourite pointer */
 	int		ret = 0;		/* returned value */
@@ -212,6 +243,7 @@ ao_nparams(OPT *aod, char *opt, int inst)
 	return(ret);
 }
 
+
 /*
  *	ao_param
  *
@@ -223,7 +255,7 @@ ao_nparams(OPT *aod, char *opt, int inst)
  *	Returns:	- parameter string or NULL
  */
 char *
-ao_param(OPT *aod, char *opt, int inst, int idx)
+ao_param(OPT *aod, const char *opt, int inst, int idx)
 {
 	struct ao_opt	*p = 0;			/* favourite pointer */
 	struct ao_inst	*pinst;			/* ptr instance */
@@ -244,6 +276,7 @@ ao_param(OPT *aod, char *opt, int inst, int idx)
 	return(param);
 }
 
+
 /*
  *	ao_intparam
  *
@@ -256,7 +289,7 @@ ao_param(OPT *aod, char *opt, int inst, int idx)
  *	Returns:	- 0 or LAMERROR
  */
 int
-ao_intparam(OPT *aod, char *opt, int inst, int idx, int *pparam)
+ao_intparam(OPT *aod, const char *opt, int inst, int idx, int *pparam)
 {
 	struct ao_opt	*p = 0;			/* favourite pointer */
 	struct ao_inst	*pinst;			/* ptr instance */
@@ -279,6 +312,7 @@ ao_intparam(OPT *aod, char *opt, int inst, int idx, int *pparam)
 
 }
 
+
 /*
  *	ao_tail
  *
@@ -302,6 +336,67 @@ ao_tail(OPT *aod, int *tailc, char ***tailv)
 	return(ret);
 }
 
+
+/*
+ *	ao_unused
+ *
+ *	Function:	- return the unused argc/argv
+ *	Accepts:	- ptr option descriptor
+ *			- ptr unused argc (returned value)
+ *			- ptr unused argv (returned value)
+ *	Returns:	- 0 or LAMERROR
+ */
+int
+ao_unused(OPT *aod, int *unusedc, char ***unusedv)
+{
+  int ret = LAMERROR;
+
+  if (aod) {
+    *unusedc = aod->ao_unusedc;
+    *unusedv = aod->ao_unusedv;
+    ret = 0;
+  }
+
+  return ret;
+}
+
+
+/*
+ *	ao_setflags
+ *
+ *	Function:	- return the unused argc/argv
+ *	Accepts:	- ptr option descriptor
+ *			- flags
+ *	Returns:	- 0 or LAMERROR
+ */
+int
+ao_setflags(OPT *aod, int flags)
+{
+  int ret = LAMERROR;
+
+  if (aod) {
+    aod->ao_flags = flags;
+    ret = 0;
+  }
+
+  return ret;
+}
+
+
+/*
+ *	ao_argv0
+ *
+ *	Function:	- return a copy of the original argv[0]
+ *	Accepts:	- ptr option descriptor
+ *	Returns:	- ptr to copy of original argv[0] or NULL
+ */
+char *
+ao_argv0(OPT *aod)
+{
+  return (aod) ? aod->ao_argv0 : NULL;
+}
+
+
 /*
  *	ao_setopt
  *
@@ -315,8 +410,7 @@ ao_tail(OPT *aod, int *tailc, char ***tailv)
  *	Returns:	- 0 or LAMERROR
  */
 int
-ao_setopt(OPT *aod, char *opt, char *mutex, int nparams,
-	  int flags)
+ao_setopt(OPT *aod, const char *opt, const char *mutex, int nparams, int flags)
 {
 	struct ao_opt	*p;			/* favourite pointer */
 	struct ao_opt	*mtx;			/* ptr mutex option */
@@ -379,6 +473,7 @@ ao_setopt(OPT *aod, char *opt, char *mutex, int nparams,
 	return(0);
 }
 
+
 /*
  *	ao_setopt1
  *
@@ -392,10 +487,10 @@ ao_setopt(OPT *aod, char *opt, char *mutex, int nparams,
  *	Returns:	- 0 or LAMERROR
  */
 int
-ao_setopt1(OPT *aod, char *opt, char *mutex, int nparams,
+ao_setopt1(OPT *aod, const char *opt, const char *mutex, int nparams,
 	   int flags)
 {
-	char	*p;
+	const char	*p;
 	char		opt1[2];
 
 	opt1[1] = '\0';
@@ -411,6 +506,7 @@ ao_setopt1(OPT *aod, char *opt, char *mutex, int nparams,
 	return(0);
 }
 
+
 /*
  *	ao_parse
  *
@@ -427,6 +523,7 @@ ao_setopt1(OPT *aod, char *opt, char *mutex, int nparams,
 int
 ao_parse(OPT *aod, int *pcmdc, char **cmdv)
 {
+	int		i;
 	int		num;			/* parsed number */
 	int		fl_onelet;		/* one-letter options? */
 	int		argc;			/* argc for parsing */
@@ -451,6 +548,80 @@ ao_parse(OPT *aod, int *pcmdc, char **cmdv)
 			break;
 		}
 	}
+
+	/* If AOPRESERVE_ARGV is set, copy argc/argv into ao_unused*,
+	   and operate from there */
+
+	if (aod->ao_flags & AOPRESERVE_ARGV) {
+	  aod->ao_unusedc = *pcmdc;
+	  aod->ao_unusedv = aod->ao_argv_dup = sfh_argv_dup(cmdv);
+
+          /* Remember that sfh_argv_dup() mallocs two major kinds of
+             things: the argv vector itself (i.e., char **), and a set
+             of strings that are held in the argv vector (i.e., a
+             bunch of (char*)'s).  We've taken care of freeing the
+             argv vector by holding its valie in ao_argv_dup (which is
+             freed in ao_free()).  But the individual strings must
+             also be freed when no longer used.
+
+             This is a problem because it violates one of the original
+             assumptions of the ao_parse code: in the parsing
+             algorithm below, when an option is reconized, its string
+             is simply discarded, assuming that it was an argument to
+             main(), and therefore we don't need to free it.  However,
+             with AOPRESERVE_ARGV, this string now *is* specifically
+             malloc'ed (and therefore must be freed when it is no
+             longer being used).  There are two options on how to keep
+             track of all of these strings so that they can be freed
+             properly:
+
+             1. Implement a new type that holds the string and a flag
+                indicating whether the string needs to be freed or
+                not.  Instead of having an argv of (char*), now have
+                an argv of this new type.  Whenever an element in this
+                argv is discarded, examine the flag and see if the
+                string needs to be freed.
+
+             2. Save all malloc'ed strings when they are initially
+                strdup'ed.  At the end of time, free all of them. 
+
+             The second approach is slightly more wasteful in that the
+             strings will not be freed as they are discarded by the
+             parsing algorithm, but it still frees them all at the end
+             of time.  And it's dramatically simpler and less
+             error-prone to implement.  Hence, that's what we'll
+             do. 
+
+             Since the vector pointed to by ao_argv_dup will shrink as
+             options are removed from it, we need another vector that
+             simply holds pointers to all the strings that were dup'ed
+             by sfh_argv_dup (above) -- and doesn't dup them again.
+             Hence, we have ao_string_dups, which is a
+             manually-malloc'ed vector that we preserve all the string
+             pointers in so that they can be freed in ao_free(). */
+
+          aod->ao_string_dups = malloc(sizeof(char**) * (*pcmdc + 1));
+          for (i = 0; i < *pcmdc; ++i)
+            aod->ao_string_dups[i] = aod->ao_argv_dup[i];
+          aod->ao_string_dups[i] = NULL;
+
+	  pcmdc = NULL;
+	  cmdv = NULL;
+	}
+
+	/* If pcmdc/cmdv are NULL, use the unusedc/unusedv (e.g., if
+	   this is a re-parse) */
+
+	if (pcmdc == NULL && cmdv == NULL) {
+	  pcmdc = &aod->ao_unusedc;
+	  cmdv = aod->ao_unusedv;
+	}
+
+	/* Save argv[0] if we haven't done so already */
+
+	if (aod->ao_argv0 == NULL && cmdv != NULL && cmdv[0] != NULL)
+	  aod->ao_argv0 = strdup(cmdv[0]);
+
 /*
  * Skip argv[0].
  */
@@ -615,6 +786,68 @@ ao_parse(OPT *aod, int *pcmdc, char **cmdv)
 	return(0);
 }
 
+
+/*
+ * ao_exam_isopt
+ *
+ * Function:
+ *  Determine whether opt has been added to the options descriptor or
+ *  not.
+ * Accepts:
+ *  aod - options descriptor
+ *  opt - option in question
+ * Returns:
+ *  1        - opt has been added
+ *  0        - opt has not bee added
+ */
+int
+ao_exam_isopt(OPT *aod, const char *opt)
+{
+  struct ao_opt	*p;
+
+  if (aod) {
+    p = findopt(aod, opt);
+    if (p)
+      return 1;
+  }
+
+  return 0;
+} 
+
+
+/*
+ * ao_exam_nparams
+ *
+ * Function:
+ *  Find number of expected (NOT TAKEN) params for a given option
+ *
+ * Accepts:
+ *  aod - options descriptor
+ *  opt - option in question
+ * Output:
+ *  nparams - number of params (May be AOVARNUM)
+ * Returns:
+ *  0        - success
+ *  LAMERROR - internal accounting error (aka option not found)
+ */
+int
+ao_exam_nparams(OPT *aod, const char *opt, int *nparams)
+{
+  struct ao_opt *p;
+
+  if (aod) {
+    p = findopt(aod, opt);
+
+    if (p) {
+      *nparams = p->aoo_nmaxparam;
+      return 0;
+    }
+  }
+
+  return LAMERROR;
+}
+
+
 /*
  *	findopt
  *
@@ -624,7 +857,7 @@ ao_parse(OPT *aod, int *pcmdc, char **cmdv)
  *	Returns:	- ptr option or NULL
  */
 static struct ao_opt *
-findopt(OPT *desc, char *optstr)
+findopt(OPT *desc, const char *optstr)
 {
 	struct ao_opt	*p;			/* favourite pointer */
 /*
@@ -649,6 +882,7 @@ findopt(OPT *desc, char *optstr)
 	return(p);
 }
 
+
 /*
  *	strtonum
  *
@@ -660,7 +894,7 @@ findopt(OPT *desc, char *optstr)
  *	Returns:	- 0 or LAMERROR
  */
 static int
-strtonum(char *str, int *pnum)
+strtonum(const char *str, int *pnum)
 {
 	char		*endstr;		/* end of parsed string */
 
@@ -674,6 +908,7 @@ strtonum(char *str, int *pnum)
 	return(0);
 }
 
+
 /*
  *	mutexchk
  *
@@ -694,6 +929,7 @@ mutexchk(struct ao_opt *opt)
 	return(0);
 }
 
+
 /*
  *	countparam
  *
@@ -736,6 +972,7 @@ countparam(OPT *aod, struct ao_opt *opt, char **argv)
 	return(n);
 }
 
+
 /*
  *	addinst
  *
