@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: do_args.c,v 6.2.2.1 2002/10/09 19:48:49 brbarret Exp $
+ * $Id: do_args.c,v 6.4 2003/02/12 19:09:33 jsquyres Exp $
  * 
  *	Function:	- do_args(3) backward compatibility
  *			- uses all_opt package
@@ -31,18 +29,16 @@
 /*
  * global variables
  */
-int			argtailc = 0;
-char			**argtailv = 0;
+int lam_argtailc = 0;
+char **lam_argtailv = 0;
+OPT *lam_args_optd = 0;
 
-/*
- * local variables
- */
-static OPT		*optd = 0;		/* option descriptor */
 
 /*
  * local functions
  */
-static int		init();
+static int init(void);
+
 
 /*
  *	validopts
@@ -51,10 +47,7 @@ static int		init();
  *	Accepts:	- string of options
  */
 void
-validopts(str)
-
-CONST char		*str;
-
+validopts(CONST char *str)
 {
 	char		opt[2];
 
@@ -65,10 +58,12 @@ CONST char		*str;
  */
 	for (opt[1] = '\0'; *str; ++str) {
 		*opt = (*str == '0') ? '#' : *str;
-		if (ao_setopt(optd, opt, (char *) 0, 0, AODOARGS)) return;
+		if (ao_setopt(lam_args_optd, opt, (char *) 0, 0, AODOARGS)) 
+		  return;
 	}
 }
 
+
 /*
  *	exclusive
  *
@@ -76,10 +71,7 @@ CONST char		*str;
  *	Accepts:	- string of options
  */
 void
-exclusive(str)
-
-CONST char		*str;
-
+exclusive(CONST char *str)
 {
 	char		opt[2];
 	char		first[2];
@@ -90,16 +82,17 @@ CONST char		*str;
  * Declare first option.
  */
 	first[0] = *str++; first[1] = '\0';
-	if (ao_setopt(optd, first, (char *) 0, 0, AODOARGS)) return;
+	if (ao_setopt(lam_args_optd, first, (char *) 0, 0, AODOARGS)) return;
 /*
  * Declare the rest.
  */
 	for (opt[1] = '\0'; *str; ++str) {
 		*opt = *str;
-		if (ao_setopt(optd, opt, first, 0, AODOARGS)) return;
+		if (ao_setopt(lam_args_optd, opt, first, 0, AODOARGS)) return;
 	}
 }
 
+
 /*
  *	followed
  *
@@ -107,10 +100,7 @@ CONST char		*str;
  *	Accepts:	- string of options
  */
 void
-followed(str)
-
-CONST char		*str;
-
+followed(CONST char *str)
 {
 	char		opt[2];
 
@@ -121,10 +111,12 @@ CONST char		*str;
  */
 	for (opt[1] = '\0'; *str; ++str) {
 		*opt = *str;
-		if (ao_setopt(optd, opt, (char *) 0, 1, AODOARGS)) return;
+		if (ao_setopt(lam_args_optd, opt, (char *) 0, 1, AODOARGS)) 
+		  return;
 	}
 }
 
+
 /*
  *	twoletter
  *
@@ -132,10 +124,7 @@ CONST char		*str;
  *	Accepts:	- string of options
  */
 void
-twoletter(str)
-
-CONST char		*str;
-
+twoletter(CONST char *str)
 {
 	char		opt[3];
 
@@ -150,10 +139,12 @@ CONST char		*str;
 
 	for ( ; *str; ++str) {
 		opt[1] = *str;
-		if (ao_setopt(optd, opt, (char *) 0, 0, AODOARGS)) return;
+		if (ao_setopt(lam_args_optd, opt, (char *) 0, 0, AODOARGS)) 
+		  return;
 	}
 }
 
+
 /*
  *	opt_taken
  *
@@ -162,19 +153,17 @@ CONST char		*str;
  *	Returns:	- TRUE or FALSE
  */
 int
-opt_taken(let)
-
-int			let;
-
+opt_taken(int let)
 {
 	char		opt[2];
 
 	opt[0] = (let == '0') ? '#' : let & 0xFF;
 	opt[1] = '\0';
 
-	return(ao_taken(optd, opt));
+	return(ao_taken(lam_args_optd, opt));
 }
 
+
 /*
  *	opt_chosen
  *
@@ -183,21 +172,19 @@ int			let;
  *	Returns:	- chosen option
  */
 int
-opt_chosen(let)
-
-int			let;
-
+opt_chosen(int let)
 {
 	char		opt[2];
 	char		*chosen;
 
 	opt[0] = let & 0xFF; opt[1] = '\0';
 
-	chosen = ao_chosen(optd, opt);
+	chosen = ao_chosen(lam_args_optd, opt);
 
 	return((chosen) ? *chosen : LAMERROR);
 }
 
+
 /*
  *	do_args
  *
@@ -207,22 +194,19 @@ int			let;
  *	Returns:	- 0 or LAMERROR
  */
 int
-do_args(argc, argv)
-
-int			*argc;
-char			**argv;
-
+do_args(int *argc, char **argv)
 {
-	if (optd == 0) return(LAMERROR);
+	if (lam_args_optd == 0) return(LAMERROR);
 
-	if (ao_parse(optd, argc, argv)) return(LAMERROR);
+	if (ao_parse(lam_args_optd, argc, argv)) return(LAMERROR);
 
-	argtailc = optd->ao_tailc;
-	argtailv = optd->ao_tailv;
+	lam_argtailc = lam_args_optd->ao_tailc;
+	lam_argtailv = lam_args_optd->ao_tailv;
 
 	return(0);
 }
 
+
 /*
  *	getparam
  *
@@ -231,19 +215,17 @@ char			**argv;
  *	Returns:	- parameter or NULL
  */
 char *
-getparam(let)
-
-int			let;
-
+getparam(int let)
 {
 	char		opt[2];
 
 	opt[0] = (let == '0') ? '#' : let & 0xFF;
 	opt[1] = '\0';
 
-	return(ao_param(optd, opt, 0, 0));
+	return(ao_param(lam_args_optd, opt, 0, 0));
 }
 
+
 /*
  *	intparam
  *
@@ -253,11 +235,7 @@ int			let;
  *	Returns:	- 0 or LAMERROR
  */
 int
-intparam(let, pint)
-
-int			let;
-int			*pint;
-
+intparam(int let, int *pint)
 {
 	char		*param;
 
@@ -267,6 +245,7 @@ int			*pint;
 	return(xtoi(&param, pint));
 }
 
+
 /*
  *	get_2nd_let
  *
@@ -278,15 +257,12 @@ int			*pint;
  *	Returns:	- 2nd letter or LAMERROR
  */
 int
-get_2nd_let(let)
-
-int			let;
-
+get_2nd_let(int let)
 {
 	struct ao_opt	*p;
 	
 
-	for (p = optd->ao_opttop; p; p = p->aoo_next) {
+	for (p = lam_args_optd->ao_opttop; p; p = p->aoo_next) {
 
 		if ((p->aoo_string[0] == let) && (p->aoo_flags & AOTAKEN)) {
 			p->aoo_flags &= ~AOTAKEN;
@@ -297,24 +273,25 @@ int			let;
 	return(LAMERROR);
 }
 
+
 /*
  *	reset_args
  *
  *	Function:	- free the arguments
  */
 void
-reset_args()
-
+reset_args(void)
 {
-	if (optd) {
-		ao_free(optd);
-		optd = 0;
+	if (lam_args_optd) {
+		ao_free(lam_args_optd);
+		lam_args_optd = 0;
 	}
 
-	argtailc = 0;
-	argtailv = 0;
+	lam_argtailc = 0;
+	lam_argtailv = 0;
 }
 
+
 /*
  *	xtoi
  *
@@ -325,11 +302,7 @@ reset_args()
  *	Returns:	- 0 or LAMERROR
  */
 int
-xtoi(pstr, pint)
-
-char			**pstr;
-int			*pint;
-
+xtoi(char **pstr, int *pint)
 {
 	char		*str;
 
@@ -348,6 +321,7 @@ int			*pint;
 	return(0);
 }
 
+
 /*
  *	init
  *
@@ -355,12 +329,11 @@ int			*pint;
  *	Returns:	- 0 or LAMERROR
  */
 static int
-init()
-
+init(void)
 {
-	if (optd == 0) {
-		optd = ao_init();
+	if (lam_args_optd == 0) {
+		lam_args_optd = ao_init();
 	}
 
-	return((optd) ? 0 : LAMERROR);
+	return((lam_args_optd) ? 0 : LAMERROR);
 }
