@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: ndi_parse.c,v 6.8.2.2 2003/01/20 03:40:04 brbarret Exp $
+ * $Id: ndi_parse.c,v 6.10 2003/03/19 19:15:35 jsquyres Exp $
  *
  *	Function:	- node identifier parsing package
  *			- id_range() based in part on Trollius 2.0,
@@ -22,29 +20,23 @@
  *			  Research Foundation
  */
 
+#include <lam_config.h>
+
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
 
-#include "all_list.h"
-#include "args.h"
-#include "ndi.h"
-#include "rreq.h"
-#include "lam.h"
+#include <all_list.h>
+#include <args.h>
+#include <ndi.h>
+#include <rreq.h>
+#include <lam.h>
 
 #define NEW		1
 #define ONE		2
 #define MANY		3
 
-/*
- * global functions
- */
-LIST *			ndi_parse();
-int4			ndi_parse1();
-void			ndi_fmt();
-void			ndi_usage();
 
 /*
  * local functions
@@ -52,6 +44,7 @@ void			ndi_usage();
 static int id_range(char *s, int4 flags, int *ixp, int is_node, LIST *lp);
 static int resolve_cpu(int cpuid);
 
+
 /*
  *	ndi_parse
  *
@@ -96,7 +89,6 @@ ndi_parse(int argc, char **argv, char ***othersv)
  * Is it a generic node identifier?
  */
 		if ((**argv == 'n') && isdigit((int) *((*argv + 1)))) {
-
 			if (id_range(*argv + 1, 0, &index, 1, nodelist)) {
 				al_free(nodelist);
 				if (othersv) argvfree(*othersv);
@@ -107,7 +99,6 @@ ndi_parse(int argc, char **argv, char ***othersv)
  * Is it a generic CPU identifier?
  */
 		else if ((**argv == 'c') && isdigit((int) *((*argv + 1)))) {
-
 			if (id_range(*argv + 1, 0, &index, 0, nodelist)) {
 				al_free(nodelist);
 				if (othersv) argvfree(*othersv);
@@ -132,32 +123,13 @@ ndi_parse(int argc, char **argv, char ***othersv)
 /*
  * Is it a multicast destination?
  */
-/*
- * The "O" notation is not in any of the current documentation, and is
- * based on the old OTB stuff, so we're not going to support it
- * anymore.  JMS 6 March 2000.
- */
-#if 0
-		else if ((**argv == 'O') && (*(*argv + 1) == '\0')) {
-			newndi.ndi_node = HOST2OTB;
-			newndi.ndi_flags = NT_CAST;
-			newndi.ndi_count = -1;
-			newndi.ndi_index = index++;
-
-			if (al_append(nodelist, &newndi) == 0) {
-				al_free(nodelist);
-				if (othersv) argvfree(*othersv);
-				return(0);
-			}
-		}
-#endif
 /*
  * The "C" notation has been redfined from its original OTB definition
  * -- it now means "one per CPU".
  */
 		else if ((**argv == 'C') && (*(*argv + 1) == '\0')) {
-			newndi.ndi_node = HOST2COMP;
-			newndi.ndi_flags = NT_CAST;
+			newndi.ndi_node = HOST2COMPCPU;
+			newndi.ndi_flags = NT_CAST | NT_WASTE;
 			newndi.ndi_count = -1;
 			newndi.ndi_index = index++;
 
@@ -169,8 +141,8 @@ ndi_parse(int argc, char **argv, char ***othersv)
 		}
 
 		else if ((**argv == 'N') && (*(*argv + 1) == '\0')) {
-			newndi.ndi_node = HOST2ALL;
-			newndi.ndi_flags = NT_CAST;
+		        newndi.ndi_node = HOST2COMPNODE;
+			newndi.ndi_flags = NT_CAST | NT_WASTE;
 			newndi.ndi_count = -1;
 			newndi.ndi_index = index++;
 
@@ -199,6 +171,7 @@ ndi_parse(int argc, char **argv, char ***othersv)
 	return(nodelist);
 }
 
+
 /*
  *	ndi_fmt
  *
@@ -213,28 +186,20 @@ ndi_fmt(int4 node, char *fmt_str)
 		strcpy(fmt_str, "?");
 	} else if (node == LOCAL) {
 		strcpy(fmt_str, "local");
-/*
- * The "O" notation is not in any of the current documentation, and is
- * based on the old OTB stuff, so we're not going to support it
- * anymore.  JMS 6 March 2000.
- */
-#if 0
-	} else if (node == HOST2OTB) {
-		strcpy(fmt_str, "O");
-#endif
-	} else if (node == HOST2ALL) {
+	} else if (node == HOST2COMPNODE) {
 		strcpy(fmt_str, "N");
 /*
  * The "C" notation has been redfined from its original OTB definition
  * -- it now means "one per CPU".
  */
-	} else if (node == HOST2COMP) {
+	} else if (node == HOST2COMPCPU) {
 		strcpy(fmt_str, "C");
 	} else {
 		sprintf(fmt_str, "n%d", node);
 	}
 }
 
+
 /*
  *	ndi_parse1
  *
@@ -251,23 +216,14 @@ ndi_parse1(char *s)
 		node = LOCAL;
 	} else if (strcmp(s, "local") == 0) {
 		node = LOCAL;
-/*
- * The "O" notation is not in any of the current documentation, and is
- * based on the old OTB stuff, so we're not going to support it
- * anymore.  JMS 6 March 2000.
- */
-#if 0
-	} else if (strcmp(s, "O") == 0) {
-		node = HOST2OTB;
-#endif
 	} else if (strcmp(s, "N") == 0) {
-		node = HOST2ALL;
+		node = HOST2COMPNODE;
 /*
  * The "C" notation has been redfined from its original OTB definition
  * -- it now means "one per CPU".
  */
 	} else if (strcmp(s, "C") == 0) {
-		node = HOST2COMP;
+		node = HOST2COMPCPU;
 	}
 
 	else if (sscanf(s, " n%d", &node) == 1) {
@@ -283,18 +239,20 @@ ndi_parse1(char *s)
 	return(node);
 }
 
+
 /*
  *	ndi_usage
  *
  *	Function:	- prints a nodespec usage message
  */
 void
-ndi_usage()
+ndi_usage(void)
 {
 	printf("\nNodes:		n<list>, eg., n0-3,5,0xa,12-15\n");
 	printf("\nCPUs:			c<list>, eg., c0-3,5,0xa,12-15\n");
 }
 
+
 /*
  *	id_range
  *
