@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,35 +10,10 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1994 The Ohio State University
- *	RBD
+ * $HEADER$
+ * 
+ * $Id: f2cstring.c,v 6.4 2003/05/31 22:28:49 jsquyres Exp $
  *
- *	$Log: f2cstring.c,v $
- *	Revision 6.2.2.1  2002/10/09 19:48:51  brbarret
- *	* Updated license header to match current requirements
- *	
- *	Revision 6.2  1999/05/24 16:56:48  kmeyer1
- *	added $copyright$
- *	
- *	Revision 6.1  1996/11/23 19:57:55  nevin
- *	Ohio Release
- *	
- * Revision 6.0  96/02/29  13:46:30  gdburns
- * Ohio Release
- * 
- * Revision 5.2  94/08/22  13:57:00  gdburns
- * Ohio Release
- * 
- * Revision 5.1  94/05/18  10:56:43  gdburns
- * Ohio Release
- * 
- * Revision 2.3  94/04/22  12:34:58  gdburns
- * Ohio Release
- * 
- * Revision 2.2  94/02/15  18:06:16  raja
- * pre-release to 2.3
- * 
  *	Function:	- create a C string from a F77 string
  *	Accepts:	- F77 string
  *			- string length
@@ -48,6 +23,9 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <etc_misc.h>
+
+
 char *
 f2cstring(str, len)
 
@@ -65,7 +43,7 @@ unsigned int		len;
 	}
 
 	if (len > 0) {
-		strncpy(cstr, str, (int) len);
+		lam_strncpy(cstr, str, (int) len);
 	}
 /*
  * Bypass trailing spaces and null-terminate the string.
