@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: getinetaddr.c,v 1.1.2.1 2002/10/09 19:48:51 brbarret Exp $
+ * $Id: getinetaddr.c,v 1.3.2.1 2003/07/17 18:04:26 jsquyres Exp $
  * 
  *	Function:	- return the machine Internet address (OTB only)
  *			- address is a 4-byte array
@@ -30,18 +28,21 @@
 #include <terror.h>
 #include <typical.h>
 
+#include <sys/types.h>
+#include <netinet/in.h>
+
+#include <lam_config.h>
+#include <lam_network.h>
+
+
 int
-getinetaddr(hostname, hostaddr)
-
-char			*hostname;
-unsigned char		*hostaddr;
-
+getinetaddr(char *hostname, unsigned char *hostaddr)
 {
 	struct hostent	*p;
 /*
  * Get host name entry.
  */
-	p = gethostbyname(hostname);
+	p = lam_gethostbyname(hostname);
 
 	if (p == 0) {
 		errno = EBADHOST;
