@@ -10,11 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1997 The Ohio State University
  *	RBD
  *
- *	$Id: getworkdir.c,v 6.3.2.1 2002/10/09 19:48:51 brbarret Exp $
+ *	$Id: getworkdir.c,v 6.5 2002/10/21 04:48:07 brbarret Exp $
  * 
  *	Function:	- get the user's current working directory
  *			- dynamically allocated space
@@ -46,7 +47,7 @@ getworkdir()
 /*
  * Get the working directory.  Work around SunOS bug.
  */
-#if HAVE_GETCWD && !defined(SUNOS)
+#if LAM_HAVE_GETCWD && !defined(SUNOS)
 	if (getcwd(cwd, PATHSIZE)) {
 #else
 	if (getwd(cwd)) {
