@@ -10,11 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1997 The Ohio State University
  *	RBD
  *
- *	$Id: lamunix.c,v 6.5.2.1 2002/10/09 19:48:52 brbarret Exp $
+ *	$Id: lamunix.c,v 6.8 2002/10/21 04:48:07 brbarret Exp $
  * 
  *	Function:	- We make UNIX portable in here!
  *			- #ifdef'ing the universe is cool!
@@ -24,23 +25,13 @@
 #include <signal.h>
 #include <stdlib.h>
 
-/*
- * global functions
- */
-void			(*(_lam_signal()))();
-void			_lam_atexit();
-
 /*
  *	_lam_signal
  *
  *	Function:	- signal() wrapper
  */
 void (*(
-_lam_signal(signum, handler)))()
-
-int			signum;
-void			(*handler)();
-
+_lam_signal(int signum, void (*handler)())))()
 {
 	struct sigaction
 			newact, old;
@@ -66,12 +57,9 @@ void			(*handler)();
  *	Function:	- wrapper for atexit()
  */
 void
-_lam_atexit(func)
-
-void			(*func)();
-
+_lam_atexit(void (*func)())
 {
-#if HAVE_ATEXIT
+#if LAM_HAVE_ATEXIT
 	atexit(func);
 #else
 	on_exit((int ((*)())) func, (char *) 0);
