@@ -10,14 +10,16 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1993 The Ohio State University
  *	RBD
  *
  *	$Log: memucmp.c,v $
- *	Revision 6.2.2.1  2002/10/09 19:48:52  brbarret
- *	* Updated license header to match current requirements
- *	
+ *	Revision 6.3  2002/10/09 20:57:02  brbarret
+ *	* Update tree to match the latest, greatest, license for LAM/MPI.  And this
+ *	  one is even done properly :)
+ *
  *	Revision 6.2  1999/05/24 16:56:49  kmeyer1
  *	added $copyright$
  *	
