@@ -10,19 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1995 The Ohio State University
  *	RBD
  *
- *	$Log: nmsgconv.c,v $
- *	Revision 6.2.2.1  2002/10/09 19:48:52  brbarret
- *	* Updated license header to match current requirements
- *	
- *	Revision 6.2  1999/05/24 16:56:51  kmeyer1
- *	added $copyright$
- *	
- *	Revision 6.1  1996/11/23 19:58:11  nevin
- *	Ohio Release
+ *      $Id: nmsgconv.c,v 6.5 2002/10/09 20:57:03 brbarret Exp $
  *	
  * Revision 6.0  96/02/29  13:48:21  gdburns
  * Ohio Release
@@ -59,10 +52,7 @@
  *	Accepts:	- ptr to network message descriptor
  */
 void
-_ltot_sysnmsg(nhead)
-
-struct nmsg		*nhead;
-
+_ltot_sysnmsg(struct nmsg *nhead)
 {
 	ltoti4(&(nhead->nh_dl_event), &(nhead->nh_dl_event));
 	ltoti4(&(nhead->nh_dl_link), &(nhead->nh_dl_link));
@@ -81,10 +71,7 @@ struct nmsg		*nhead;
  *	Accepts:	- ptr to network message descriptor
  */
 void
-_ttol_sysnmsg(nhead)
-
-struct nmsg		*nhead;
-
+_ttol_sysnmsg(struct nmsg *nhead)
 {
 	ttoli4(&(nhead->nh_dl_event), &(nhead->nh_dl_event));
 	ttoli4(&(nhead->nh_dl_link), &(nhead->nh_dl_link));
@@ -103,10 +90,7 @@ struct nmsg		*nhead;
  *	Accepts:	- ptr to network message descriptor
  */
 void
-_ltot_usrnmsg(nhead)
-
-struct nmsg		*nhead;
-
+_ltot_usrnmsg(struct nmsg *nhead)
 {
 /*
  * Convert the data pouch.
@@ -161,10 +145,7 @@ struct nmsg		*nhead;
  *	Accepts:	- ptr to network message descriptor
  */
 void
-_ttol_usrnmsg(nhead)
-
-struct nmsg		*nhead;
-
+_ttol_usrnmsg(struct nmsg *nhead)
 {
 /*
  * Convert the data pouch.
