@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,22 +10,16 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *      Software for Humanity
- *      Public Domain
- *
- *      This program is freely distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- *	$Id: pty.c,v 6.7.2.1 2002/10/09 19:48:52 brbarret Exp $
+ * $Id: pty.c,v 6.12.2.1 2003/10/11 19:04:01 brbarret Exp $
  * 
  *	Function:	- pseudo terminal support
  *			- derived from Steven's "Advanced Programming
  *			  in the Unix Environment".
  */
 
-#include "lam_config.h"
+#include <lam_config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,12 +28,16 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#if HAVE_PTY_H
+#if defined(HAVE_PTY_H) && HAVE_PTY_H
 #include <pty.h>
 #endif
+#if defined(HAVE_UTIL_H) && HAVE_UTIL_H
+#include <util.h>
+#endif
 
-#include "typical.h"
-#include "terror.h"
+#include <typical.h>
+#include <terror.h>
+#include <etc_misc.h>
 
 
 /*
@@ -72,7 +70,7 @@ ptym_open(char *pts_name)
     umask(mode);
     return (-1);
   }
-  strncpy(pts_name, name, max_name_len);
+  lam_strncpy(pts_name, name, max_name_len);
 
   umask(mode);
   return fdm;
@@ -88,7 +86,7 @@ ptys_open(int fdm, char *pts_name)
   return static_fds;
 }
 
-#elif LAM_HAVE_SYSV_PTYS && HAVE_STROPTS_H
+#elif LAM_HAVE_SYSV_PTYS && defined(HAVE_STROPTS_H) && HAVE_STROPTS_H
 
 #include <stropts.h>
 
@@ -155,7 +153,7 @@ ptys_open(int fdm, char *pts_name)
     return(fds);
 }
 
-#elif LAM_HAVE_BSD_PTYS && HAVE_GRP_H
+#elif LAM_HAVE_BSD_PTYS && defined(HAVE_GRP_H) && HAVE_GRP_H
 
 #include <grp.h>
 
