@@ -10,19 +10,25 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	$Id: show_version.c,v 1.3.2.2 2002/10/09 19:48:52 brbarret Exp $
+ *	$Id: show_version.c,v 1.9.2.1 2003/11/01 23:22:24 brbarret Exp $
  * 
  *	Function:	- show the version of LAM/MPI
  *
  */
 
 #include <lam_config.h>
+#include <lam_build_info.h>
 
 #include <stdio.h>
 
 #include <etc_misc.h>
 
+
+/*
+ * Show a header indicating the version of LAM
+ */
 void
 lam_show_version(int verbose)
 {
@@ -35,10 +41,17 @@ lam_show_version(int verbose)
 #endif
 #if LAM_WANT_ROMIO
   printf("/ROMIO");
+#endif
+#if LAM_HAVE_BPROC
+  printf("/bproc");
 #endif
   printf(" - Indiana University\n\n");
+
   if (verbose == 1) {
     printf("\tArch:\t\t%s\n", LAM_ARCH);
-    printf("\tRPI:\t\t%s\n", LAM_RPI);
-  }
+    printf("\tPrefix:\t\t%s\n", LAM_PREFIX);
+    printf("\tConfigured by:\t%s\n", LAM_CONFIGURE_USER);
+    printf("\tConfigured on:\t%s\n", LAM_CONFIGURE_DATE);
+    printf("\tConfigure host:\t%s\n", LAM_CONFIGURE_HOST);
+   }
 }
