@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1998 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: lfopenfd.c,v 6.3.2.1 2002/10/09 19:48:55 brbarret Exp $
+ * $Id: lfopenfd.c,v 6.6 2003/04/10 17:44:26 jsquyres Exp $
  *
  *	Function:	- pass stdio file descriptors to local filed
  *	Accepts:	- name to use for connection
@@ -35,22 +33,18 @@
 #include <net.h>
 #include <portable.h>
 #include <typical.h>
+#include <etc_misc.h>
+#include <laminternal.h>
 
-/*
- * external functions
- */
-extern int		mread();
 
 /*
  * external variables
  */
 extern struct fclient	_ufd[FUMAX];		/* client file desc. table */
 
+
 int
-lam_lfopenfd(server)
-
-char			*server;
-
+lam_lfopenfd(char *server)
 {
 	struct nmsg	nhead;			/* network message desc. */
 	struct freq	*request;		/* filed request */
@@ -71,11 +65,12 @@ char			*server;
 /*
  * Send request to filed.
  */
+        LAM_ZERO_ME(nhead);
 	request = (struct freq *) nhead.nh_data;
 	reply = (struct freply *) nhead.nh_data;
 
 	request->fq_src_node = getnodeid();
-	request->fq_src_event = -getpid();
+	request->fq_src_event = -lam_getpid();
 	request->fq_req = FQOPENFD;
 
 	nhead.nh_node = LOCAL;
@@ -132,7 +127,7 @@ char			*server;
 /*
  * Receive response from filed.
  */
-	nhead.nh_event = -getpid();
+	nhead.nh_event = -lam_getpid();
 	nhead.nh_length = 0;
 	nhead.nh_msg = 0;
 
