@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,8 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
+ *	Ohio Trollius
+ *	Copyright 1994 The Ohio State University
+ *	GDB
  *
- *	$Id: rf77.c,v 6.3.2.2 2003/01/17 20:39:31 brbarret Exp $
+ *	$Id: rf77.c,v 6.6 2002/10/09 20:57:13 brbarret Exp $
  *
  *	Function:	- functions to support F77 I/O
  */
@@ -24,6 +28,7 @@
 #include <ksignal.h>
 #include <net.h>
 #include <typical.h>
+#include <etc_misc.h>
 
 /*
  * external variables
@@ -66,7 +71,7 @@ char			fmt;
 	reply = (struct freply *) nhead.nh_data;
 
 	request->fq_src_node = getnodeid();
-	request->fq_src_event = -getpid();
+	request->fq_src_event = -lam_getpid();
 	request->fq_req = FQF77BACK;
 	request->fq_tfd = _ufd[fd].fu_tfd;
 	request->fq_fmt = fmt;
@@ -87,7 +92,7 @@ char			fmt;
 	}
 
 	mask = ksigblock(sigmask(SIGUDIE) | sigmask(SIGARREST));
-	nhead.nh_event = -getpid();
+	nhead.nh_event = -lam_getpid();
 
 	if (nrecv(&nhead)) {
 		return((int4) ERROR);
@@ -139,7 +144,7 @@ char			fmt;
 	reply = (struct freply *) nhead.nh_data;
 
 	request->fq_src_node = getnodeid();
-	request->fq_src_event = -getpid();
+	request->fq_src_event = -lam_getpid();
 	request->fq_req = FQF77READ;
 	request->fq_tfd = _ufd[fd].fu_tfd;
 	request->fq_length = len;
@@ -166,7 +171,7 @@ char			fmt;
  * Each packet contains the length of the total input data.
  */
 	nhead.nh_msg = buf;
-	nhead.nh_event = -getpid();
+	nhead.nh_event = -lam_getpid();
 
 	do {
 		nhead.nh_type = 0;
