@@ -10,8 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	$Id: MPISYSF.h,v 6.7.2.3 2002/10/09 19:48:58 brbarret Exp $
+ *	$Id: MPISYSF.h,v 6.12.2.1 2003/11/06 18:22:22 jsquyres Exp $
  *
  *	Function:	- F77 wrappers for MPI library
  */
@@ -25,8 +26,8 @@
 /*
  * macros
  */
-#define GETHDL(x)		(((x) >= 0) ? lam_F_handles[(x)] : 0)
-#define BUF(x)			(((x) == lam_F_bottom) ? (char *) 0 : (x))
+#define GETHDL(x)		(((x) >= 0) ? lam_F_handles[(x)] : NULL)
+#define BUF(x)			(((x) == lam_F_bottom) ? NULL : (x))
 
 
 #if LAM_FORTRANNOUNDERSCORE
@@ -243,6 +244,7 @@
 #define mpi_win_set_errhandler_		pmpi_win_set_errhandler
 #define mpi_win_set_name_		pmpi_win_set_name
 #define mpi_win_start_			pmpi_win_start
+#define mpi_win_test_			pmpi_win_test
 #define mpi_win_wait_			pmpi_win_wait
 #define mpi_wtick_			pmpi_wtick
 #define mpi_wtime_			pmpi_wtime
@@ -445,6 +447,7 @@
 #define mpi_waitall_			mpi_waitall
 #define mpi_waitany_			mpi_waitany
 #define mpi_waitsome_			mpi_waitsome
+#define mpi_win_call_errhandler_	mpi_win_call_errhandler
 #define mpi_win_complete_		mpi_win_complete
 #define mpi_win_create_			mpi_win_create
 #define mpi_win_create_errhandler_	mpi_win_create_errhandler
@@ -457,11 +460,14 @@
 #define mpi_win_get_errhandler_		mpi_win_get_errhandler
 #define mpi_win_get_group_		mpi_win_get_group
 #define mpi_win_get_name_		mpi_win_get_name
+#define mpi_win_lock_			mpi_win_lock
 #define mpi_win_post_			mpi_win_post
 #define mpi_win_set_attr_		mpi_win_set_attr
 #define mpi_win_set_errhandler_		mpi_win_set_errhandler
 #define mpi_win_set_name_		mpi_win_set_name
 #define mpi_win_start_			mpi_win_start
+#define mpi_win_test_			mpi_win_test
+#define mpi_win_unlock_			mpi_win_unlock
 #define mpi_win_wait_			mpi_win_wait
 #define mpi_wtick_			mpi_wtick
 #define mpi_wtime_			mpi_wtime
@@ -696,6 +702,7 @@
 #define mpi_waitall_			pmpi_waitall__
 #define mpi_waitany_			pmpi_waitany__
 #define mpi_waitsome_			pmpi_waitsome__
+#define mpi_win_call_errhandler_	pmpi_win_call_errhandler__
 #define mpi_win_complete_		pmpi_win_complete__
 #define mpi_win_create_			pmpi_win_create__
 #define mpi_win_create_errhandler_	pmpi_win_create_errhandler__
@@ -708,11 +715,14 @@
 #define mpi_win_get_errhandler_		pmpi_win_get_errhandler__
 #define mpi_win_get_group_		pmpi_win_get_group__
 #define mpi_win_get_name_		pmpi_win_get_name__
+#define mpi_win_lock_			pmpi_win_lock__
 #define mpi_win_post_			pmpi_win_post__
 #define mpi_win_set_attr_		pmpi_win_set_attr__
 #define mpi_win_set_errhandler_		pmpi_win_set_errhandler__
 #define mpi_win_set_name_		pmpi_win_set_name__
 #define mpi_win_start_			pmpi_win_start__
+#define mpi_win_test_			pmpi_win_test__
+#define mpi_win_unlock_			pmpi_win_unlock__
 #define mpi_win_wait_			pmpi_win_wait__
 #define mpi_wtick_			pmpi_wtick__
 #define mpi_wtime_			pmpi_wtime__
@@ -915,6 +925,7 @@
 #define mpi_waitall_			mpi_waitall__
 #define mpi_waitany_			mpi_waitany__
 #define mpi_waitsome_			mpi_waitsome__
+#define mpi_win_call_errhandler_	mpi_win_call_errhandler__
 #define mpi_win_complete_		mpi_win_complete__
 #define mpi_win_create_			mpi_win_create__
 #define mpi_win_create_errhandler_	mpi_win_create_errhandler__
@@ -927,11 +938,14 @@
 #define mpi_win_get_errhandler_		mpi_win_get_errhandler__
 #define mpi_win_get_group_		mpi_win_get_group__
 #define mpi_win_get_name_		mpi_win_get_name__
+#define mpi_win_lock_			mpi_win_lock__
 #define mpi_win_post_			mpi_win_post__
 #define mpi_win_set_attr_		mpi_win_set_attr__
 #define mpi_win_set_errhandler_		mpi_win_set_errhandler__
 #define mpi_win_set_name_		mpi_win_set_name__
 #define mpi_win_start_			mpi_win_start__
+#define mpi_win_test_			mpi_win_test__
+#define mpi_win_unlock_			mpi_win_unlock__
 #define mpi_win_wait_			mpi_win_wait__
 #define mpi_wtick_			mpi_wtick__
 #define mpi_wtime_			mpi_wtime__
@@ -1166,6 +1180,7 @@
 #define mpi_waitall_			PMPI_WAITALL
 #define mpi_waitany_			PMPI_WAITANY
 #define mpi_waitsome_			PMPI_WAITSOME
+#define mpi_win_call_errhandler_	PMPI_MPI_CALL_ERRHANDLER
 #define mpi_win_complete_		PMPI_WIN_COMPLETE
 #define mpi_win_create_			PMPI_WIN_CREATE
 #define mpi_win_create_errhandler_	PMPI_WIN_CREATE_ERRHANDLER
@@ -1178,12 +1193,15 @@
 #define mpi_win_get_errhandler_		PMPI_WIN_GET_ERRHANDLER
 #define mpi_win_get_group_		PMPI_WIN_GET_GROUP
 #define mpi_win_get_name_		PMPI_WIN_GET_NAME
+#define mpi_win_lock_			PMPI_WIN_LOCK
 #define mpi_win_post_			PMPI_WIN_POST
 #define mpi_win_set_attr_		PMPI_WIN_SET_ATTR
 #define mpi_win_set_errhandler_		PMPI_WIN_SET_ERRHANDLER
 #define mpi_win_set_name_		PMPI_WIN_SET_NAME
 #define mpi_win_start_			PMPI_WIN_START
+#define mpi_win_test_			PMPI_WIN_TEST
 #define mpi_win_wait_			PMPI_WIN_WAIT
+#define mpi_win_unlock_			PMPI_WIN_UNLOCK
 #define mpi_wtick_			PMPI_WTICK
 #define mpi_wtime_			PMPI_WTIME
 
@@ -1385,6 +1403,7 @@
 #define mpi_waitall_			MPI_WAITALL
 #define mpi_waitany_			MPI_WAITANY
 #define mpi_waitsome_			MPI_WAITSOME
+#define mpi_win_call_errhandler_	MPI_WIN_CALL_ERRHANDLER
 #define mpi_win_complete_		MPI_WIN_COMPLETE
 #define mpi_win_create_			MPI_WIN_CREATE
 #define mpi_win_create_errhandler_	MPI_WIN_CREATE_ERRHANDLER
@@ -1397,11 +1416,14 @@
 #define mpi_win_get_errhandler_		MPI_WIN_GET_ERRHANDLER
 #define mpi_win_get_group_		MPI_WIN_GET_GROUP
 #define mpi_win_get_name_		MPI_WIN_GET_NAME
+#define mpi_win_lock_			MPI_WIN_LOCK
 #define mpi_win_post_			MPI_WIN_POST
 #define mpi_win_set_attr_		MPI_WIN_SET_ATTR
 #define mpi_win_set_errhandler_		MPI_WIN_SET_ERRHANDLER
 #define mpi_win_set_name_		MPI_WIN_SET_NAME
 #define mpi_win_start_			MPI_WIN_START
+#define mpi_win_test_			MPI_WIN_TEST
+#define mpi_win_unlock_			MPI_WIN_UNLOCK
 #define mpi_win_wait_			MPI_WIN_WAIT
 #define mpi_wtick_			MPI_WTICK
 #define mpi_wtime_			MPI_WTIME
@@ -1668,6 +1690,7 @@
 #define mpi_waitall_			pmpi_waitall_
 #define mpi_waitany_			pmpi_waitany_
 #define mpi_waitsome_			pmpi_waitsome_
+#define mpi_win_call_errhandler_	pmpi_win_call_errhandler_
 #define mpi_win_complete_		pmpi_win_complete_
 #define mpi_win_create_			pmpi_win_create_
 #define mpi_win_create_errhandler_	pmpi_win_create_errhandler_
@@ -1681,10 +1704,13 @@
 #define mpi_win_get_group_		pmpi_win_get_group_
 #define mpi_win_get_name_		pmpi_win_get_name_
 #define mpi_win_post_			pmpi_win_post_
+#define mpi_win_lock_			pmpi_win_lock_
 #define mpi_win_set_attr_		pmpi_win_set_attr_
 #define mpi_win_set_errhandler_		pmpi_win_set_errhandler_
 #define mpi_win_set_name_		pmpi_win_set_name_
 #define mpi_win_start_			pmpi_win_start_
+#define mpi_win_test_			pmpi_win_test_
+#define mpi_win_unlock_			pmpi_win_unlock_
 #define mpi_win_wait_			pmpi_win_wait_
 #define mpi_wtick_			pmpi_wtick_
 #define mpi_wtime_			pmpi_wtime_
@@ -1887,6 +1913,7 @@
 #define mpi_waitall_			mpi_waitall_
 #define mpi_waitany_			mpi_waitany_
 #define mpi_waitsome_			mpi_waitsome_
+#define mpi_win_call_errhandler_	mpi_win_call_errhandler_
 #define mpi_win_complete_		mpi_win_complete_
 #define mpi_win_create_			mpi_win_create_
 #define mpi_win_create_errhandler_	mpi_win_create_errhandler_
@@ -1900,10 +1927,13 @@
 #define mpi_win_get_group_		mpi_win_get_group_
 #define mpi_win_get_name_		mpi_win_get_name_
 #define mpi_win_post_			mpi_win_post_
+#define mpi_win_lock_			mpi_win_lock_
 #define mpi_win_set_attr_		mpi_win_set_attr_
 #define mpi_win_set_errhandler_		mpi_win_set_errhandler_
 #define mpi_win_set_name_		mpi_win_set_name_
 #define mpi_win_start_			mpi_win_start_
+#define mpi_win_test_			mpi_win_test_
+#define mpi_win_unlock_			mpi_win_unlock_
 #define mpi_win_wait_			mpi_win_wait_
 #define mpi_wtick_			mpi_wtick_
 #define mpi_wtime_			mpi_wtime_
