@@ -1,35 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
- * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	$Id: mpi.h,v 6.27.2.1 2001/11/09 20:57:50 brbarret Exp $
+ *	$Id: mpi.h,v 6.37.2.1 2003/08/21 15:33:43 brbarret Exp $
  *
  *	Function:	- MPI header file
  */
@@ -118,8 +101,6 @@ typedef int	(MPI_Win_delete_attr_function)();
 #define MPI_ANY_TAG		-1		/* match any message tag */
 #define MPI_GER_TAG		-2		/* used for GER protocol */
 #define MPI_MERGE_TAG		-3		/* used to merge inter-comm */
-#define MPI_MAX_CID		0xFFE		/* max pt2pt context ID */
-#define MPI_GER_CID		0xFFF		/* used for GER protocol */
 #define MPI_MAX_PROCESSOR_NAME	256		/* max proc. name length */
 #define MPI_MAX_ERROR_STRING	256		/* max error message length */
 #define MPI_MAX_OBJECT_NAME	64		/* max object name length */
@@ -128,6 +109,9 @@ typedef int	(MPI_Win_delete_attr_function)();
 #define MPI_GRAPH		2		/* graph topology */
 #define MPI_KEYVAL_INVALID	-1		/* invalid key value */
 
+/*
+ * Predefined attribute keyvals
+ */
 #define MPI_TAG_UB		0		/* predefined attribute key */
 #define MPI_HOST		1		/* predefined attribute key */
 #define MPI_IO			2		/* predefined attribute key */
@@ -137,26 +121,38 @@ typedef int	(MPI_Win_delete_attr_function)();
 #define MPI_WIN_BASE		6		/* predefined attribute key */
 #define MPI_WIN_SIZE		7		/* predefined attribute key */
 #define MPI_WIN_DISP_UNIT	8		/* predefined attribute key */
-/* Even though these four are IMPI attributes, they need to be there
-   for all MPI jobs */
+
+/* 
+ * Even though these four are IMPI attributes, they need to be there
+ * for all MPI jobs
+ */
 #define IMPI_CLIENT_SIZE        9               /* predefined attribute key */
 #define IMPI_CLIENT_COLOR       10              /* predefined attribute key */
 #define IMPI_HOST_SIZE          11              /* predefined attribute key */
 #define IMPI_HOST_COLOR         12              /* predefined attribute key */
 
 /*
- * Predefined attributes, but LAM-specific
+ * Predefined attribute keyvals, but LAM-specific
  */
 #define LAM_UNIVERSE_NCPUS	13		/* predefined attribute key */
 #define LAM_UNIVERSE_NNODES	14		/* predefined attribute key */
 
+/*
+ * Predefined attribute keyvals, but LAM/MPI-specific
+ */
+#define LAM_MPI_SSI_COLL                    15
+#define LAM_MPI_SSI_COLL_CROSSOVER          16
+#define LAM_MPI_SSI_COLL_ASSOCIATIVE        17
+#define LAM_MPI_SSI_COLL_REDUCE_CROSSOVER   18
+
+
 #define MPI_BOTTOM		((void *) 0)	/* base reference address */
 #define MPI_BSEND_OVERHEAD	40		/* size of bsend header + ptr */
 #define MPI_MAX_INFO_KEY	36		/* max info key length */
 #define MPI_MAX_INFO_VAL	256		/* max info value length */
 #define MPI_ARGV_NULL		((char **) 0)	/* NULL argument vector */
 #define MPI_ARGVS_NULL		((char ***) 0)	/* NULL argument vectors */
-#define MPI_ERRCODES_IGNORE	((void *) 0)	/* don't return error codes */
+#define MPI_ERRCODES_IGNORE	((int *) 0)	/* don't return error codes */
 #define MPI_MAX_PORT_NAME       36              /* max port name length */
 #define MPI_MAX_NAME_LEN	MPI_MAX_PORT_NAME /* max port name length */
 #define MPI_ORDER_C		0		/* C row major order */
@@ -338,6 +334,9 @@ extern struct _dtype		lam_mpi_2real, lam_mpi_2dblprec;
 extern struct _dtype		lam_mpi_2integer, lam_mpi_longdbl_int;
 extern struct _dtype		lam_mpi_wchar, lam_mpi_long_long_int;
 extern struct _dtype		lam_mpi_unsigned_long_long;
+extern struct _dtype            lam_mpi_cxx_cplex, lam_mpi_cxx_dblcplex;
+extern struct _dtype            lam_mpi_cxx_ldblcplex;
+extern struct _dtype            lam_mpi_cxx_bool;
 
 extern struct _errhdl		lam_mpi_errors_are_fatal;
 extern struct _errhdl		lam_mpi_errors_return;
@@ -678,6 +677,7 @@ extern int MPI_Wait __ARGS((MPI_Request *, MPI_Status *));
 extern int MPI_Waitsome __ARGS((int, MPI_Request *, int *, int *, 
 				MPI_Status *));
 extern MPI_Fint MPI_Win_c2f __ARGS((MPI_Win));
+extern int MPI_Win_call_errhandler __ARGS((MPI_Win, int));
 extern int MPI_Win_complete __ARGS((MPI_Win));
 extern int MPI_Win_create __ARGS((void *, MPI_Aint, int, MPI_Info, MPI_Comm, 
 				  MPI_Win *));
@@ -695,11 +695,14 @@ extern int MPI_Win_get_attr __ARGS((MPI_Win, int, void *, int *));
 extern int MPI_Win_get_errhandler __ARGS((MPI_Win, MPI_Errhandler *));
 extern int MPI_Win_get_group __ARGS((MPI_Win, MPI_Group *));
 extern int MPI_Win_get_name __ARGS((MPI_Win, char *, int *));
+extern int MPI_Win_lock __ARGS((int, int, int, MPI_Win));
 extern int MPI_Win_post __ARGS((MPI_Group, int, MPI_Win));
 extern int MPI_Win_set_attr __ARGS((MPI_Win, int, void *));
 extern int MPI_Win_set_errhandler __ARGS((MPI_Win, MPI_Errhandler));
 extern int MPI_Win_set_name __ARGS((MPI_Win, char *));
 extern int MPI_Win_start __ARGS((MPI_Group, int, MPI_Win));
+extern int MPI_Win_test __ARGS((MPI_Win, int *));
+extern int MPI_Win_unlock __ARGS((int, MPI_Win));
 extern int MPI_Win_wait __ARGS((MPI_Win));
 extern double MPI_Wtick __ARGS((void));
 extern double MPI_Wtime __ARGS((void));
@@ -707,14 +710,6 @@ extern double MPI_Wtime __ARGS((void));
 /*
  * profiling prototypes
  */
-extern int PMPI_DUP_FN __ARGS((MPI_Comm, int, void *,
-			       void *, void *, int *));
-extern int PMPI_COMM_DUP_FN __ARGS((MPI_Comm, int, void *,
-				    void *, void *, int *));
-extern int PMPI_TYPE_DUP_FN __ARGS((MPI_Datatype, int, void *,
-				    void *, void *, int *));
-extern int PMPI_WIN_DUP_FN __ARGS((MPI_Win, int, void *,
-				   void *, void *, int *));
 
 extern int PMPI_Abort __ARGS((MPI_Comm, int));
 extern int PMPI_Accumulate __ARGS((void *, int, MPI_Datatype, int, MPI_Aint, 
@@ -987,6 +982,7 @@ extern int PMPI_Wait __ARGS((MPI_Request *, MPI_Status *));
 extern int PMPI_Waitsome __ARGS((int, MPI_Request *, int *, int *, 
 				 MPI_Status *));
 extern MPI_Fint PMPI_Win_c2f __ARGS((MPI_Win));
+extern int PMPI_Win_call_errhandler __ARGS((MPI_Win, int));
 extern int PMPI_Win_complete __ARGS((MPI_Win));
 extern int PMPI_Win_create __ARGS((void *, MPI_Aint, int, MPI_Info, MPI_Comm, 
 				   MPI_Win *));
@@ -1004,11 +1000,14 @@ extern int PMPI_Win_get_attr __ARGS((MPI_Win, int, void *, int *));
 extern int PMPI_Win_get_errhandler __ARGS((MPI_Win, MPI_Errhandler *));
 extern int PMPI_Win_get_group __ARGS((MPI_Win, MPI_Group *));
 extern int PMPI_Win_get_name __ARGS((MPI_Win, char *, int *));
+extern int PMPI_Win_lock __ARGS((int, int, int, MPI_Win));
 extern int PMPI_Win_post __ARGS((MPI_Group, int, MPI_Win));
 extern int PMPI_Win_set_attr __ARGS((MPI_Win, int, void *));
 extern int PMPI_Win_set_errhandler __ARGS((MPI_Win, MPI_Errhandler));
 extern int PMPI_Win_set_name __ARGS((MPI_Win, char *));
 extern int PMPI_Win_start __ARGS((MPI_Group, int, MPI_Win));
+extern int PMPI_Win_test __ARGS((MPI_Win, int *));
+extern int PMPI_Win_unlock __ARGS((int, MPI_Win));
 extern int PMPI_Win_wait __ARGS((MPI_Win));
 extern double PMPI_Wtick __ARGS((void));
 extern double PMPI_Wtime __ARGS((void));
@@ -1079,7 +1078,7 @@ extern int MPI_Comm_get_contextid __ARGS((MPI_Comm, int *));
  * exist yet) 
  */
 #if LAM_WANT_MPI2CPP & !LAM_BUILDING
-#include <mpi++.h>
+#include <mpicxx.h>
 #endif
 #endif
 
