@@ -1,35 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
- * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	$Id: mpisys.h,v 6.27 2001/01/25 06:04:37 jsquyres Exp $
+ * $Id: mpisys.h,v 6.57.2.2 2003/08/28 23:04:56 jsquyres Exp $
  *
  *	Function:	- MPI system header file
  *			- the real meat is here
@@ -50,6 +33,9 @@
 #include <mpi.h>
 #include <portable.h>
 #include <terror.h>
+#include <all_opt.h>
+#include <lam-ssi-coll.h>
+
 
 /*
  * LAM constants & macros
@@ -59,9 +45,7 @@
 #define LAM_COLLMAXLIN		4		/* linear coll. op. threshold */
 #define LAM_COLLMAXDIM		16		/* maximum cube dimension */
 #define LAM_PORTLEN		16		/* maximum port name length */
-#define LAM_MAXTAG		32767		/* maximum tag */
 
-#define RPI_SPLIT(l, c, args) 	((lam_c2c) ? c args : l args)
 
 /*
  * These constants correspond to FORTRAN datatypes.  They are used internally
@@ -78,6 +62,15 @@
 #define MPI_F_2REAL		((MPI_Datatype) &lam_mpi_2real)
 #define MPI_F_2DOUBLE_PRECISION	((MPI_Datatype) &lam_mpi_2dblprec)
 
+/*
+ * These constants correspnd to C++ datatypes.  They are used internally
+ * and should not be visible to C/Fortran MPI user code.
+ */
+#define MPI_CXX_BOOL            ((MPI_Datatype) &lam_mpi_cxx_bool)
+#define MPI_CXX_COMPLEX         ((MPI_Datatype) &lam_mpi_cxx_cplex)
+#define MPI_CXX_DOUBLE_COMPLEX  ((MPI_Datatype) &lam_mpi_cxx_dblcplex)
+#define MPI_CXX_LONG_DOUBLE_COMPLEX ((MPI_Datatype) &lam_mpi_cxx_ldblcplex)
+
 /*
  * Fortran copy and delete function generic types.
  */
@@ -165,7 +158,9 @@ struct _comm {
 #define LAM_CINTER	0x10			/* intercommunicator? */
 #define LAM_CLDEAD	0x20			/* local group dead? */
 #define LAM_CRDEAD	0x40			/* remote group dead? */
-#define LAM_CFAKE	0x80			/* fake IMPI comms */
+#define LAM_CIMPI	0x80			/* IMPI communicators */
+#define LAM_CHOMOG	0x100			/* this comm is endian homog */
+#define LAM_CHIDDEN	0x200			/* hidden communications */
 	int		c_contextid;		/* context ID */
 	int		c_refcount;		/* reference count */
 	MPI_Group	c_group;		/* local group */
@@ -187,7 +182,10 @@ struct _comm {
   /* This is not #if WANT_IMPI'ed out so that we can stay binary
      compatible with non-IMPI compiled programs */
         MPI_Comm        c_shadow;               /* shadow com for IMPI comms */
-        long		c_reserved[4];		/* for expansion */
+        long            c_reserved[4];          /* used in IMPI :-( */
+
+        lam_ssi_coll_actions_t  c_ssi_coll;
+        struct lam_ssi_coll_data *c_ssi_coll_data;
 };
 
 struct _osdhdr {
@@ -198,7 +196,7 @@ struct _osdhdr {
 	int		os_type;		/* target datatype basic type */
 	int		os_blklen;		/* target datatype block len. */
 	int		os_stride;		/* target datatype stride */
-	int		os_pad;			/* padding */
+	int		os_dcount;		/* target datatype count */
 	int8		os_disp;		/* target displacement */
 };
 
@@ -308,6 +306,8 @@ struct _port {
  */
 extern MPI_Comm		lam_comm_parent;
 extern struct _proc	*lam_myproc;
+extern struct _proc	**lam_procs;
+extern int		lam_num_procs;
 extern LIST		*lam_comms;
 extern LIST		*lam_ports;
 extern float8		lam_clockskew;
@@ -321,6 +321,8 @@ extern int		lam_ger;
 extern int		lam_homog;
 extern struct jobid_t	lam_jobid;
 extern int		lam_universe_size;
+extern int		lam_universe_ncpus;
+extern int		lam_universe_nnodes;
 extern int		lam_appnum;
 extern int		lam_rq_flblock;
 extern int		lam_rq_nreqs;
@@ -331,10 +333,20 @@ extern MPI_Request	lam_rq_bottom;
 extern int		lam_topfunc;
 extern int		lam_flinit;
 extern int		lam_flfinal;
+extern int		lam_mpi_max_tag;
+extern int		lam_mpi_max_cid;
 extern MPI_Datatype	lam_osdhdr_type;
 
+extern OPT              *lam_mpi_ao;
+
+extern char             *lam_argv0;
+
 extern struct kio_t     _kio;			/* kernel I/O block */
 
+/* Variable instantiated & initialised in lammpiinit.c for TotalView
+   DLL support */
+extern int lam_tv_comm_seqnum;
+
 /*
  * state checking macros
  */
@@ -402,26 +414,11 @@ extern struct kio_t     _kio;			/* kernel I/O block */
 	lam_rq_flblock = 1;			\
 }
 
-#if 0
-#define lam_setfunc_m(locfunc)				\
-{							\
-	if (lam_topfunc == 0) {				\
-		lam_topfunc = (locfunc);		\
-	}						\
-}
-
-#define lam_resetfunc_m(locfunc)				\
-if (lam_topfunc == (locfunc)) {					\
-	((struct _fyiproc *) _kio.ki_fyi)->fyp_func = 0;	\
-	lam_topfunc = 0;					\
-}
-
-#define lam_getfunc_m()	(lam_topfunc)
-#else
+/* Used to be macros - now more complex functions */
 #define lam_setfunc_m(locfunc) lam_setfunc((locfunc))
 #define lam_resetfunc_m(locfunc) lam_resetfunc((locfunc))
 #define lam_getfunc_m() lam_getfunc()
-#endif
+
 
 #define lam_inited_m()	(lam_flinit)
 
@@ -430,6 +427,19 @@ if (lam_topfunc == (locfunc)) {					\
 
 #define lam_initerr_m()	lam_initerr()
 
+
+/*
+ * In some cases, the underlying memcpy() is broken.  So we have our
+ * own.  See the note in share/mpi/lammemcpy.c for more details.
+ */
+
+#if LAM_WANT_PREFIX_MEMCPY
+  extern void *lam_memcpy(void *dest, const void *src, size_t size);
+#else
+  #define lam_memcpy(a, b, c) memcpy((a), (b), (c))
+#endif
+
+
 /*
  * profiling
  */
@@ -445,6 +455,8 @@ if (lam_topfunc == (locfunc)) {					\
 #define MPIL_Universe_size		PMPIL_Universe_size
 #define MPIL_Trace_on			PMPIL_Trace_on
 #define MPIL_Trace_off			PMPIL_Trace_off
+#define MPIL_Request_set_name           PMPIL_Request_set_name
+#define MPIL_Request_get_name           PMPIL_Request_get_name
 
 #define MPI_Abort			PMPI_Abort 
 #define MPI_Accumulate			PMPI_Accumulate
@@ -484,7 +496,6 @@ if (lam_topfunc == (locfunc)) {					\
 #define MPI_Comm_delete_attr		PMPI_Comm_delete_attr 
 #define MPI_Comm_disconnect		PMPI_Comm_disconnect
 #define MPI_Comm_dup			PMPI_Comm_dup 
-#define MPI_COMM_DUP_FN			PMPI_COMM_DUP_FN
 #define MPI_Comm_f2c			PMPI_Comm_f2c
 #define MPI_Comm_free			PMPI_Comm_free 
 #define MPI_Comm_free_keyval		PMPI_Comm_free_keyval
@@ -506,7 +517,6 @@ if (lam_topfunc == (locfunc)) {					\
 #define MPI_Comm_spawn_multiple		PMPI_Comm_spawn_multiple
 #define MPI_Comm_split			PMPI_Comm_split 
 #define MPI_Comm_test_inter		PMPI_Comm_test_inter 
-#define MPI_DUP_FN			PMPI_DUP_FN
 #define MPI_Dims_create			PMPI_Dims_create 
 #define MPI_Errhandler_c2f              PMPI_Errhandler_c2f
 #define MPI_Errhandler_create		PMPI_Errhandler_create 
@@ -607,7 +617,6 @@ if (lam_topfunc == (locfunc)) {					\
 #define MPI_Startall			PMPI_Startall
 #define MPI_Status_c2f			PMPI_Status_c2f
 #define MPI_Status_f2c			PMPI_Status_f2c
-#define MPI_TYPE_DUP_FN			PMPI_TYPE_DUP_FN
 #define MPI_Test			PMPI_Test 
 #define MPI_Test_cancelled		PMPI_Test_cancelled
 #define MPI_Testall			PMPI_Testall 
@@ -648,12 +657,12 @@ if (lam_topfunc == (locfunc)) {					\
 #define MPI_Type_vector			PMPI_Type_vector 
 #define MPI_Unpack			PMPI_Unpack 
 #define MPI_Unpublish_name		PMPI_Unpublish_name
-#define MPI_WIN_DUP_FN			PMPI_WIN_DUP_FN
 #define MPI_Wait			PMPI_Wait 
 #define MPI_Waitall			PMPI_Waitall 
 #define MPI_Waitany			PMPI_Waitany 
 #define MPI_Waitsome			PMPI_Waitsome 
 #define MPI_Win_c2f			PMPI_Win_c2f
+#define MPI_Win_call_errhandler		PMPI_Win_call_errhandler
 #define MPI_Win_complete		PMPI_Win_complete
 #define MPI_Win_create			PMPI_Win_create
 #define MPI_Win_create_errhandler	PMPI_Win_create_errhandler
@@ -667,11 +676,14 @@ if (lam_topfunc == (locfunc)) {					\
 #define MPI_Win_get_errhandler		PMPI_Win_get_errhandler
 #define MPI_Win_get_group		PMPI_Win_get_group 
 #define MPI_Win_get_name		PMPI_Win_get_name 
+#define MPI_Win_lock			PMPI_Win_lock 
 #define MPI_Win_post			PMPI_Win_post
 #define MPI_Win_set_attr		PMPI_Win_set_attr 
 #define MPI_Win_set_errhandler		PMPI_Win_set_errhandler
 #define MPI_Win_set_name		PMPI_Win_set_name 
 #define MPI_Win_start			PMPI_Win_start
+#define MPI_Win_test			PMPI_Win_test
+#define MPI_Win_unlock			PMPI_Win_unlock
 #define MPI_Win_wait			PMPI_Win_wait
 #define MPI_Wtick			PMPI_Wtick 
 #define MPI_Wtime			PMPI_Wtime 
@@ -695,10 +707,16 @@ extern int	lam_chkarg __ARGS((void *, int, MPI_Datatype,
 				int, int, MPI_Comm));
 extern int	lam_clocksync __ARGS((int, struct _gps *, float8 *));
 extern int	lam_coll2pt __ARGS((int));
+extern int	lam_mpi_coll2pt __ARGS((int));
 extern void	lam_commfault __ARGS((int));
+extern void	lam_comm_errfatal __ARGS((MPI_Comm *, int *, ...));
 extern int	lam_comm_free __ARGS((MPI_Comm));
 extern int	lam_comm_new __ARGS((int, MPI_Group, MPI_Group,
 				int, MPI_Comm *));
+extern int	lam_coll_alloc_inter_cid __ARGS((MPI_Comm, int, 
+						 MPI_Comm, int,
+						 int, int, int *));
+extern int	lam_coll_alloc_intra_cid __ARGS((MPI_Comm, int, int, int *));
 extern int	lam_cubedim __ARGS((int));
 extern int	lam_delkey __ARGS((HASH *, int));
 extern void	lam_dtalign __ARGS((MPI_Datatype));
@@ -711,12 +729,16 @@ extern void	lam_emptystat __ARGS((MPI_Status *));
 extern int	lam_err_comm __ARGS((MPI_Comm, int, int, char *));
 extern int	lam_err_win __ARGS((MPI_Win, int, int, char *));
 extern int	lam_errfunc __ARGS((MPI_Comm, int, int));
-extern void	lam_comm_errfatal __ARGS((MPI_Comm *, int *, ...));
-extern void	lam_win_errfatal __ARGS((MPI_Win *, int *, ...));
 extern void	lam_errreturn __ARGS((void *, int *, ...));
+extern int      lam_exchange_endian_info(struct _proc **procs, int numprocs);
 extern int	lam_finalized __ARGS((void));
 extern int	lam_freekey __ARGS((int, int));
 extern void	lam_free_onesided __ARGS((void));
+extern int      lam_get_mpi_world __ARGS((int4 world_n, struct _gps *world, 
+					  struct _gps *mpiworld,
+					  char *prefix));
+extern int      lam_match_ssi_modules __ARGS((int4 world_n, struct _gps *world, 
+                                              char *prefix));
 extern struct _attrkey *lam_getattr __ARGS((int));
 extern int	lam_getcid __ARGS((void));
 extern int	lam_getfunc __ARGS((void));
@@ -727,10 +749,12 @@ extern int	lam_getpublished  __ARGS((char *, char **));
 extern int	lam_hibit __ARGS((int, int));
 #if LAM_WANT_IMPI
 extern int	lam_impicoll2pt __ARGS((int));
+extern int	lam_mpi_impicoll2pt __ARGS((int));
 #endif
 extern int	lam_inited __ARGS((void));
 extern void	lam_initerr __ARGS((void));
 extern int	lam_initfault __ARGS((void));
+extern int	lam_init_cid __ARGS((void));
 extern int      lam_init_comm_world __ARGS((struct _gps *mpi_procs, 
 					    int mpi_nprocs));
 extern int	lam_init_onesided __ARGS((void));
@@ -744,7 +768,7 @@ extern int      lam_get_proc_array __ARGS((struct _gps **, int *, int *,
 extern int	lam_keyval_create __ARGS((void_fn *, void_fn *, int *,
 					void *, int));
 extern void	lam_land __ARGS((void *, void *, int *, MPI_Datatype *));
-extern int	lam_linit __ARGS((char *, int *, int *, int *, 
+extern int	lam_linit __ARGS((int, char **, char *, int *, int *, int *, 
 				struct _gps **, int *));
 extern void	lam_lor __ARGS((void *, void *, int *, MPI_Datatype *));
 extern void	lam_lxor __ARGS((void *, void *, int *, MPI_Datatype *));
@@ -754,15 +778,21 @@ extern void	lam_min __ARGS((void *, void *, int *, MPI_Datatype *));
 extern void	lam_minloc __ARGS((void *, void *, int *, MPI_Datatype *));
 extern void	lam_replace __ARGS((void *, void *, int *, MPI_Datatype *));
 extern void	lam_mkcoll __ARGS((MPI_Comm));
+extern void	lam_mpi_mkcoll __ARGS((MPI_Comm));
 extern int	lam_mkerr __ARGS((int, int));
 #if LAM_WANT_IMPI
 extern void     lam_mkimpicoll __ARGS((MPI_Comm comm));
+extern void     lam_mpi_mkimpicoll __ARGS((MPI_Comm comm));
 #endif
 extern int	lam_mkkey __ARGS((void));
 extern void	lam_mkpt __ARGS((MPI_Comm));
+extern void	lam_mpi_mkpt __ARGS((MPI_Comm));
+extern int      lam_mpi_init(int *pargc, char ***pargv, int requested, 
+			     int *pprovided, int blk);
 extern void	lam_mpi_reset_sighandlers  __ARGS((void));
 extern int	lam_mpi_set_sighandlers  __ARGS((void));
 extern struct _proc *lam_nextproc __ARGS((void));
+extern int      lam_nbarrier(struct _proc **procs, int nprocs);
 extern int	lam_nprocs __ARGS((void));
 extern void	lam_nukecids __ARGS((void));
 extern void	lam_nukefunc __ARGS((void));
@@ -788,19 +818,25 @@ extern int	lam_port_is_open __ARGS((int));
 extern int	lam_port_open __ARGS((void));
 extern struct _proc *lam_procadd __ARGS((struct _gps *));
 extern int	lam_proccmp __ARGS((struct _proc *, struct _proc *));
+extern void     lam_procsort(struct _proc **procs, int nprocs);
 extern struct _proc *lam_procfind __ARGS((struct _gps *));
 extern int	lam_procfree __ARGS((struct _proc *));
 extern int	lam_procrm __ARGS((struct _proc *));
 extern void	lam_prod __ARGS((void *, void *, int *, MPI_Datatype *));
 extern int	lam_pt2coll __ARGS((int));
+extern int	lam_mpi_pt2coll __ARGS((int));
 #if LAM_WANT_IMPI
 extern int	lam_pt2impicoll __ARGS((int));
+extern int	lam_mpi_pt2impicoll __ARGS((int));
 extern int      lam_pt2impidatasync __ARGS((int));
+extern int      lam_mpi_pt2impidatasync __ARGS((int));
 extern int      lam_pt2impisyncack __ARGS((int));
+extern int      lam_mpi_pt2impisyncack __ARGS((int));
 #endif
 extern int	lam_publish __ARGS((char *, char *));
 extern int	lam_putkey __ARGS((HASH **, int));
 extern int	lam_rank_in_group __ARGS((MPI_Group, struct _proc *));
+extern int	lam_mpi_rank_in_group __ARGS((MPI_Group, struct _proc *));
 extern int	lam_recv __ARGS((void *, int, MPI_Datatype, int,
 				int, MPI_Comm, MPI_Status *));
 extern void	lam_resetfunc __ARGS((int));
@@ -810,9 +846,6 @@ extern int	lam_send __ARGS((void *, int, MPI_Datatype, int, int,
 extern void	lam_setcid __ARGS((int));
 extern void	lam_setfunc __ARGS((int));
 extern void	lam_setparam __ARGS((int, int, int));
-extern int      lam_get_mpi_world __ARGS((int4 world_n, struct _gps *world, 
-					  struct _gps *mpiworld,
-					  char *prefix));
 extern void	lam_sum __ARGS((void *, void *, int *, MPI_Datatype *));
 extern int	lam_test __ARGS((MPI_Request *, int, int *, MPI_Status *));
 extern struct _proc *lam_topproc __ARGS((void));
@@ -838,12 +871,19 @@ extern int      lam_tr_buoy __ARGS((const char *, double));
 extern int      lam_tr_coloron __ARGS((int, int, int, double));
 extern int      lam_tr_coloroff __ARGS((double));
 extern int      lam_tr_commname __ARGS((char *, int, double));
+extern void     *lam_tv_load_type_defs(void);
 extern int	lam_type_free __ARGS((MPI_Datatype));
 extern void	lam_type_setdefaults __ARGS((MPI_Datatype));
 extern int	lam_unpack __ARGS((char *, int, char *, int, MPI_Datatype));
 extern int	lam_unpublish __ARGS((char *));
+extern int      lam_updown_init __ARGS((void));
+extern int      lam_updown_finalize __ARGS((void));
+extern int      lam_updown_reset_thr_level __ARGS((void));
 extern void	lam_unsyncsql __ARGS((int, int, int *, int *,
 					int *, int *, int *, int *));
+extern void	lam_mpi_unsyncsql __ARGS((int, int, int *, int *,
+                                          int *, int *, int *, int *));
+extern void	lam_win_errfatal __ARGS((MPI_Win *, int *, ...));
 extern void	lam_yield __ARGS((void));
 
 extern void	_m2l_ack __ARGS((struct nmsg *));
@@ -871,6 +911,7 @@ extern int	_rpi_c2c_advance __ARGS((MPI_Request, int));
 extern int	_rpi_c2c_build __ARGS((MPI_Request));
 extern int	_rpi_c2c_destroy __ARGS((MPI_Request));
 extern int	_rpi_c2c_finalize __ARGS((struct _proc *));
+extern char  	*_rpi_c2c_get_name __ARGS((void));
 extern int	_rpi_c2c_init __ARGS((void));
 extern int	_rpi_c2c_iprobe __ARGS((MPI_Request));
 extern int	_rpi_c2c_start __ARGS((MPI_Request, MPI_Request));
