@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: kinit.c,v 6.7.2.1 2002/10/09 19:49:04 brbarret Exp $
+ * $Id: kinit.c,v 6.19 2003/03/18 04:03:53 jsquyres Exp $
  *
  *	Function:	- initializes data structures for kernel interface
  *			- calls kattach()
@@ -24,16 +22,21 @@
  *	Accepts:	- scheduling priority
  */
 
-#include <fcntl.h>
 #include <stdio.h>
+#include <fcntl.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include <freq.h>
 #include <kio.h>
 #include <kreq.h>
 #include <ktrace.h>
 #include <net.h>
+#include <priority.h>
 #include <typical.h>
+#include <vreq.h>
+#include <etc_misc.h>
+
 
 /*
  * external data structures
@@ -52,9 +55,15 @@ extern void		_vcfree();	/* free v.c. table */
 int
 kinit(int priority)
 {
-    int		i;		/* favourite index */
+    int i, ret;
+    char *version_string;
 
-    if (_kio.ki_pid != getpid()) {
+    /*
+     * Register the pid with the pid cache code
+     */
+    lam_register_pid(getpid());
+
+    if (_kio.ki_pid != lam_getpid()) {
 	LAM_ZERO_ME(_kio);
 /*
  * Initialize signal handling module.
@@ -77,7 +86,7 @@ kinit(int priority)
 	_kio.ki_origin = NOTNODEID;
 	_kio.ki_index = -1;
 	_kio.ki_jobid.jid_node = getnodeid();
-	_kio.ki_jobid.jid_pid = getpid();
+	_kio.ki_jobid.jid_pid = lam_getpid();
 /*
  * Free the virtual circuit table.
  */
@@ -108,12 +117,37 @@ kinit(int priority)
 	_ufd[2].fu_tfd = _kio.ki_stderr;
 
 	atkexit(rfatexit, (char *) 0);
+
 /*
  * Attach process to the kernel.
  */
 	if (kattach(priority)) {
 	    return(LAMERROR);
 	}
+
+	if (priority == PRCMD) {
+	  /* Call down to the lamd and ensure that its LAM version
+	     matches the LAM version that we were compiled against.
+	     Bail if the versions do not match. 
+	  
+	     It isn't perfect, but we only do this if priority is
+	     PRCMD because we don't want to get into a chicken and egg
+	     problem if we are trying to bring up the pds as seperate
+	     processes.  In this case, the versiond calls kinit, which
+	     leads to all kinds of badness.  We only do PRCMD because
+	     all the lam utils, MPI_INIT, and pretty much anything
+	     else of interest that isn't a pd runs as priority PRCMD
+	     and neither Jeff nor I really wanted to figure out the
+	     mojo that is pd priority assignments. */
+
+	  if (rversion(&version_string, NULL, NULL, NULL, NULL, 
+		       NULL) == LAMERROR || errno == EFAULT) {
+	    show_help(NULL, "version-mismatch", LAM_VERSION, version_string,
+		      NULL);
+	    kexit(1);
+	  }
+	}
+
 /*
  * Initialize the trace subsystem.
  */
@@ -121,13 +155,15 @@ kinit(int priority)
 	    lam_trinit();
 	}
 
-	return(0);
+	ret = 0;
     }
 
     else {
 /*
  * Attach process to the kernel.
  */
-	return(kattach(priority));
+      ret = kattach(priority);
     }
+
+    return ret;
 }
