@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN/RBD
+ * $HEADER$
  *
- *	$Id: MPI.c,v 6.4 2001/03/20 06:15:08 jsquyres Exp $
+ * $Id: MPI.c,v 6.8 2003/05/31 22:28:50 jsquyres Exp $
  *
  *	Function:	- F77 support
  */
@@ -48,6 +28,7 @@
 #include <mpisys.h>
 #include <terror.h>
 #include <typical.h>
+#include <etc_misc.h>
 
 /*
  * public functions
@@ -246,9 +227,10 @@ int			func;
 	if ((*errcode == MPI_SUCCESS) && (c_handle != 0)) {
 
 		if ((*f77_handle = lam_F_alloc_hdl()) < 0) {
-
+			lam_setfunc_m(func);
 			*errcode = lam_errfunc(MPI_COMM_WORLD, func,
 					lam_mkerr(MPI_ERR_OTHER, errno));
+			lam_resetfunc_m(func);
 			*f77_handle = -1;
 		} else {
 			lam_F_handles[*f77_handle] = c_handle;
@@ -336,7 +318,7 @@ int			len;
  * Copy F77 string into C string and NULL terminate it.
  */
 	if (len > 0) {
-		strncpy(cstr, str, len);
+		lam_strncpy(cstr, str, len);
 	}
 	cstr[len] = 0;
 
@@ -361,7 +343,7 @@ int			len;
 {
 	int		i;
 	
-	strncpy(f77str, cstr, len);
+	lam_strncpy(f77str, cstr, len);
 
 	for (i = strlen(cstr); i < len; i++) {
 		f77str[i] = ' ';
