@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	JRV/RBD
+ * $HEADER$
  *
- *	$Id: allgatherv.c,v 6.4.2.1 2002/10/09 19:49:05 brbarret Exp $
+ * $Id: allgatherv.c,v 6.9 2003/02/04 18:06:51 jsquyres Exp $
  *
  *	Function:	- gather varying length buffers at all process ranks
  *	Accepts:	- send buffer
@@ -28,10 +26,13 @@
  *	Returns:	- MPI_SUCCESS or an MPI error code
  */
 
+#include <lam_config.h>
+
 #include <blktype.h>
 #include <mpi.h>
 #include <mpisys.h>
-#include <lam_config.h>
+#include <lam-ssi-coll.h>
+
 
 /*@
 
@@ -71,7 +72,7 @@ Notes:
 
 .N Errors
 .N MPI_ERR_COMM
-.N MPI_ERR_INTERCOMM
+.N MPI_ERR_COLL_NOT_IMPLEMENTED
 .N MPI_ERR_IMPI
 .N MPI_ERR_BUFFER
 .N MPI_ERR_COUNT
@@ -83,62 +84,54 @@ int MPI_Allgatherv(void *sbuf, int scount, MPI_Datatype sdtype,
 		   void * rbuf, int *rcounts, int *disps, 
 		   MPI_Datatype rdtype, MPI_Comm comm)
 {
-	int		i;			/* favourite index */
-	int		size;			/* group size */
-	int		err;			/* error code */
+        int err;
+	lam_ssi_coll_allgatherv_fn_t func;
 
 	lam_initerr();
 	lam_setfunc(BLKMPIALLGATHERV);
 /*
  * Check for invalid arguments.
  */
-	if ((comm == MPI_COMM_NULL) || LAM_IS_INTER(comm)) {
-		return(lam_errfunc(comm, BLKMPIALLGATHERV,
-				lam_mkerr(MPI_ERR_COMM, 0)));
+	if (comm == MPI_COMM_NULL) {
+	  return(lam_errfunc(comm, BLKMPIALLGATHERV,
+			     lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
+	func = LAM_IS_INTER(comm) ?
+	  comm->c_ssi_coll.lsca_allgatherv_inter :
+	  comm->c_ssi_coll.lsca_allgatherv_intra;
 
 	if ((sdtype == MPI_DATATYPE_NULL) || (rdtype == MPI_DATATYPE_NULL)) {
-		return(lam_errfunc(comm, BLKMPIALLGATHERV,
-				lam_mkerr(MPI_ERR_TYPE, 0)));
+	  return(lam_errfunc(comm, BLKMPIALLGATHERV,
+			     lam_mkerr(MPI_ERR_TYPE, EINVAL)));
 	}
 
 	if (disps == 0) {
-		return(lam_errfunc(comm, BLKMPIALLGATHERV,
-				lam_mkerr(MPI_ERR_ARG, 0)));
+	  return(lam_errfunc(comm, BLKMPIALLGATHERV,
+			     lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	if ((scount < 0) || (rcounts == 0)) {
-		return(lam_errfunc(comm, BLKMPIALLGATHERV,
-				lam_mkerr(MPI_ERR_COUNT, 0)));
+	  return(lam_errfunc(comm, BLKMPIALLGATHERV,
+			     lam_mkerr(MPI_ERR_COUNT, EINVAL)));
 	}
 
-#if LAM_WANT_IMPI
-
-	/* Remove this when IMPI collectives are implemented */
-
-        if (LAM_IS_IMPI(comm)) {
-	  return lam_err_comm(comm, MPI_ERR_COMM, 0, 
-			      "Collectives not yet implemented on IMPI communicators");
+	if (func == NULL) {
+	  return(lam_errfunc(comm, BLKMPIALLGATHERV,
+			     lam_mkerr(MPI_ERR_OTHER, ENOT_IMPLEMENTED)));
 	}
-#endif
 
 	LAM_TRACE(lam_tr_cffstart(BLKMPIALLGATHERV));
-/*
- * Collect all values at each process, one at a time.
- */
-	MPI_Comm_size(comm, &size);
 
-	for (i = 0; i < size; ++i) {
+	/* Call the coll SSI to actually perform the barrier */
+    
+	if ((err = func(sbuf, scount, sdtype, rbuf, rcounts, disps, 
+			rdtype, comm)) != MPI_SUCCESS)
+	  return(lam_errfunc(comm, BLKMPIALLGATHERV,
+			     lam_mkerr(MPI_ERR_COMM, err)));
 
-		err = MPI_Gatherv(sbuf, scount, sdtype, rbuf,
-					rcounts, disps, rdtype, i, comm);
-		if (err != MPI_SUCCESS) {
-			return(lam_errfunc(comm, BLKMPIALLGATHERV, err));
-		}
-	}
+	/* All done */
 
 	LAM_TRACE(lam_tr_cffend(BLKMPIALLGATHERV, -1, comm, sdtype, scount));
-
 	lam_resetfunc(BLKMPIALLGATHERV);
 	return(MPI_SUCCESS);
 }
