@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	JRV/RBD
+ * $HEADER$
  *
- *	$Id: allreduce.c,v 6.5.2.1 2002/10/09 19:49:05 brbarret Exp $
+ * $Id: allreduce.c,v 6.9 2003/01/31 22:20:45 jsquyres Exp $
  *
  *	Function:	- reduce and return data to all processes
  *	Accepts:	- send buffer
@@ -27,9 +25,11 @@
  */
 
 #include <lam_config.h>
+
 #include <blktype.h>
 #include <mpi.h>
 #include <mpisys.h>
+#include <lam-ssi-coll.h>
 
 /*@
 
@@ -54,7 +54,7 @@ Output Parameter:
 
 .N Errors
 .N MPI_ERR_COMM
-.N MPI_ERR_INTERCOMM
+.N MPI_ERR_COLL_NOT_IMPLEMENTED
 .N MPI_ERR_IMPI
 .N MPI_ERR_BUFFER
 .N MPI_ERR_COUNT
@@ -66,39 +66,43 @@ Output Parameter:
 int MPI_Allreduce(void *sbuf, void *rbuf, int count, 
 		  MPI_Datatype dtype, MPI_Op op, MPI_Comm comm)
 {
-	int		err;			/* error code */
+        int err;
+	lam_ssi_coll_allreduce_fn_t func;
 
 	lam_initerr();
 	lam_setfunc(BLKMPIALLREDUCE);
 /*
  * Check for invalid arguments.
  */
-	if ((comm == MPI_COMM_NULL) || LAM_IS_INTER(comm)) {
-		return (lam_errfunc(comm, BLKMPIALLREDUCE,
-				lam_mkerr(MPI_ERR_COMM, 0)));
+	if (comm == MPI_COMM_NULL) {
+	  return (lam_errfunc(comm, BLKMPIALLREDUCE,
+			      lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
+	func = LAM_IS_INTER(comm) ?
+	  comm->c_ssi_coll.lsca_allreduce_inter :
+	  comm->c_ssi_coll.lsca_allreduce_intra;
 
 	if (dtype == MPI_DATATYPE_NULL) {
-		return(lam_errfunc(comm, BLKMPIALLREDUCE,
-				lam_mkerr(MPI_ERR_TYPE, 0)));
+	  return(lam_errfunc(comm, BLKMPIALLREDUCE,
+			     lam_mkerr(MPI_ERR_TYPE, EINVAL)));
+	}
+
+	if (func == NULL) {
+	  return(lam_errfunc(comm, BLKMPIALLREDUCE,
+			     lam_mkerr(MPI_ERR_OTHER, ENOT_IMPLEMENTED)));
 	}
 
 	LAM_TRACE(lam_tr_cffstart(BLKMPIALLREDUCE));
-/*
- * Reduce to 0 and broadcast.
- */
-	err = MPI_Reduce(sbuf, rbuf, count, dtype, op, 0, comm);
-	if (err != MPI_SUCCESS) {
-		return (lam_errfunc(comm, BLKMPIALLREDUCE, err));
-	}
 
-	err = MPI_Bcast(rbuf, count, dtype, 0, comm);
-	if (err != MPI_SUCCESS) {
-		return (lam_errfunc(comm, BLKMPIALLREDUCE, err));
-	}
+	/* Call the coll SSI to actually perform the barrier */
+    
+	if ((err = func(sbuf, rbuf, count, dtype, op, comm)) != MPI_SUCCESS)
+	  return(lam_errfunc(comm, BLKMPIALLREDUCE,
+			     lam_mkerr(MPI_ERR_COMM, err)));
+
+	/* All done */
 
 	LAM_TRACE(lam_tr_cffend(BLKMPIALLREDUCE, -1, comm, dtype, count));
-
 	lam_resetfunc(BLKMPIALLREDUCE);
 	return(MPI_SUCCESS);
 }
