@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: cfree.c,v 6.6.2.1 2002/10/09 19:49:06 brbarret Exp $
+ * $Id: cfree.c,v 6.10 2002/12/14 13:39:40 jsquyres Exp $
  *
  *	Function:	- free a communicator
  *	Accepts:	- communicator (inout)
@@ -78,21 +76,33 @@ MPI_Comm_free(MPI_Comm *comm)
  */
 	if (comm == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPICOMMFREE, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPICOMMFREE, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if (*comm == MPI_COMM_NULL || ((*comm)->c_flags & LAM_PREDEF)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPICOMMFREE, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPICOMMFREE, lam_mkerr(MPI_ERR_COMM, EINVAL)));
+	}
+/*
+ * Once parent communicator is freed subsequent calls to MPI_Comm_get_parent()
+ * must return MPI_COMM_NULL. See MPI-2 Standard, Section 5.3.2.
+ */
+	if (*comm == lam_comm_parent) {
+		lam_comm_parent = MPI_COMM_NULL;
 	}
 /*
  * Delete the communicator if it has no pending requests.
  */
 	if (--((*comm)->c_refcount) == 0) {
 
+	  /* Tell the coll SSI module that this comm is now gone */
+
+	  if ((err = (*comm)->c_ssi_coll.lsca_finalize(*comm)) != 0)
+	    return(lam_errfunc(MPI_COMM_WORLD, BLKMPICOMMFREE, err));
+
 #if LAM_WANT_IMPI
-	  if (LAM_IS_IMPI(*comm))
-	    MPI_Comm_free(&(*comm)->c_shadow);
+		if (LAM_IS_IMPI(*comm))
+			MPI_Comm_free(&(*comm)->c_shadow);
 #endif
 		err = lam_comm_free(*comm);
 		*comm = MPI_COMM_NULL;
