@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: connect.c,v 6.8 2001/01/22 17:35:37 jsquyres Exp $
+ * $Id: connect.c,v 6.14.2.1 2003/08/21 06:32:00 brbarret Exp $
  *
  *	Function:	- connect to an MPI port
  *	Accepts:	- port name
@@ -57,6 +37,7 @@
 #include <mpisys.h>
 #include <net.h>
 #include <portable.h>
+#include <lam-ssi-rpi.h>
 #include <rpisys.h>
 #include <terror.h>
 #include <typical.h>
@@ -134,26 +115,26 @@ MPI_Comm_connect(char *port_name, MPI_Info info, int root, MPI_Comm comm,
  */
 	if ((comm == MPI_COMM_NULL) || LAM_IS_INTER(comm)) {
 		return(lam_errfunc(comm, BLKMPICOMMCONNECT,
-					lam_mkerr(MPI_ERR_COMM, 0)));
+					lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	MPI_Comm_size(comm, &size);
 
 	if ((root >= size) || (root < 0)) {
 		return(lam_errfunc(comm, BLKMPICOMMCONNECT,
-					lam_mkerr(MPI_ERR_ROOT, 0)));
+					lam_mkerr(MPI_ERR_ROOT, EINVAL)));
 	}
 
 	if (newcomm == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPICOMMCONNECT,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	MPI_Comm_rank(comm, &rank);
 	
 	if ((rank == root) && (strlen(port_name) > LAM_PORTLEN)) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPICOMMCONNECT,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	LAM_TRACE(lam_tr_cffstart(BLKMPICOMMCONNECT));
@@ -294,11 +275,18 @@ MPI_Comm_connect(char *port_name, MPI_Info info, int root, MPI_Comm comm,
 /*
  * setup new processes
  */
-	if (RPI_SPLIT(_rpi_lamd_addprocs, _rpi_c2c_addprocs, ())) {
+	if (RPI_ADDPROCS(svrgrp->g_procs, svrgrp->g_nprocs)) {
 		return(lam_errfunc(comm, BLKMPICOMMCONNECT,
 				lam_mkerr(MPI_ERR_OTHER, errno)));
 	}
 	
+	/* Let the collective SSI modules battle over who will do
+           collectives on this new communicator.  */
+
+        if (lam_ssi_coll_base_init_comm(*newcomm) != 0)
+            return(lam_errfunc(comm, BLKMPICOMMCONNECT,
+                                    lam_mkerr(MPI_ERR_INTERN, ENOSYS)));
+
 	LAM_TRACE(lam_tr_cffend(BLKMPICOMMCONNECT, root, comm, 0, 0));
         lam_resetfunc_m(BLKMPICOMMCONNECT);
 	return(MPI_SUCCESS);
@@ -381,7 +369,7 @@ connect_to_port(char *port_name, MPI_Comm comm, int cid, int *svrcid,
  * Check for error code from server.
  */
 	if (inmsg.nh_data[3] != 0) {
-		return(lam_mkerr(inmsg.nh_data[3], 0));
+		return(lam_mkerr(MPI_ERR_OTHER, inmsg.nh_data[3]));
 	}
 
 	*svrcid = inmsg.nh_data[1];
@@ -454,5 +442,5 @@ fill_fyi(MPI_Comm comm)
         fyi->fyp_func = lam_getfunc();
         fyi->fyp_me = (lam_myproc->p_gps.gps_grank << 16)
 				| comm->c_group->g_myrank;
-        fyi->fyp_cidtag = (lam_pt2coll(comm->c_contextid) << 16);
+        fyi->fyp_cidtag = (lam_mpi_pt2coll(comm->c_contextid) << 16);
 }
