@@ -10,12 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
- *
- *	$Id: graphmap.c,v 6.3.2.1 2002/10/09 19:49:08 brbarret Exp $
+ * $Id: graphmap.c,v 6.5 2002/12/11 19:15:11 jsquyres Exp $
  *
  *	Function:	- map processes to graph coordinates
  *	Accepts:	- old communicator
@@ -68,17 +65,17 @@ int MPI_Graph_map(MPI_Comm comm, int nnodes, int *index,
  */
 	if (comm == MPI_COMM_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIGRAPHMAP, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPIGRAPHMAP, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if (LAM_IS_INTER(comm)) {
 		return(lam_errfunc(comm,
-			BLKMPIGRAPHMAP, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPIGRAPHMAP, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if ((nnodes < 1) || (index == 0) || (edges == 0) || (pnewrank == 0)) {
 		return(lam_errfunc(comm,
-			BLKMPIGRAPHMAP, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIGRAPHMAP, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Assign new rank.
