@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,12 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
- *
- *	$Id: join.c,v 1.6.2.1 2002/10/09 19:49:09 brbarret Exp $
+ * $Id: join.c,v 1.14.2.2 2003/10/11 19:04:02 brbarret Exp $
  *
  *	Function:	- join MPI applications
  *	Accepts:	- socket file descriptor
@@ -28,7 +25,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#if HAVE_STRINGS_H
+#if defined(HAVE_STRINGS_H) && HAVE_STRINGS_H
 #include <strings.h>
 #endif
 #include <sys/time.h>				/* LINUX FD_SET etc. */
@@ -36,7 +33,7 @@
 #include <sys/socket.h>
 #include <unistd.h>
 
-#if LAM_NEED_SYS_SELECT_H
+#if defined(HAVE_SYS_SELECT_H) && HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
@@ -46,6 +43,7 @@
 #include <mpi.h>
 #include <mpisys.h>
 #include <portable.h>
+#include <lam-ssi-rpi.h>
 #include <rpisys.h>
 #include <typical.h>
 
@@ -53,8 +51,9 @@
 /*
  * private functions
  */
-static int		exchange_with_peer();
-static int		readwrite();
+static int exchange_with_peer(int fd, int cid, int *peercid,
+                              struct _gps *peergps);
+static int readwrite(int fd, int nbytes, char *inbuf, char *outbuf);
 
 
 /*@
@@ -77,9 +76,45 @@ they were either initially lamboot(1)ed together, or a lamgrow(1)
 command was given to grow an initial LAM universe such that the
 resulting set includes the two hosts in question.
 
-Once this call completes successfully, 'fd' may not be used by the
-caller for any reason.  LAM will eventually close the socket (possibly
-as late as during 'MPI_Finalize'(3)).  
+The socket may not be used by the calling application until
+'MPI_Comm_join' has completed.  When 'MPI_Comm_join' completes, 'fd'
+may be used by the calling application.  Per MPI-2, the socket is only
+used for initial handshaking -- it will not be used as the
+communications channel between the newly connected processes.
+
+Sample:
+
+The following code shows an abbreviated example using 'MPI_Comm_join'
+since no example is provided in the MPI-2 standard.  Note that only
+one rank in each MPI application should call this function since each
+socket can only have two endpoints (i.e., an endpoint in each
+process).  Upon successful completion, the intercommunicator that is
+returned will have two members -- the local process and the remote
+process.
+
+.vb
+  MPI_Comm
+  do_join(char* server_name, int port) {
+    int sfd, fd = -1;
+    unsigned char addr[4];
+    MPI_Comm intercomm;
+
+    if (server_name != NULL) {
+      if (getinetaddr(server_name, &addr) == 0)
+        fd = open_socket_to_server(addr, port);
+    } else {
+      if ((sfd = open_listening_socket(&port)) >= 0) {
+        fd = accept_client_socket(sfd, -1);
+        close(sfd);
+      }
+    }
+    if (fd < 0)
+      MPI_Abort(MPI_COMM_WORLD, 0);
+
+    MPI_Comm_join(fd, &intercomm);
+    return intercomm;
+  }
+.ve
 
 .N fortran
 
@@ -110,12 +145,12 @@ MPI_Comm_join(int fd, MPI_Comm *newcomm)
  * Check the arguments.
  */
 	if (fd < 0) {
-		return(lam_err_comm(MPI_COMM_SELF, MPI_ERR_ARG, 0,
+		return(lam_err_comm(MPI_COMM_SELF, MPI_ERR_ARG, EINVAL,
 			"illegal file descriptor"));
 	}
 
 	if (newcomm == 0) {
-		return(lam_err_comm(MPI_COMM_SELF, MPI_ERR_ARG, 0,
+		return(lam_err_comm(MPI_COMM_SELF, MPI_ERR_ARG, EINVAL,
 			"null pointer"));
 	}
 
@@ -187,16 +222,24 @@ MPI_Comm_join(int fd, MPI_Comm *newcomm)
 /*
  * Setup the new processes.
  */
-	if (RPI_SPLIT(_rpi_lamd_addprocs, _rpi_c2c_addprocs, ())) {
+	if (RPI_ADDPROCS(peergrp->g_procs, peergrp->g_nprocs)) {
 		return(lam_err_comm(MPI_COMM_SELF, MPI_ERR_OTHER, errno, ""));
 	}
 
+	/* Let the collective SSI modules battle over who will do
+	   collectives on this new communicator.  */
+
+	if (lam_ssi_coll_base_init_comm(*newcomm) != 0)
+	  return(lam_errfunc(MPI_COMM_SELF, BLKMPICOMMJOIN,
+			     lam_mkerr(MPI_ERR_INTERN, ENOSYS)));
+
 	LAM_TRACE(lam_tr_cffend(BLKMPICOMMJOIN, 0, 0, 0, 0));
 
         lam_resetfunc_m(BLKMPICOMMJOIN);
 	return(MPI_SUCCESS);
 }
 
+
 /*
  *	exchange_with_peer
  *
@@ -208,13 +251,7 @@ MPI_Comm_join(int fd, MPI_Comm *newcomm)
  *	Returns:	- 0 or LAMERROR
  */
 static int
-exchange_with_peer(fd, cid, peercid, peergps)
-
-int			fd;
-int			cid;
-int			*peercid;
-struct _gps		*peergps;
-
+exchange_with_peer(int fd, int cid, int *peercid, struct _gps *peergps)
 {
 	struct _gps	mygps;
 	int4		cid4;
@@ -241,6 +278,7 @@ struct _gps		*peergps;
 	return(0);
 }
 
+
 /*
  *	readwrite
  *
@@ -252,13 +290,7 @@ struct _gps		*peergps;
  *	Returns:	- 0 or LAMERROR
  */
 static int
-readwrite(fd, nbytes, inbuf, outbuf)
-
-int			fd;
-int			nbytes;
-char			*inbuf;
-char			*outbuf;
-
+readwrite(int fd, int nbytes, char *inbuf, char *outbuf)
 {
 	fd_set		rfd;
 	fd_set		wfd;
