@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: keyfree.c,v 6.4.2.1 2002/10/09 19:49:10 brbarret Exp $
+ * $Id: keyfree.c,v 6.6 2002/12/11 19:15:12 jsquyres Exp $
  *
  *	Function:	- free attribute key
  *	Accepts:	- key value (in-out)
