@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN/RBD
+ * $HEADER$
  *
- *	$Id: lamcomm.c,v 6.9 2000/10/31 03:58:58 jsquyres Exp $
+ * $Id: lamcomm.c,v 6.20 2003/05/08 22:40:13 jsquyres Exp $
  *
  *	Function:	- miscellaneous communicator functions
  */
@@ -45,10 +25,11 @@
 #include <mpisys.h>
 #include <typical.h>
 #include <rpisys.h>
-#include <debug.h>
+#include <lamdebug.h>
 #if LAM_WANT_IMPI
 #include <impi.h>
 #endif
+#include <etc_misc.h>
 
 
 /*
@@ -63,7 +44,12 @@ static MPI_Group make_local(MPI_Group orig);
 /*
  * private variables
  */
-static int fl_debug = 0;
+static int comm_mpid = -1;
+
+
+/* Variable to keep track of changes in lam communicators for TV queue
+   support */
+extern int lam_tv_comm_seqnum;
 
 
 /*
@@ -79,6 +65,7 @@ lam_comm_free(MPI_Comm comm)
 	int		err;
 	int		key;
 	struct _attr	*p;
+
 /* 
  * Drop a trace saying that this name is no longer being used 
  */
@@ -137,7 +124,10 @@ lam_comm_free(MPI_Comm comm)
  * Remove the communicator from the communicator list and free it.
  */
 	al_delete(lam_comms, al_find(lam_comms, &comm));
-	free((char *) comm);
+	lam_mp_free(comm_mpid, comm);
+
+	/* Signal for TV queue library that lam_comms has changed */
+	++lam_tv_comm_seqnum;
 
 	return(MPI_SUCCESS);
 }
@@ -159,11 +149,18 @@ int
 lam_comm_new(int cid, MPI_Group l_group, MPI_Group r_group, int flags,
 	     MPI_Comm *comm)
 {
+	int i;
+	int homog = 1;
+        struct _proc *p;
+
 	if (*comm == 0) {
-		if ((*comm = (MPI_Comm) malloc(sizeof(struct _comm))) == 0) {
-			return(LAMERROR);
-		}
-	}
+          if (comm_mpid == -1)
+ 	    comm_mpid = lam_mp_init(sizeof(struct _comm), NULL, NULL,
+                                    LAM_DEFAULT_MP_POOL_SIZE,
+ 				    LAM_DEFAULT_MP_INCR_SIZE);
+ 	  if ((*comm = (MPI_Comm) lam_mp_malloc(comm_mpid)) == 0)
+            return(LAMERROR);
+        }
 
 	(*comm)->c_flags = flags;
 	(*comm)->c_f77handle = -1;
@@ -185,6 +182,33 @@ lam_comm_new(int cid, MPI_Group l_group, MPI_Group r_group, int flags,
 
 	memset((*comm)->c_name, 0, sizeof((*comm)->c_name));
 
+	/* Zero out the SSI function data, just so that there's no
+	   confusion */
+	
+	memset(&((*comm)->c_ssi_coll), 0, sizeof((*comm)->c_ssi_coll));
+	(*comm)->c_ssi_coll_data = NULL;
+
+	/* Is this communicator endian homogeneous? */
+
+	if (l_group != MPI_GROUP_NULL)
+	  for (i = 0; i < l_group->g_nprocs; ++i) {
+	    p = l_group->g_procs[i];
+	    if (p != lam_myproc && !(p->p_mode & LAM_PHOMOG)) {
+	      homog = 0;
+	      break;
+	    }
+	  }
+	if ((flags & LAM_CINTER) && homog)
+	  for (i = 0; i < r_group->g_nprocs; ++i) {
+	    p = r_group->g_procs[i];
+	    if (!(p->p_mode & LAM_PHOMOG)) {
+	      homog = 0;
+	      break;
+	    }
+	  }
+	if (homog)
+	  (*comm)->c_flags |= LAM_CHOMOG;
+
 	(*comm)->c_shadow = 0;
 #if LAM_WANT_IMPI
         /* If at least one member of the local or remote group is not
@@ -194,27 +218,33 @@ lam_comm_new(int cid, MPI_Group l_group, MPI_Group r_group, int flags,
 	   original communicator, but will be compressed to start with
 	   zero. */
 	if (_kio.ki_rtf & RTF_IMPI) {
-	  DBUG("Checking for remote nodes\n");
 	  if (num_remotes(l_group) > 0 || 
 	      num_remotes((*comm)->c_rgroup) > 0) {
 	    MPI_Group newl;
 	    MPI_Group newr;
 	    
-	    DBUG("Found remote notes -- making new local groups\n");
+	    /* This communicator is no longer endian-homogenous */
+
+	    (*comm)->c_flags &= ~(LAM_CHOMOG);
+
 	    newl = make_local(l_group);
 	    newr = make_local((*comm)->c_rgroup);
-	    DBUG("Making shadow communicator\n");
 	    
 	    /* Trickyness in lamcid.c -- cid's are incremented by 2, so
 	       we know that we can grab this contextid + 1 */
 	    
 	    lam_comm_new(cid + 1, newl, newr, flags, &((*comm)->c_shadow));
 	    (*comm)->c_shadow->c_errhdl->eh_refcount++;
+
+            if (lam_ssi_coll_base_init_comm((*comm)->c_shadow) != 0)
+              return LAMERROR;
 	  }
 	}
 #endif
 
-	DBUG("new communicator made!\n");
+	/* Signal for TV queue library that lam_comms has changed */
+	++lam_tv_comm_seqnum;
+
 	return(0);
 }
 
@@ -234,24 +264,18 @@ num_remotes(MPI_Group group)
   int i, count = 0;
 
   if (group == MPI_GROUP_NULL || group == MPI_GROUP_EMPTY) {
-    DBUG("No remote members of group\n");
     return 0;
   }
 
-  DBUG("Checking for remote members of group\n");
   for (i = 0; i < group->g_nprocs; i++)
     if (LAM_GPSCMP(&(group->g_procs[i]->p_gps), &gimpid) == 0) {
-      DBUG("Found remote member\n");
       count++;
     }
 
-  DBUG("%d: Returning %d remote members of this group\n", 
-       group->g_myrank, count);
   return count;
 }
 
 
-
 /*
  *	make_local
  *
@@ -268,7 +292,6 @@ make_local(MPI_Group orig)
   int *ranks;
   MPI_Group newg;
   
-  DBUG("Making local group\n");
   if (orig == MPI_GROUP_NULL || orig == MPI_GROUP_EMPTY)
     return orig;
 
@@ -276,20 +299,15 @@ make_local(MPI_Group orig)
   ranks = malloc(sizeof(int) * num);
   for (j = i = 0; i < orig->g_nprocs; i++)
     if (LAM_GPSCMP(&(orig->g_procs[i]->p_gps), &gimpid) == 0) {
-      DBUG("Found rank to exclude: %d (j = %d) pid %d\n", i, j,
-	   orig->g_procs[i]->p_gps.gps_pid);
       ranks[j++] = i;
     }
 
-  if (MPI_Group_excl(orig, num, ranks, &newg) != MPI_SUCCESS)
-    DBUG("MPI GROUP EXCL BARF!!!\n");
+  if (MPI_Group_excl(orig, num, ranks, &newg) != MPI_SUCCESS) {
+    free(ranks);
+    return MPI_GROUP_NULL;
+  }
   free(ranks);
 
-  DBUG("Got new group size: %d\n", newg->g_nprocs);
-  for (i = 0; i < newg->g_nprocs; i++)
-    DBUG("PID in new group: %d\n", newg->g_procs[i]->p_gps.gps_pid);
-
-  DBUG("All done making new group\n");
   return newg;
 }
 
