@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: lamdeferr.c,v 6.7 1999/08/26 20:53:55 jsquyres Exp $
+ * $Id: lamdeferr.c,v 6.12.2.1 2003/08/21 06:32:00 brbarret Exp $
  *
  *	Function:	- default MPI error handlers
  */
@@ -47,6 +27,7 @@
 #endif
 
 #include <lam_config.h>
+#include <etc_misc.h>
 #include <mpi.h>
 #include <mpisys.h>
 #include <tstdio.h>
@@ -109,7 +90,7 @@ va_dcl
 	errmsg = va_arg(arglist, char *);
 	va_end(arglist);
 
-	comm = (errcomm) ? *errcomm : MPI_COMM_WORLD;
+	comm = (errcomm) ? *errcomm : MPI_COMM_NULL;
 
 	errcode = lam_mkerr(*errclass, error);
 	report_error(comm, errcode, errmsg, "comm");
@@ -195,10 +176,10 @@ char			*type;
 		MPI_Comm_rank(comm, &rank);
 
 		if (comm->c_name[0]) {
-		    strncpy(name, comm->c_name, MPI_MAX_OBJECT_NAME);
+		    lam_strncpy(name, comm->c_name, MPI_MAX_OBJECT_NAME);
 		} else {
 		    MPIL_Comm_id(comm, &cid);
-		    cid = lam_coll2pt(cid);
+		    cid = lam_mpi_coll2pt(cid);
 		    sprintf(name, "%s %d", type, cid);
 		}
 
@@ -225,19 +206,19 @@ char			*type;
 		if (_kio.ki_rtf & RTF_IO) {
 			if (*errmsg) {
 				printf("%s: %s (node %d, pid %d)\n", mpierrmsg,
-					errmsg, getnodeid(), (int) getpid());
+					errmsg, getnodeid(), (int) lam_getpid());
 			} else {
 				printf("%s (node %d, pid %d)\n",
-					mpierrmsg, getnodeid(), (int) getpid());
+					mpierrmsg, getnodeid(), (int) lam_getpid());
 			}
 			fflush(stdout);
 		} else {
 			if (*errmsg) {
 				tprintf("%s: %s (node %d, pid %d)\n", mpierrmsg,
-					errmsg, getnodeid(), (int) getpid());
+					errmsg, getnodeid(), (int) lam_getpid());
 			} else {
 				tprintf("%s (node %d, pid %d)\n",
-					mpierrmsg, getnodeid(), (int) getpid());
+					mpierrmsg, getnodeid(), (int) lam_getpid());
 			}
 		}
 	}
