@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN/RBD
+ * $HEADER$
  *
- *	$Id: lamdtype.c,v 6.7.2.1 2002/10/09 19:49:10 brbarret Exp $
+ * $Id: lamdtype.c,v 6.11.2.2 2003/09/11 12:40:56 jsquyres Exp $
  *
  *	Function:	- miscellaneous and conversion datatype functions
  */
@@ -52,9 +50,11 @@ extern void		lam_tr_dtypefree();
 /*
  * local functions
  */
-static void		cpy_hvector();
-static void		cpy_hindexed();
-static void		cpy_struct();
+static void cpy_hvector(char *dest, char *src, int num, MPI_Datatype dtype, 
+                        int fl_vec);
+static void cpy_hindexed(char *dest, char *src, int num, MPI_Datatype dtype, 
+                         int fl_idx);
+static void cpy_struct(char *dest, char *src, int num, MPI_Datatype dtype);
 
 
 /*
@@ -65,10 +65,7 @@ static void		cpy_struct();
  *	Returns:	- 0 or LAMERROR
  */
 int
-lam_type_free(dt)
-
-MPI_Datatype		dt;
-
+lam_type_free(MPI_Datatype dt)
 {
     MPI_Datatype	subtype;
     struct _attr	*p;
@@ -103,8 +100,9 @@ MPI_Datatype		dt;
 		if (err != MPI_SUCCESS) return(err);
 	    }
 
-	    if (dt->dt_format == LAM_DTHINDEXED
-			|| dt->dt_format == LAM_DTINDEXED) {
+	    if (dt->dt_format == LAM_DTHINDEXED ||
+                dt->dt_format == LAM_DTHINDEXEDCREAT ||
+                dt->dt_format == LAM_DTINDEXED) {
 		if (dt->dt_disps) {
 		    free((char *) dt->dt_disps);
 		}
@@ -113,7 +111,7 @@ MPI_Datatype		dt;
 	    break;
 
 	case LAM_DTINDEXEDBLK:
-	    return(lam_mkerr(MPI_ERR_ARG, 0));
+	    return(lam_mkerr(MPI_ERR_ARG, EINVAL));
 
 	case LAM_DTSTRUCT:
 	case LAM_DTSTRUCTCREAT:
@@ -150,7 +148,7 @@ MPI_Datatype		dt;
 	    break;
 
 	default:
-	    return(lam_mkerr(MPI_ERR_ARG, 0));
+	    return(lam_mkerr(MPI_ERR_ARG, EINVAL));
 	}
     }
 /*
@@ -188,6 +186,7 @@ MPI_Datatype		dt;
     return(MPI_SUCCESS);
 }
 
+
 /*
  *	lam_type_setdefaults
  *
@@ -195,10 +194,7 @@ MPI_Datatype		dt;
  *	Accepts:	- datatype
  */
 void
-lam_type_setdefaults(dt)
-
-MPI_Datatype		dt;
-
+lam_type_setdefaults(MPI_Datatype dt)
 {
 	dt->dt_align = 1;
 	dt->dt_commit = 0;
@@ -226,6 +222,7 @@ MPI_Datatype		dt;
 	memset(dt->dt_name, 0, sizeof(dt->dt_name));
 }
 
+
 /*
  *	lam_dtsndrcv
  *
@@ -242,24 +239,15 @@ MPI_Datatype		dt;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_dtsndrcv(sbuf, scount, sdtype, rbuf, rcount, rdtype, tag, comm)
-
-void			*sbuf;
-int			scount;
-MPI_Datatype		sdtype;
-void			*rbuf;
-int			rcount;
-MPI_Datatype		rdtype;
-int			tag;
-MPI_Comm		comm;
-
+lam_dtsndrcv(void *sbuf, int scount, MPI_Datatype sdtype, void *rbuf,
+	     int rcount, MPI_Datatype rdtype, int tag, MPI_Comm comm)
 {
 	int		err;			/* error code */
 	int		size;			/* packed size */
 	int		rank;			/* caller's rank */
 	MPI_Status	stat;			/* status info */
 
-	err = lam_mkerr(MPI_ERR_TRUNCATE, 0);
+	err = lam_mkerr(MPI_ERR_TRUNCATE, EIO);
 /*
  * If same datatypes used, just copy.
  */
@@ -306,6 +294,7 @@ MPI_Comm		comm;
 	return(err);
 }
 
+
 /*
  *	lam_dtcpy
  *
@@ -317,13 +306,7 @@ MPI_Comm		comm;
  *			- datatype
  */
 void
-lam_dtcpy(dest, src, count, dtype)
-
-char			*dest;
-char			*src;
-int			count;
-MPI_Datatype		dtype;
-
+lam_dtcpy(char *dest, char *src, int count, MPI_Datatype dtype)
 {
 /*
  * If no packing or extent adjustment is required do a direct copy.
@@ -331,7 +314,7 @@ MPI_Datatype		dtype;
  */
 	if ((dtype->dt_flags & LAM_DTNOPACK)
 	    	&& ((count == 1) || (dtype->dt_flags & LAM_DTNOXADJ))) {
-	    memcpy(dest, src, count * dtype->dt_size);
+	  lam_memcpy(dest, src, count * dtype->dt_size);
 	}
 	else {
 	    switch(dtype->dt_format) {
@@ -368,20 +351,14 @@ MPI_Datatype		dtype;
 	}
 }
 
+
 /*
  *	cpy_hvector
  *
  *	Function:	- copy vector or hvector
  */
 static void
-cpy_hvector(dest, src, num, dtype, fl_vec)
-
-char			*dest;
-char			*src;
-int			num;
-MPI_Datatype		dtype;
-int			fl_vec;
-
+cpy_hvector(char *dest, char *src, int num, MPI_Datatype dtype, int fl_vec)
 {
 	MPI_Datatype	subtype;		/* type of vector elements */
 	int		blksize;		/* size of block */
@@ -446,7 +423,8 @@ int			fl_vec;
 	    else {
 		for (i = 0; i < num; ++i, src += extent, dest += extent) {
 		    for (j = 0; j < count; ++j) {
-			memcpy(dest + j * stride, src + j * stride, blksize);
+		      lam_memcpy(dest + j * stride, 
+				 src + j * stride, blksize);
 		    }
 		}
 	    }
@@ -464,20 +442,14 @@ int			fl_vec;
 	}
 }
 
+
 /*
  *	cpy_hindexed
  *
  *	Function:	- copy indexed or hindexed
  */
 static void
-cpy_hindexed(dest, src, num, dtype, fl_idx)
-
-char			*dest;
-char			*src;
-int			num;
-MPI_Datatype		dtype;
-int			fl_idx;
-
+cpy_hindexed(char *dest, char *src, int num, MPI_Datatype dtype, int fl_idx)
 {
 	MPI_Aint	*disp;			/* ptr displacements */
 	int		*len;			/* ptr lengths */
@@ -502,19 +474,14 @@ int			fl_idx;
 	}
 }
 
+
 /*
  *	cpy_struct
  *
  *	Function:	- copy struct
  */
 static void
-cpy_struct(dest, src, num, dtype)
-
-char			*dest;
-char			*src;
-int			num;
-MPI_Datatype		dtype;
-
+cpy_struct(char *dest, char *src, int num, MPI_Datatype dtype)
 {
 	MPI_Aint	*disp;			/* ptr displacements */
 	MPI_Datatype	*type;			/* ptr datatypes */
@@ -537,6 +504,7 @@ MPI_Datatype		dtype;
 	}
 }
 
+
 /*
  *	lam_dtblock
  *
@@ -549,13 +517,7 @@ MPI_Datatype		dtype;
  *			- block displacement
  */
 void
-lam_dtblock(newdtype, old, count, disp)
-
-MPI_Datatype		newdtype;
-MPI_Datatype		old;
-int			count;
-int			disp;
-
+lam_dtblock(MPI_Datatype newdtype, MPI_Datatype old, int count, int disp)
 {
 	int		extent;			/* extent of the block */
 	int		upper;			/* upper bound of new type */
@@ -677,7 +639,9 @@ int			disp;
 		} else {
 			if (!(newdtype->dt_flags & LAM_DTHASLB)
 					&& (old->dt_size > 0)) {
+			  if (newdtype->dt_datalow < newdtype->dt_lower) {
 				newdtype->dt_lower = newdtype->dt_datalow;
+			  }
 				newdtype->dt_flags |= LAM_DTLOWSET;
 			}
 		}
@@ -696,9 +660,16 @@ int			disp;
 			newdtype->dt_lower = newdtype->dt_upper;
 			newdtype->dt_flags |= LAM_DTLOWSET;
 		}
+	} else {
+	  /* If this is the last block and it is empty then we have to mark the
+	     upper bound for this new datatype */
+	  upper = disp;
+	  if (upper > newdtype->dt_upper)
+	    newdtype->dt_upper = upper;
 	}
 }
 
+
 /*
  *	lam_dtalign
  *
@@ -706,10 +677,7 @@ int			disp;
  *	Accepts:	- datatype
  */
 void
-lam_dtalign(dtype)
-
-MPI_Datatype		dtype;
-
+lam_dtalign(MPI_Datatype dtype)
 {
 	int		extent;			/* extent of datatype */
 	int		epsilon;		/* extra needed for alignment */
@@ -744,6 +712,7 @@ MPI_Datatype		dtype;
 	}
 }
 
+
 /*
  *	lam_dtbuffer
  *
@@ -755,13 +724,7 @@ MPI_Datatype		dtype;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_dtbuffer(dtype, count, buffer, origin)
-
-MPI_Datatype		dtype;
-int			count;
-char			**buffer;
-char			**origin;
-
+lam_dtbuffer(MPI_Datatype dtype, int count, char **buffer, char **origin)
 {
 	int		extent;			/* space required for extent */
 	int		up;			/* upper limit */
@@ -795,6 +758,7 @@ char			**origin;
 	return(MPI_SUCCESS);
 }
 
+
 /*
  *	lam_copyrev2
  *
@@ -806,12 +770,7 @@ char			**origin;
  *			- # elements in array
  */
 void
-lam_copyrev2(dest, src, num)
-
-void			*dest;
-void			*src;
-int			num;
-
+lam_copyrev2(void *dest, void *src, int num)
 {
 	unsigned char	*d, *s;
 	int		i;
@@ -826,6 +785,7 @@ int			num;
 	}
 }
 
+
 /*
  *	lam_copyrev4
  *
@@ -837,12 +797,7 @@ int			num;
  *			- # elements in array
  */
 void
-lam_copyrev4(dest, src, num)
-
-void			*dest;
-void			*src;
-int			num;
-
+lam_copyrev4(void *dest, void *src, int num)
 {
 #ifdef LAM_USE_SHIFTING
 	int		i;
@@ -878,6 +833,7 @@ int			num;
 #endif	
 }
 
+
 /*
  *	lam_copyrev8
  *
@@ -889,12 +845,7 @@ int			num;
  *			- # elements in array
  */
 void
-lam_copyrev8(dest, src, num)
-
-void			*dest;
-void			*src;
-int			num;
-
+lam_copyrev8(void *dest, void *src, int num)
 {
 	unsigned char	*d, *s;
 	int		i;
@@ -915,6 +866,7 @@ int			num;
 	}
 }
 
+
 /*
  *	lam_copyrevn
  *
@@ -927,13 +879,7 @@ int			num;
  *			- # elements in array
  */
 void
-lam_copyrevn(dest, src, nbytes, num)
-
-void			*dest;
-void			*src;
-int			nbytes;
-int			num;
-
+lam_copyrevn(void *dest, void *src, int nbytes, int num)
 {
 	unsigned char	*d, *s;
 	int		i, j;
