@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: laminit.c,v 6.24.2.1 2002/10/09 19:49:10 brbarret Exp $
+ * $Id: laminit.c,v 6.62.2.1 2003/09/11 20:17:07 jsquyres Exp $
  *
  *	Function:	- LAM specific initialization for MPI
  */
@@ -22,25 +20,33 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/param.h>
 
+#include <lam_config.h>
 #include <app_mgmt.h>
 #include <blktype.h>
 #include <laminternal.h>
 #include <mpi.h>
 #include <mpisys.h>
 #include <mpitrace.h>
+#include <mpi_apps.h>
 #include <net.h>
 #include <preq.h>
+#include <lam-ssi-rpi.h>
+#include <lam-ssi-cr.h>
 #include <rpisys.h>
 #include <terror.h>
 #include <typical.h>
 #include <t_types.h>
-#include <args.h>
+#include <etc_misc.h>
 #include <trreq.h>
 #if LAM_WANT_IMPI
 #include <impi.h>
 #endif
-#include <debug.h>
+#include <lamdebug.h>
+#include <lam_network.h>
+#include <lammpithreads.h>
+
 
 /*
  * public variables
@@ -78,6 +84,9 @@ struct _dtype		lam_mpi_2integer, lam_mpi_longdbl_int;
 struct _dtype		lam_mpi_wchar, lam_mpi_long_long_int;
 struct _dtype		lam_mpi_unsigned_long_long;
 
+struct _dtype lam_mpi_cxx_cplex, lam_mpi_cxx_dblcplex, lam_mpi_cxx_ldblcplex;
+struct _dtype lam_mpi_cxx_bool;
+
 struct _errhdl		lam_mpi_errors_are_fatal;
 struct _errhdl		lam_mpi_errors_return;
 
@@ -92,22 +101,24 @@ int			lam_ger = 0;
 int			lam_homog = 0;
 struct jobid_t		lam_jobid = { 0, 0 };
 int			lam_universe_size = -1;
+int			lam_universe_ncpus = -1;
+int			lam_universe_nnodes = -1;
 int			lam_appnum = -1;
-int		        lam_maxtag = LAM_MAXTAG;
+int		        lam_mpi_max_tag = INT_MAX;
+int		        lam_mpi_max_cid = 65536;
 
 MPI_Fint		*MPI_F_STATUS_IGNORE;
 MPI_Fint		*MPI_F_STATUSES_IGNORE;
 
+char                    *lam_argv0 = 0;
+
+OPT *                   lam_mpi_ao = NULL;
+
 
 #if LAM_WANT_IMPI
 struct _gps             gimpid;
 #endif
 
-/*
- * private variables
- */
-static int fl_debug = 0;
-
 
 /*
  * private functions
@@ -115,6 +126,12 @@ static int fl_debug = 0;
 static int get_singleton_idx(int pid);
 
 
+/*
+ * global functions
+ */
+int lam_send_pid_idx(struct nmsg *nhead);
+
+
 /*
  *	lam_linit
  *
@@ -128,33 +145,33 @@ static int get_singleton_idx(int pid);
  *	Returns:	- 0 or LAMERROR
  */
 int
-lam_linit(char *name, int *world_n, int *parent_n, int *cid, 
-	  struct _gps **worlds, int *root)
+lam_linit(int argc, char *argv[], char *name, int *world_n, int *parent_n, 
+	  int *cid, struct _gps **worlds, int *root)
 {
-    	struct pstate	pstate;			/* for getting kenya index */
 	char		*trworld;		/* world trace */
 	int		trwlen;			/* length of world trace */
-	char            *env;
 	struct _gps	*procs;			/* process GPS array */
 	struct _gps	*p;			/* favourite pointer */
 	struct nmsg	nhead;			/* network message header */
 	struct _proc	*proc;			/* process entry */
 	int		i;
-	int	        procs_n;		/* # processes in total */
+	int	        procs_n;
 /*
  * Attach process to the kernel and turn it into an MPI process.
  */
-	DBUG("In laminit\n");
 
+	lam_argv0 = name;
 	if (kenter(name, 0)) {
 	  /* ENOKERNEL = no kernel */
 	  /* ENOKDESCRIPTORS = kernel descriptors full */
-	  if (errno == ENOKERNEL)
-	    show_help("MPI", "no-lamd", "MPI_Init", NULL);
-	  else if (errno == ENOKDESCRIPTORS) {
+	  if (errno == ENOKERNEL) {
+	    char hostname[MAXHOSTNAMELEN];
+	    gethostname(hostname, MAXHOSTNAMELEN);
+	    show_help("MPI", "no-lamd", "MPI_Init", hostname, NULL);
+	  } else if (errno == ENOKDESCRIPTORS) {
 	    char hostname[128];
 	    char num[128];
-	    gethostname(hostname, 128);
+	    lam_gethostname(hostname, 128);
 	    snprintf(num, 127, "%d", KPMAX);
 	    show_help("MPI", "lamd-full", "MPI_Init", num, hostname, NULL);
 	  }
@@ -162,6 +179,9 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 	  errno = ENOKERNEL;
 	  return(LAMERROR);
 	}
+
+        /* If we were forked (no mpirun), then disable all tracing. */
+
 	if (_kio.ki_rtf & RTF_FORKED) {
 	  _kio.ki_rtf &= ~RTF_TRON;
 	}
@@ -174,11 +194,18 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 	  _kio.ki_rtf |= RTF_MPIC2C;
 
 	_kio.ki_rtf |= RTF_MPI;
-/*
- * If process was not launched with mpirun, it will not be set to 
- * catch signals.  Catching signal is 1) a good thing, and 2) the mpirun
- * default, so we should set it here by default.
- */
+
+	/* If process was not launched with mpirun, there will not
+	   necessarily be a way to notice signals.  Normally, the lamd
+	   will notice that a process died by a signal and propogate
+	   that information back to mpirun, who will print out a
+	   warning message and then call rpdoom().  If there is no
+	   mpirun, we don't have that luxury (particularly in the
+	   model where the user launches "./master", who then invokes
+	   MPI_COMM_SPAWN to launch a bunch of slaves).  So we have to
+	   handle signals here (print out an error message, call
+	   rpdoom(), and possibly call the next signal handler in the
+	   chain). */
 	if ((_kio.ki_rtf & RTF_MPIRUN) == 0)
 	  _kio.ki_rtf |= RTF_MPISIGS;
 	if (lpattach(name)) 
@@ -189,7 +216,15 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 	    kpause();
 	}
 
-	DBUG("Before lam_get_proc_array\n");
+	/* Setup the argc/argv argument parsing */
+
+	lam_mpi_ao = ao_init();
+	ao_setflags(lam_mpi_ao, AOPRESERVE_ARGV | AOIGN_UNRECOG);
+        if (argc > 0 && argv != NULL)
+          ao_parse(lam_mpi_ao, &argc, argv);
+
+	/* Get the proc array from whoever launched us */
+
 	procs_n = 0; 
 	if (lam_get_proc_array(&procs, world_n, cid, &procs_n, 0)
 	    == LAMERROR) {
@@ -202,39 +237,13 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
  */
 	lam_c2c = _kio.ki_rtf & RTF_MPIC2C;
 	lam_ger = _kio.ki_rtf & RTF_MPIGER;
-	lam_homog = _kio.ki_rtf & RTF_HOMOG;
 /* 
  * If started by mpirun, send back my real PID and IDX to give
  * mpirun the ability to run shell scripts and debuggers.
  */
 	LAM_ZERO_ME(nhead);
 	if (_kio.ki_rtf & RTF_MPIRUN) {
-	    if ((env = getenv("LAMRANK")) == 0) {
-		free((char *) procs);
-		return(LAMERROR);
-	    }
-/*
- * Get our process (kenya) index.
- */
-	    if (rpstate(LOCAL, SELECT_PID, getpid(), &pstate, 1) < 1) {
-		free((char *) procs);
-		return(LAMERROR);
-	    }
-/*
- * The event is -(mpirun's pid). This will distinguish it from user
- * events. The type is 2 to distinguish it from "wait" messages which
- * the same event and type 1.
- */
-	    nhead.nh_node = _kio.ki_jobid.jid_node;
-	    nhead.nh_event = (-_kio.ki_jobid.jid_pid) & 0xBFFFFFFF;
-	    nhead.nh_type = 2;
-	    nhead.nh_flags = DINT4DATA;
-	    nhead.nh_length = 0;
-	    nhead.nh_data[0] = stoi((unsigned char*) env);
-	    nhead.nh_data[1] = getpid();
-	    nhead.nh_data[2] = pstate.ps_index;
-
-	    if (nsend(&nhead)) {
+	    if (lam_send_pid_idx(&nhead)) {
 		free((char *) procs);
 		return(LAMERROR);
 	    }
@@ -247,7 +256,7 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
  */
 	if ((_kio.ki_parent > 0) || (_kio.ki_rtf & RTF_MPIRUN)) {
 
-		nhead.nh_event = (-getpid()) & 0xBFFFFFFF;
+		nhead.nh_event = (-lam_getpid()) & 0xBFFFFFFF;
 		nhead.nh_type = BLKMPIINIT;
 		nhead.nh_flags = DINT4DATA;
 		nhead.nh_length = procs_n * sizeof(struct _gps);
@@ -270,13 +279,13 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 	} else {
 		if (procs_n == 1) {
 			procs->gps_node = getnodeid();
-			procs->gps_pid = getpid();
+			procs->gps_pid = lam_getpid();
 			procs->gps_idx = get_singleton_idx(procs->gps_pid);
 			if (procs->gps_idx < 0)	return(LAMERROR);
 		} else {
 			for (i = 0, p = procs; i < procs_n; ++i, ++p) {
 				p->gps_node = i;
-				p->gps_pid = getpid();
+				p->gps_pid = lam_getpid();
 				p->gps_idx = 0;
 			}
 		}
@@ -296,10 +305,13 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 	for (i = 0, p = procs; i < *world_n; ++i, ++p) {
 		p->gps_grank = i;
 		proc = lam_procadd(p);
-		if (proc == 0) return(LAMERROR);
+		if (proc == 0) 
+                  return(LAMERROR);
 
-		if ((p->gps_node == getnodeid()) && (p->gps_pid == getpid())) {
+		if ((p->gps_node == getnodeid()) && 
+                    (p->gps_pid == lam_getpid())) {
 			lam_myproc = proc;
+			lam_myproc->p_mode |= LAM_PHOMOG;
 		}
 
 		if (_kio.ki_parent > 0) {
@@ -316,12 +328,7 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 		}
 		proc->p_mode |= LAM_PCLIENT;
 	}  
-/*
- * interface specific initialization
- */
-	if (RPI_SPLIT(_rpi_lamd_init, _rpi_c2c_init, ())) {
-		return(LAMERROR);
-	}
+
 /*
  * Synchronize clocks across the nodes.
  */
@@ -363,6 +370,7 @@ lam_linit(char *name, int *world_n, int *parent_n, int *cid,
 
 }
 
+
 /*
  *	init_comm_world
  *
@@ -414,6 +422,7 @@ lam_init_comm_world(struct _gps *mpi_procs, int mpi_nprocs)
 	lam_setcid(cid);
 	comm->c_errhdl->eh_refcount++;
 	strcpy(comm->c_name, "MPI_COMM_WORLD");
+
 	if (LAM_TRACE_TOP())
 	  lam_tr_commname(comm->c_name, comm->c_contextid, MPI_Wtime());
 
@@ -423,6 +432,7 @@ lam_init_comm_world(struct _gps *mpi_procs, int mpi_nprocs)
 	return 0;
 }
 
+
 /*
  *	get_singleton_idx
  *
@@ -442,6 +452,7 @@ get_singleton_idx(int pid)
 	return(state.ps_index);
 }
 
+
 int
 lam_get_proc_array(struct _gps **procs, int *world_n, int *cid,
 		   int *procs_n, int is_impi)
@@ -449,7 +460,6 @@ lam_get_proc_array(struct _gps **procs, int *world_n, int *cid,
 /*
  *  This subroutine is called twice if it is an IMPI job 
  */
-  DBUG("In get_proc_array\n");
 
 /*
  * Get the # of processes in the world.
@@ -462,11 +472,7 @@ lam_get_proc_array(struct _gps **procs, int *world_n, int *cid,
     } else {
       *world_n = getenv("LAMWORLDNODES") ? getncomp() : 1;
     }
-    
-    DBUG("IN laminit, the world is size %d\n", *world_n);
-    
-  }
-  else {
+  } else {
     
 #if LAM_WANT_IMPI
     /* 
@@ -477,15 +483,8 @@ lam_get_proc_array(struct _gps **procs, int *world_n, int *cid,
      * for them
      */
 
-    DBUG("IN laminit -- IMPI, the world_n is  %d\n", *world_n);
-    DBUG("IN laminit -- IMPI, the impi_nprocs is  %d\n",  impi_server_info.nprocs);
-        
     *cid = impi_server_info.min_cid;
-    lam_maxtag = impi_server_info.max_tag;
-    
-    DBUG("IMPID is node %d idx %d pid %d\n", gimpid.gps_node,
-	   gimpid.gps_idx, gimpid.gps_pid);
-    DBUG("Done with initial IMPI setup\n");
+    lam_mpi_max_tag = impi_server_info.max_tag;
 #endif
   }
   
@@ -494,11 +493,54 @@ lam_get_proc_array(struct _gps **procs, int *world_n, int *cid,
   *procs = (struct _gps *)
     malloc((unsigned) *procs_n * sizeof(struct _gps));
 
-    DBUG("IN laminit, the procs_n is size %d\n", *procs_n);
-  
   if (*procs == 0)
     return(LAMERROR);
   else
     return 0;
 }
 
+
+/*
+ * 	lam_send_pid_idx
+ *
+ * 	Function:	- sends the pid and kenyad index to mpirun
+ * 	Accepts:	- network message descriptor
+ * 	Returns:	- 0 (on success) or LAMERROR
+ */
+int 
+lam_send_pid_idx(struct nmsg *nhead)
+{
+  char *env;
+  struct pstate	pstate;
+  struct mpi_app_proc_info *mapi;
+
+  if ((env = getenv("LAMRANK")) == 0)
+    return(LAMERROR);
+  
+  /*
+   * Get our process (kenya) index.
+   */
+  if (rpstate(LOCAL, SELECT_PID, lam_getpid(), &pstate, 1) < 1)
+    return(LAMERROR);
+  
+  /*
+   * The event is -(mpirun's pid). This will distinguish it from user
+   * events. The type is 2 to distinguish it from "wait" messages which
+   * the same event and type 1.
+   */
+  LAM_ZERO_ME(mapi);
+  nhead->nh_node = _kio.ki_jobid.jid_node;
+  nhead->nh_event = (-_kio.ki_jobid.jid_pid) & 0xBFFFFFFF;
+  nhead->nh_type = 2;
+  nhead->nh_flags = DINT4DATA;
+  nhead->nh_length = 0;
+  mapi = (struct mpi_app_proc_info *) &nhead->nh_data;
+  mapi->mapi_mcw_rank = stoi((unsigned char*) env);
+  mapi->mapi_pid = lam_getpid();
+  mapi->mapi_ps_index = pstate.ps_index;
+
+  if (nsend(nhead))
+    return(LAMERROR);
+
+  return(0);
+}
