@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: lammisc.c,v 6.13.2.1 2002/10/09 19:49:10 brbarret Exp $
+ * $Id: lammisc.c,v 6.16.2.2 2003/10/11 19:04:02 brbarret Exp $
  *
  *	Function:	- miscellaneous functions
  */
@@ -22,17 +20,17 @@
 #include <lam_config.h>
 
 #include <string.h>
-#if HAVE_STRINGS_H
+#if defined(HAVE_STRINGS_H) && HAVE_STRINGS_H
 #include <strings.h>
 #endif
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/time.h>
-#if LAM_NEED_SYS_SELECT_H
+#if defined(HAVE_SYS_SELECT_H) && HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#if HAVE_SCHED_H
+#if defined(HAVE_SCHED_H) && HAVE_SCHED_H
 #include <sched.h>
 #endif
 
@@ -44,7 +42,7 @@
 /*
  * prototypes for OS's that don't have them (cough cough AIX 4.1 cough cough)
  */
-#if HAVE_YIELD && !LAM_USE_SELECT_YIELD
+#if LAM_HAVE_YIELD && !LAM_USE_SELECT_YIELD
 void yield(void);
 #endif
 
@@ -126,11 +124,11 @@ MPI_Status		*pstat;
 void
 lam_yield()
 {
-#if HAVE_YIELD && !LAM_USE_SELECT_YIELD
+#if LAM_HAVE_YIELD && !LAM_USE_SELECT_YIELD
 
 	yield();
 
-#elif HAVE_SCHED_YIELD && !LAM_USE_SELECT_YIELD
+#elif LAM_HAVE_SCHED_YIELD && !LAM_USE_SELECT_YIELD
 
 	sched_yield();
 
