@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: lamonesided.c,v 1.4.2.2 2002/10/09 19:49:10 brbarret Exp $
+ * $Id: lamonesided.c,v 1.11.2.2 2004/01/02 00:01:12 jsquyres Exp $
  *
  *	Function:	- one sided communication
  *
@@ -48,11 +46,11 @@ struct accstate {
 /*
  * private functions
  */
-static MPI_Datatype	header2dtype();
-static int		accumulate_handler();
-static int		lam_accumulate();
-static int		lam_get();
-static int		lam_put();
+static int lam_accumulate(MPI_Win win, int rank);
+static int lam_put(MPI_Win win, int rank);
+static int lam_get(MPI_Win win, int rank);
+static MPI_Datatype header2dtype(struct _osdhdr *hdr);
+static int accumulate_handler(MPI_Request req);
 
 /*
  * public variables
@@ -66,8 +64,7 @@ MPI_Datatype		lam_osdhdr_type = MPI_DATATYPE_NULL;
  *	Returns:	- 0 or LAMERROR
  */
 int
-lam_init_onesided()
-
+lam_init_onesided(void)
 {
 	struct _osdhdr	header;			/* 1-sided header packet */
 	MPI_Datatype	types[2];
@@ -127,8 +124,7 @@ lam_init_onesided()
  *	Function:	- cleanup one-sided communication
  */
 void
-lam_free_onesided()
-
+lam_free_onesided(void)
 {
 	MPI_Type_free(&lam_osdhdr_type);
 }
@@ -140,10 +136,7 @@ lam_free_onesided()
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_blkadv(win)
-
-MPI_Win			win;
-
+lam_osd_blkadv(MPI_Win win)
 {
 	int		err;
 /*
@@ -168,8 +161,7 @@ MPI_Win			win;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_push()
-
+lam_osd_push(void)
 {
 /*
  * This is always done non-blocking.
@@ -187,11 +179,7 @@ lam_osd_push()
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_complete(win, reqtype)
-
-MPI_Win			win;
-int			reqtype;
-
+lam_osd_complete(MPI_Win win, int reqtype)
 {
 	MPI_Request	*reqs;
 	MPI_Request	p;
@@ -246,26 +234,24 @@ int			reqtype;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_fillheader(func, op, disp, count, dtype, header)
-
-int			func;
-int			op;
-MPI_Aint		disp;
-int			count;
-MPI_Datatype		dtype;
-struct _osdhdr		*header;
-
+lam_osd_fillheader(int func, int op, MPI_Aint disp, int count, 
+		   MPI_Datatype dtype, struct _osdhdr *header)
 {
+	MPI_Datatype t = MPI_DATATYPE_NULL;
 /*
  * Check if datatype supported.  Currently we only support basic types and
  * vectors or contigs of basic types.
  */
-	if (dtype && !(dtype->dt_format == LAM_DTBASIC
-		|| ((dtype->dt_format == LAM_DTVECTOR
-			|| dtype->dt_format == LAM_DTCONTIG)
-			&& dtype->dt_dtype->dt_format == LAM_DTBASIC))) {
+	if (dtype) {
+		t = dtype;
+		while (t->dt_flags & LAM_DTISDUP)
+			t = (MPI_Datatype)t->dt_uargs;
 
-		return(MPI_ERR_TYPENOTSUP);
+		if (!(t->dt_format == LAM_DTBASIC
+			|| ((t->dt_format == LAM_DTVECTOR
+				|| t->dt_format == LAM_DTCONTIG)
+				&& t->dt_dtype->dt_format == LAM_DTBASIC)))
+			return(MPI_ERR_TYPENOTSUP);
 	}
 /*
  * Fill in information.
@@ -279,17 +265,21 @@ struct _osdhdr		*header;
 
 	if (dtype) {
 		header->os_count = count;
-		header->os_format = dtype->dt_format;
+		header->os_format = t->dt_format;
 
-		if (dtype->dt_format == LAM_DTBASIC) {
-			header->os_type = dtype->dt_f77handle;
-		} else if (dtype->dt_format == LAM_DTCONTIG) {
-			header->os_type = dtype->dt_dtype->dt_f77handle;
-			header->os_blklen = dtype->dt_count;
+		if (t->dt_format == LAM_DTBASIC) {
+			header->os_type = t->dt_f77handle;
 		} else {
-			header->os_type = dtype->dt_dtype->dt_f77handle;
-			header->os_blklen = dtype->dt_length;
-			header->os_stride = dtype->dt_stride;
+			t = t->dt_dtype;
+			while (t->dt_flags & LAM_DTISDUP)
+				t = (MPI_Datatype)t->dt_uargs;
+
+			header->os_type = t->dt_f77handle;
+			header->os_dcount = dtype->dt_count;
+			if (dtype->dt_format == LAM_DTVECTOR) {
+				header->os_blklen = dtype->dt_length;
+				header->os_stride = dtype->dt_stride;
+			}
 		}
 	}
 
@@ -306,26 +296,17 @@ struct _osdhdr		*header;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_send(buffer, count, dtype, dest, tag, comm, origin, dynamic)
-
-void			*buffer;
-int			count;
-MPI_Datatype		dtype;
-int			dest;
-int			tag;
-MPI_Comm		comm;
-int			origin;
-int			dynamic;
-
+lam_osd_send(void *buffer, int count, MPI_Datatype dtype, int dest, 
+	     int tag, MPI_Comm comm, int origin, int dynamic)
 {
 	MPI_Request	req = MPI_REQUEST_NULL;
 	int		savecid = comm->c_contextid;
 	int		err;
 
-	lam_mkpt(comm);
+	lam_mpi_mkpt(comm);
 
 	err = _mpi_req_build(buffer, count, dtype,
-			dest, tag, comm, LAM_RQISEND, &req);
+			     dest, tag, comm, LAM_RQISEND, &req);
 	if (err != MPI_SUCCESS) return(err);
 
 	if (dynamic) {
@@ -347,6 +328,9 @@ int			dynamic;
 	} else {
 		req->rq_flags |= LAM_RQFORPHAN;
 		req->rq_marks |= (origin) ? LAM_RQFOSORIG : LAM_RQFOSTARG;
+#if LAM_WANT_DEBUG
+		MPIL_Request_set_name(req, "Onesided send request");
+#endif
 		_mpi_req_add_m(req);
 	}
 
@@ -363,22 +347,14 @@ int			dynamic;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_recv(buffer, count, dtype, src, tag, comm, origin)
-
-void			*buffer;
-int			count;
-MPI_Datatype		dtype;
-int			src;
-int			tag;
-MPI_Comm		comm;
-int			origin;
-
+lam_osd_recv(void *buffer, int count, MPI_Datatype dtype, int src, 
+	     int tag, MPI_Comm comm, int origin)
 {
 	MPI_Request	req = MPI_REQUEST_NULL;
 	int		savecid = comm->c_contextid;
 	int		err;
 
-	lam_mkpt(comm);
+	lam_mpi_mkpt(comm);
 
 	err = _mpi_req_build(buffer, count, dtype,
 			src, tag, comm, LAM_RQIRECV, &req);
@@ -414,10 +390,7 @@ int			origin;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_osd_handler(req)
-
-MPI_Request		req;
-
+lam_osd_handler(MPI_Request req)
 {
 	MPI_Win		win;			/* communication window */
 	int		rank;			/* rank of origin */
@@ -433,7 +406,7 @@ MPI_Request		req;
 				|| win->w_pstate[rank] & LAM_WFPOSTED)) {
 
 			if (win->w_flags & LAM_WFNOOUT) {
-				return(lam_mkerr(MPI_ERR_EPOCH, 0));
+				return(lam_mkerr(MPI_ERR_EPOCH, EINVAL));
 			} else {
 				win->w_flags |= LAM_WFACCEPOCH | LAM_WFEXPEPOCH;
 			}
@@ -449,7 +422,7 @@ MPI_Request		req;
 				|| win->w_pstate[rank] & LAM_WFPOSTED)) {
 
 			if (win->w_flags & LAM_WFNOOUT) {
-				return(lam_mkerr(MPI_ERR_EPOCH, 0));
+				return(lam_mkerr(MPI_ERR_EPOCH, EINVAL));
 			} else {
 				win->w_flags |= LAM_WFACCEPOCH | LAM_WFEXPEPOCH;
 			}
@@ -465,7 +438,7 @@ MPI_Request		req;
 				|| win->w_pstate[rank] & LAM_WFPOSTED)) {
 
 			if (win->w_flags & LAM_WFNOOUT) {
-				return(lam_mkerr(MPI_ERR_EPOCH, 0));
+				return(lam_mkerr(MPI_ERR_EPOCH, EINVAL));
 			} else {
 				win->w_flags |= LAM_WFACCEPOCH | LAM_WFEXPEPOCH;
 			}
@@ -486,7 +459,7 @@ MPI_Request		req;
 		break;
 
 	default:
-		return(lam_mkerr(MPI_ERR_INTERN, 0));
+		return(lam_mkerr(MPI_ERR_INTERN, EINVAL));
 	}
 
 	req->rq_state = LAM_RQSINIT;
@@ -502,11 +475,7 @@ MPI_Request		req;
  *	Returns:	- MPI_SUCCESS or error ode
  */
 static int
-lam_accumulate(win, rank)
-
-MPI_Win			win;
-int			rank;
-
+lam_accumulate(MPI_Win win, int rank)
 {
 	MPI_Request	req = MPI_REQUEST_NULL;
 	MPI_Datatype	dtype;
@@ -539,7 +508,7 @@ int			rank;
 	if (err != MPI_SUCCESS) return(err);
 
 	savecid = win->w_comm->c_contextid;
-	lam_mkpt(win->w_comm);
+	lam_mpi_mkpt(win->w_comm);
 
 	err = _mpi_req_build(origin, win->w_header.os_count, dtype, rank,
 			LAM_OSD_ACCUMULATE, win->w_comm, LAM_RQIRECV, &req);
@@ -605,11 +574,7 @@ int			rank;
  *	Returns:	- MPI_SUCCESS or error ode
  */
 static int
-lam_put(win, rank)
-
-MPI_Win			win;
-int			rank;
-
+lam_put(MPI_Win win, int rank)
 {
 	char		*buffer;
 	MPI_Datatype	dtype;
@@ -635,11 +600,7 @@ int			rank;
  *	Returns:	- MPI_SUCCESS or error ode
  */
 static int
-lam_get(win, rank)
-
-MPI_Win			win;
-int			rank;
-
+lam_get(MPI_Win win, int rank)
 {
 	char		*buffer;
 	MPI_Datatype	dtype;
@@ -663,10 +624,7 @@ int			rank;
  *	Returns:	- MPI_SUCCESS or error code
  */
 static int
-accumulate_handler(req)
-
-MPI_Request		req;
-
+accumulate_handler(MPI_Request req)
 {
 	struct accstate	*extra;
 /*
@@ -692,10 +650,7 @@ MPI_Request		req;
  *	Returns:	- datatype or MPI_DATATYPE_NULL (error)
  */
 static MPI_Datatype
-header2dtype(hdr)
-
-struct _osdhdr		*hdr;
-
+header2dtype(struct _osdhdr *hdr)
 {
 	MPI_Datatype	dtype;
 	int		err;
@@ -712,7 +667,7 @@ struct _osdhdr		*hdr;
 		dtype = GETHDL(hdr->os_type);
 	}
 	else {
-		err = MPI_Type_vector(hdr->os_count, hdr->os_blklen,
+		err = MPI_Type_vector(hdr->os_dcount, hdr->os_blklen,
 				hdr->os_stride, GETHDL(hdr->os_type), &dtype);
 
 		if (err != MPI_SUCCESS) {
