@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,21 +10,59 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: lamtrace.c,v 6.8.2.1 2002/10/09 19:49:11 brbarret Exp $
+ * $Id: lamtrace.c,v 6.15.2.2 2003/09/11 20:18:36 jsquyres Exp $
  *
  *	Function:	- MPI trace generation
+ *
+ * Note: there are two levels of control for tracing.
+ *
+ * Enabled / disabled: This is controlled by the RTF_TRACE run-time
+ * flag.  This controls whether *any* tracing information is saved.
+ *
+ * On / off: This is controlled by the RTF_SWITCH run-time flag.  This
+ * only controls whether message traffic is traced.
+ *
+ * All four cases expressed in (RTF_TRACE, RTF_SWITCH) pairs:
+ *
+ * (0, 0): No tracing information is saved.
+ * (0, 1): No tracing information is saved.
+ * (1, 0): Only datatype and communicator information is traced.
+ * (1, 1): Datatype, communicator, and message information is traced.
+ *
+ * The rationale behind this is that if you want to be able to only
+ * trace a small portion of your program, then you need to *enable* it
+ * at the beginning of the application (i.e., with a switch to mpirun,
+ * which does some magic in MPI_INIT).  In this case, we need to trace
+ * all datatype and communicator information, regardless of whether
+ * you are saving message passing trace information or not.
+ *
+ * If we didn't always save datatype/communicator info (regardless of
+ * on/off state), you can easily imagine a problematic scenario such
+ * as the following:
+ *
+ * T=0 enable tracing
+ * T=1 turn off tracing
+ * T=2 create communicator 7
+ * T=3 turn on tracing
+ * T=4 send on communicator 7
+ *
+ * This would be a problem because the message trace would not be able
+ * to be related to any [traced] communicator, and would result in
+ * confusion.  Hence, if tracing is *enabled*, we always have to store
+ * datatype and communicator tracing information, and conditionally
+ * store message tracing information.
  */
 
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include <lam_config.h>
 #include <app_mgmt.h>
 #include <blktype.h>
+#include <etc_misc.h>
 #include <mpi.h>
 #include <mpisys.h>
 #include <mpitrace.h>
@@ -89,12 +127,13 @@ lam_tr_comm(MPI_Comm comm)
 	struct _gps	*pgps;			/* ptr GPS info */
 	struct _proc	*pproc;			/* ptr process */
 	struct _proc	**p;			/* favourite pointer */
-/*
- * Do nothing if in C2C mode and not tracing.
- */
-	if (lam_c2c && !(_kio.ki_rtf & RTF_TRACE)) {
-		return(0);
-	}
+
+        /* Do nothing if tracing is not *enabled* (always record comm
+           activity regardless of whether tracing is on or off). */
+
+	if ((_kio.ki_rtf & RTF_TRACE) == 0)
+          return 0;
+
 /*
  * Create the flat communicator description.
  */
@@ -105,11 +144,14 @@ lam_tr_comm(MPI_Comm comm)
 
 	buf = malloc((unsigned) bufsize);
 	if (buf == 0) return(LAMERROR);
+#if LAM_DISINFECT
+	memset(buf, 0, bufsize);
+#endif
 
 	psrc = (struct trsrc *) buf;
 
 	psrc->trs_node = getnodeid();
-	psrc->trs_pid = getpid();
+	psrc->trs_pid = lam_getpid();
 	psrc->trs_listno = TRCOMM;
 
 	pcid = (struct trcid *) (psrc + 1);
@@ -166,12 +208,13 @@ lam_tr_dtype(MPI_Datatype dtype)
 {
 	char		*buf;			/* datatype buffer */
 	int		bufsize;		/* buffer size */
-/*
- * Do nothing if in C2C mode and not tracing.
- */
-	if (lam_c2c && !(_kio.ki_rtf & RTF_TRACE)) {
-		return(0);
-	}
+
+        /* Do nothing if tracing is not *enabled* (regardless of
+           whether it is currently on or off). */
+
+	if ((_kio.ki_rtf & RTF_TRACE) == 0)
+          return 0;
+
 /*
  * If datatype has a label, use it.
  */
@@ -224,7 +267,7 @@ lam_tr_timer()
 /*
  *	lam_tr_off
  *
- *	Function:	- disable tracing
+ *	Function:	- turn off message tracing
  *			- stores a run time trace and a on/off list trace
  *	Returns:	- previous trace mode or LAMERROR
  */
@@ -240,16 +283,18 @@ lam_tr_off()
 	
 	trmode = _kio.ki_rtf & RTF_TRON;
 	_kio.ki_rtf &= ~RTF_TRSWITCH;
-/*
- * If we were tracing generate tracing off trace.
- */
+
+        /* If the previous mode was "on" (i.e., had *both* trace flags
+           on (TRACE and TRSWITCH)), then we need to drop an "off"
+           trace */
+
 	if (trmode == RTF_TRON) {
 		trsize = sizeof(struct trsrc) + sizeof(struct trrthdr)
 				+ sizeof(struct tronoff);
 		psrc = (struct trsrc *) lam_trpoint(trsize);
 		
 		psrc->trs_node = ltot(getnodeid());
-		psrc->trs_pid = ltot(getpid());
+		psrc->trs_pid = ltot(lam_getpid());
 		psrc->trs_listno = ltot(TRONOFF);
 
 		ptrr = (struct trrthdr *) (psrc + 1);
@@ -278,7 +323,7 @@ lam_tr_off()
 /*
  *	lam_tr_on
  *
- *	Function:	- enable tracing
+ *	Function:	- turn on tracing
  *			- stores a run time trace and a on/off list trace
  *	Returns:	- previous trace mode or LAMERROR
  */
@@ -294,16 +339,17 @@ lam_tr_on()
 	
 	trmode = _kio.ki_rtf & RTF_TRON;
 	_kio.ki_rtf |= RTF_TRSWITCH;
-/*
- * If we weren't tracing generate tracing on trace.
- */
-	if (trmode && !(trmode & RTF_TRSWITCH)) {
+
+        /* If the previous mode was "off" (i.e., had only TRACE), then
+           we need to drop an "on" trace message. */
+
+        if (trmode == RTF_TRACE) {
 		trsize = sizeof(struct trsrc) + sizeof(struct trrthdr)
 				+ sizeof(struct tronoff);
 		psrc = (struct trsrc *) lam_trpoint(trsize);
 		
 		psrc->trs_node = ltot(getnodeid());
-		psrc->trs_pid = ltot(getpid());
+		psrc->trs_pid = ltot(lam_getpid());
 		psrc->trs_listno = ltot(TRONOFF);
 
 		ptrr = (struct trrthdr *) (psrc + 1);
@@ -355,7 +401,7 @@ lam_tr_init(char *name, float8 skew)
 		if (psrc == 0) return(LAMERROR);
 		
 		psrc->trs_node = ltot(getnodeid());
-		psrc->trs_pid = ltot(getpid());
+		psrc->trs_pid = ltot(lam_getpid());
 		psrc->trs_listno = ltot(TRONOFF);
 
 		ptrr = (struct trrthdr *) (psrc + 1);
@@ -370,7 +416,7 @@ lam_tr_init(char *name, float8 skew)
 		if (name) {
 			p = strrchr(name, STRDIR);
 			p = (p == 0) ? name : p + 1;
-			strncpy(pinit->tri_name, p, TRDPROGMAX);
+			lam_strncpy(pinit->tri_name, p, TRDPROGMAX);
 		} else {
 			pinit->tri_name[0] = 0;
 		}
@@ -424,7 +470,7 @@ lam_tr_msg(int type, double time, int syst, int blkt, int peer, int tag,
 	psrc = (struct trsrc *) lam_trpoint(trsize);
 
 	psrc->trs_node = ltot(getnodeid());
-	psrc->trs_pid = ltot(getpid());
+	psrc->trs_pid = ltot(lam_getpid());
 	psrc->trs_listno = ltot(TRRUNTIME);
 	
 	ptrr = (struct trrthdr *) (psrc + 1);
@@ -439,7 +485,7 @@ lam_tr_msg(int type, double time, int syst, int blkt, int peer, int tag,
 	pmsg->trm_blkt = blkt;
 	pmsg->trm_peer = peer;
 	pmsg->trm_tag = tag;
-	pmsg->trm_cid = (comm) ? lam_coll2pt(comm->c_contextid) : 0;
+	pmsg->trm_cid = (comm) ? lam_mpi_coll2pt(comm->c_contextid) : 0;
 	pmsg->trm_dtype = (dtype) ? dtype->dt_label : 0;
 	pmsg->trm_count = count;
 	pmsg->trm_mrank = mrank;
@@ -471,7 +517,7 @@ lam_tr_sub(int func, double time, int sub)
 	psrc = (struct trsrc *) lam_trpoint(trsize);
 
 	psrc->trs_node = ltot(getnodeid());
-	psrc->trs_pid = ltot(getpid());
+	psrc->trs_pid = ltot(lam_getpid());
 	psrc->trs_listno = ltot(TRRUNTIME);
 
 	ptrr = (struct trrthdr *) (psrc + 1);
@@ -663,7 +709,7 @@ lam_tr_buoy(const char *name, double t)
   
   psrc = (struct trsrc *) malloc(trsize);
   psrc->trs_node = ltot(getnodeid());
-  psrc->trs_pid = ltot(getpid());
+  psrc->trs_pid = ltot(lam_getpid());
   psrc->trs_listno = ltot(TRRUNTIME);
   
   /* advance to the trrthdr struct */
@@ -676,7 +722,7 @@ lam_tr_buoy(const char *name, double t)
 
   pbuoy = (struct trbuoy *) (ptrr + 1);
   memset(pbuoy->trb_name, 0, sizeof(pbuoy->trb_name));
-  strncpy(pbuoy->trb_name, name, 32);
+  lam_strncpy(pbuoy->trb_name, name, 32);
   
   if (lam_rtrstore(LOCAL,
 		   TRRUNTIME,
@@ -712,7 +758,7 @@ lam_tr_coloron(int red, int green, int blue, double t)
   
   psrc = (struct trsrc *) malloc(trsize);
   psrc->trs_node = ltot(getnodeid());
-  psrc->trs_pid = ltot(getpid());
+  psrc->trs_pid = ltot(lam_getpid());
   psrc->trs_listno = ltot(TRRUNTIME);
   
   /* advance to the trrthdr struct */
@@ -761,7 +807,7 @@ lam_tr_coloroff(double t)
   
   psrc = (struct trsrc *) malloc(trsize);
   psrc->trs_node = ltot(getnodeid());
-  psrc->trs_pid = ltot(getpid());
+  psrc->trs_pid = ltot(lam_getpid());
   psrc->trs_listno = ltot(TRRUNTIME);
   
   /* advance to the trrthdr struct */
@@ -803,7 +849,7 @@ lam_tr_commname(char *name, int cid, double t)
   
   psrc = (struct trsrc *) malloc(trsize);
   psrc->trs_node = ltot(getnodeid());
-  psrc->trs_pid = ltot(getpid());
+  psrc->trs_pid = ltot(lam_getpid());
   psrc->trs_listno = ltot(TRRUNTIME);
   
   /* advance to the trrthdr struct */
@@ -816,7 +862,7 @@ lam_tr_commname(char *name, int cid, double t)
 
   pcomm = (struct trcommname *) (ptrr + 1);
   memset(pcomm->trcomm_name, 0, sizeof(pcomm->trcomm_name));
-  strncpy(pcomm->trcomm_name, name, 128);
+  lam_strncpy(pcomm->trcomm_name, name, 128);
   pcomm->trcomm_cid = ltot(cid);
   
   if (lam_rtrstore(LOCAL,
@@ -870,12 +916,15 @@ mkflatdtype(MPI_Datatype dtype, int dtlabel, char **pbuf, int *psize)
 
 		dtbuf = malloc((unsigned) dtbufsize);
 		if (dtbuf == 0) return(LAMERROR);
+#if LAM_DISINFECT
+		memset(dtbuf, 0, dtbufsize);
+#endif
 	}
 
 	psrc = (struct trsrc *) dtbuf;
 
 	psrc->trs_node = getnodeid();
-	psrc->trs_pid = getpid();
+	psrc->trs_pid = lam_getpid();
 	psrc->trs_listno = TRDTYPE;
 
 	pdhdr = (struct trdtype *) (psrc + 1);
@@ -981,6 +1030,11 @@ flat_basic(MPI_Datatype dtype, struct trdtype **pbuff, int *psize)
  * Make sure datatype buffer has at least 1 extra record.
  */
 	if (checkdtbuf(pbuff, psize, 1)) return(LAMERROR);
+/*
+ * Eliminate any dups.
+ */
+	while (dtype->dt_flags & LAM_DTISDUP)
+		dtype = (MPI_Datatype)dtype->dt_uargs;
 /*
  * Add the basic datatype record.
  */
@@ -1052,6 +1106,14 @@ flat_basic(MPI_Datatype dtype, struct trdtype **pbuff, int *psize)
 		type = TRDTF2REAL;
 	} else if (dtype == MPI_F_2DOUBLE_PRECISION) {
 		type = TRDTF2DBLPREC;
+	} else if (dtype == MPI_CXX_COMPLEX) {
+	  type = TRDTCXXCOMPLEX;
+	} else if (dtype == MPI_CXX_DOUBLE_COMPLEX) {
+	  type = TRDTCXXDCOMPLEX;
+	} else if (dtype == MPI_CXX_LONG_DOUBLE_COMPLEX) {
+	  type = TRDTCXXLDCOMPLEX;
+	} else if (dtype == MPI_CXX_BOOL) {
+	  type = TRDTCXXBOOL;
 	} else {
 		return(LAMERROR);
 	}
