@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: mpietc.c,v 6.6 2000/10/25 23:33:18 jsquyres Exp $
+ * $Id: mpietc.c,v 6.8.2.1 2003/08/21 06:32:00 brbarret Exp $
  *
  *	Function:	- misc. shared MPI-related functions
  *			- used by libmpi and MPI tools
@@ -43,6 +23,20 @@
 #include <mpisys.h>
 #include <rpisys.h>
 
+#if LAM_MPI_EXTRA
+#define lam_pt2coll lam_mpi_pt2coll
+#define lam_coll2pt lam_mpi_coll2pt
+#define lam_pt2impicoll lam_mpi_pt2impicoll
+#define lam_impicoll2pt lam_mpi_impicoll2pt
+#define lam_pt2impidatasync lam_mpi_pt2impidatasync
+#define lam_pt2impisyncack lam_mpi_pt2impisyncack
+#define lam_mkcoll lam_mpi_mkcoll
+#define lam_mkpt lam_mpi_mkpt
+#define lam_mkimpicoll lam_mpi_mkimpicoll
+#define lam_unsyncsql lam_mpi_unsyncsql
+#define lam_rank_in_group lam_mpi_rank_in_group
+#endif
+
 /*
  * macros
  */
