@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/GDB
+ * $HEADER$
  *
- *	$Id: recv.c,v 6.22.2.1 2002/10/09 19:49:12 brbarret Exp $
+ * $Id: recv.c,v 6.28 2003/02/20 19:57:13 jsquyres Exp $
  *
  *	Function:	- receive a message
  *	Accepts:	- buffer
@@ -35,18 +33,15 @@
 #if LAM_WANT_IMPI
 #include <impi.h>
 #endif
-#include <debug.h>
+#include <lamdebug.h>
+#include <lam-ssi-rpi.h>
+
 
 /*
  * state variable, has to be public.  Ugh. (in irecv.c)
  */
 extern int lam_in_impi_irecv;
 
-/*
- * private variables
- */
-static int fl_debug = 0;
-
 
 /*@
     MPI_Recv - Basic receive
@@ -94,9 +89,9 @@ int MPI_Recv(void *buf, int count, MPI_Datatype dtype,
 	lam_initerr_m();
 	lam_setfunc_m(BLKMPIRECV);
 
-	if ((tag < 0 && tag != MPI_ANY_TAG) || tag > LAM_MAXTAG) {
+	if ((tag < 0 && tag != MPI_ANY_TAG) || tag > lam_mpi_max_tag) {
 		return(lam_errfunc(comm, BLKMPIRECV,
-				lam_mkerr(MPI_ERR_TAG, 0)));
+				lam_mkerr(MPI_ERR_TAG, EINVAL)));
 	}
 /*
  * Check the bozo MPI_PROC_NULL case.  I'm not even tracing this.
@@ -136,9 +131,7 @@ int MPI_Recv(void *buf, int count, MPI_Datatype dtype,
              MPI_Irecv, so just relay the request to there, and then
              MPI_Wait on it */
 
-	  DBUG("IMPI SPECIAL CASE RECEIVE from rank %d\n", src);
 	  MPI_Irecv(buf, count, dtype, src, tag, comm, &req);
-	  DBUG("IMPI SPECIAL CASE RECEIVE: Calling MPI_Wait\n");
 	  MPI_Wait(&req, &status);
 
 	  if (stat != MPI_STATUS_IGNORE)
@@ -149,17 +142,14 @@ int MPI_Recv(void *buf, int count, MPI_Datatype dtype,
 	  if (status.MPI_ERROR != MPI_SUCCESS)
 	    return(lam_errfunc(comm, BLKMPIRECV, status.MPI_ERROR));
 
-	  DBUG("IMPI SPECIAL CASE RECEIVE: All done; returning\n");
 	  lam_resetfunc_m(BLKMPIRECV);
 	  return (MPI_SUCCESS);
 	}
 #endif
-	DBUG("NOT AN IMPI PROXY IRECEIVE\n");
 
-#if SHORTCIRCUIT
-	if (lam_c2c && lam_rq_nactv == 0) {
+	if (RPI_HAS_FASTRECV(comm, src) && lam_rq_nactv == 0) {
 		if (comm == MPI_COMM_NULL) {
-			return(lam_mkerr(MPI_ERR_COMM, 0));
+			return(lam_mkerr(MPI_ERR_COMM, EINVAL));
 		}
 
 		if (src != MPI_ANY_SOURCE && (LAM_IS_INTER(comm)
@@ -167,8 +157,8 @@ int MPI_Recv(void *buf, int count, MPI_Datatype dtype,
 
 		        int seqnum;
 
-			err = _rpi_c2c_fastrecv(buf, count, dtype,
-						src, &tag, comm, stat, &seqnum);
+			err = RPI_FASTRECV(buf, count, dtype, src, &tag, 
+					   comm, stat, &seqnum);
 
 			if (fl_trace) {
 			    finisht = MPI_Wtime();
@@ -186,7 +176,7 @@ int MPI_Recv(void *buf, int count, MPI_Datatype dtype,
 			return (MPI_SUCCESS);
 		}
 	}
-#endif
+
 	LAM_ZERO_ME(req_storage);
 	req = &req_storage;
 	err = _mpi_req_build(buf, count, dtype, 
