@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: spawnmult.c,v 6.24 2001/01/23 13:56:09 jsquyres Exp $
+ * $Id: spawnmult.c,v 6.40 2003/04/10 19:21:09 ssankara Exp $
  *
  *	Function:	- spawn multiple MPI programs
  *	Accepts:	- number of command lines
@@ -52,15 +32,18 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <lam_config.h>
 #include <app_mgmt.h>
 #include <app_schema.h>
 #include <args.h>
 #include <blktype.h>
+#include <etc_misc.h>
 #include <mpi.h>
 #include <mpisys.h>
 #include <ndi.h>
 #include <net.h>
 #include <portable.h>
+#include <lam-ssi-rpi.h>
 #include <rpisys.h>
 #include <terror.h>
 #include <typical.h>
@@ -129,7 +112,7 @@ processes can access the same intercommunicator by using the
 communicator is zero if the process was created by 'mpirun'(1) instead
 of one of the spawn functions.  Both groups can decide to merge the
 intercommunicator into an intracommunicator (with the
-'MPI_Comm_merge'() function) and take advantage of other MPI
+'MPI_Intercomm_merge'() function) and take advantage of other MPI
 collective operations.  They can then use the merged intracommunicator
 to create new communicators and reach other processes in the MPI
 application.
@@ -151,6 +134,20 @@ the given number of processes onto LAM nodes by starting with CPU 0
 numbers, placing one process on each CPU.  If the process count is
 greater than the CPU count, the procedure repeats.
 
+Process Terminiation
+
+Note that the process[es] spawned by 'MPI_COMM_SPAWN' (and
+'MPI_COMM_SPAWN_MULTIPLE') effectively become orphans.  That is, the
+spawnning MPI application does not wait for the spawned application to
+finish.  Hence, there is no guarantee the spawned application has
+finished when the spawning completes.  Similarly, killing the spawning
+application will also have no effect on the spawned application.
+
+User applications can effect this kind of behavior with 'MPI_BARRIER'
+between the spawning and spawned processed before 'MPI_FINALIZE'.
+
+Note that 'lamclean' will kill *all* MPI processes.
+ 
 Process Count
 
 The 'maxprocs' array parameter to 'MPI_Comm_spawn_multiple' specifies
@@ -172,6 +169,9 @@ the number of processes currently in the job and spawn the difference.
 LAM sets 'MPI_UNIVERSE_SIZE' to the number of CPUs in the user''s LAM
 session (as defined in the boot schema [bhost(5)] via 'lamboot'(1)).
  
+See MPI_Init(3) for other pre-defined attributes that are helpful when
+spawning.
+ 
 Locating an Executable Program
 
 The executable program file must be located on the node(s) where the
@@ -201,7 +201,7 @@ the program name followed by the strings in corresponding entry in the
 .N MPI_ERR_OTHER
 .N MPI_ERR_INTERN
 
-.seealso appschema(5), bhost(5), lamboot(1), MPI_Comm_get_parent(3), MPI_Comm_merge(3), MPI_Comm_spawn_multiple(3), MPI_Info_create(3), MPI_Info_set(3), MPI_Info_delete(3), MPI_Info_free(3), MPI_Init(3), mpirun(1)
+.seealso appschema(5), bhost(5), lamboot(1), MPI_Comm_get_parent(3), MPI_Intercomm_merge(3), MPI_Comm_spawn_multiple(3), MPI_Info_create(3), MPI_Info_set(3), MPI_Info_delete(3), MPI_Info_free(3), MPI_Init(3), mpirun(1)
 
 .N ACK
 @*/
@@ -233,24 +233,26 @@ MPI_Comm_spawn_multiple(int count, char **commands, char ***argvs,
  */
 	if ((comm == MPI_COMM_NULL) || LAM_IS_INTER(comm)) {
 		return(lam_errfunc(comm, BLKMPICOMMSPAWNMULT,
-					lam_mkerr(MPI_ERR_COMM, 0)));
+				   lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	MPI_Comm_size(comm, &size);
 
 	if ((root >= size) || (root < 0)) {
 		return(lam_errfunc(comm, BLKMPICOMMSPAWNMULT,
-				lam_mkerr(MPI_ERR_ROOT, 0)));
+				   lam_mkerr(MPI_ERR_ROOT, EINVAL)));
 	}
 
 	if (intercomm == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPICOMMSPAWNMULT, lam_mkerr(MPI_ERR_ARG, 0)));
+				   BLKMPICOMMSPAWNMULT, 
+				   lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 #if LAM_WANT_IMPI
 	if (LAM_IS_IMPI(comm)) {
-	  return lam_err_comm(comm, MPI_ERR_COMM, 0, "MPI_Comm_spawn_multiple not possible on IMPI communicators");
+	  return lam_err_comm(comm, MPI_ERR_COMM, ENOSYS,
+			      "MPI_Comm_spawn_multiple not possible on IMPI communicators");
 	}
 #endif
 	LAM_TRACE(lam_tr_cffstart(BLKMPICOMMSPAWNMULT));
@@ -295,7 +297,7 @@ MPI_Comm_spawn_multiple(int count, char **commands, char ***argvs,
 			err = lam_mkerr(MPI_ERR_OTHER, errno);
 		} else {
 			if (spawn(app, comm, cid, maxprocs, &numkids, &kids)) {
-				err = lam_mkerr(MPI_ERR_SPAWN, 0);
+			  err = lam_mkerr(MPI_ERR_SPAWN, ECHILD);
 			}
 			set_error_codes(err, app, errcodes, msg + 3);
 			asc_free(app);
@@ -379,6 +381,7 @@ MPI_Comm_spawn_multiple(int count, char **commands, char ***argvs,
 		free((char *) kids);
 		return(lam_errfunc(comm, BLKMPICOMMSPAWNMULT, err));
 	}
+        
 /*
  * Create the child group.
  */
@@ -411,6 +414,22 @@ MPI_Comm_spawn_multiple(int count, char **commands, char ***argvs,
 		(*p)->p_refcount++;
 	}
 	free((char *) kids);
+
+	/*
+	 * Get the endian bias and potentially reset lam_homog
+	 */
+	lam_exchange_endian_info(kgrp->g_procs, numkids);
+
+        /*
+         * Check to ensure that the RPI and CRMPI modules selected by the
+         * spawned processes match.
+         */
+        if ((rank == root) && (lam_match_ssi_modules(numkids, kids, "MPI_Comm_spawn"))) {
+          free(kids);
+          return(lam_errfunc(comm, BLKMPICOMMSPAWN,
+                             lam_mkerr(MPI_ERR_OTHER, errno)));
+        }
+
 /*
  * Create the parent-child intercommunicator.
  */
@@ -436,13 +455,15 @@ MPI_Comm_spawn_multiple(int count, char **commands, char ***argvs,
 	}
 
 	lam_setcid(cid);
+
 /*
  * setup new processes
  */
-	if (RPI_SPLIT(_rpi_lamd_addprocs, _rpi_c2c_addprocs, ())) {
+	if (RPI_ADDPROCS(kgrp->g_procs, kgrp->g_nprocs)) {
 		return(lam_errfunc(comm, BLKMPICOMMSPAWNMULT,
 				lam_mkerr(MPI_ERR_OTHER, errno)));
 	}
+
 /*
  * Wait until all the children have initialized.  The root waits for
  * rank 0 in the child world to communicate this fact and then
@@ -463,8 +484,14 @@ MPI_Comm_spawn_multiple(int count, char **commands, char ***argvs,
 
 	err = MPI_Bcast((void *) 0, 0, MPI_BYTE, root, comm);
 	
+	/* Let the collective SSI modules battle over who will do
+	   collectives on this new communicator.  */
+
+	if (lam_ssi_coll_base_init_comm(*intercomm) != 0)
+	  return(lam_errfunc(comm, BLKMPICOMMSPAWNMULT,
+			     lam_mkerr(MPI_ERR_INTERN, ENOSYS)));
+
 	LAM_TRACE(lam_tr_cffend(BLKMPICOMMSPAWNMULT, root, comm, 0, 0));
-	
 	lam_resetfunc_m(BLKMPICOMMSPAWNMULT);
 	return(MPI_SUCCESS);
 }
@@ -525,7 +552,7 @@ spawn(LIST *app, MPI_Comm comm, int cid, int *nprocs, int *numkids,
 	jobid.jid_node = _kio.ki_jobid.jid_node;
 	jobid.jid_pid = _kio.ki_jobid.jid_pid;
 	_kio.ki_jobid.jid_node = getnodeid();
-	_kio.ki_jobid.jid_pid = getpid();
+	_kio.ki_jobid.jid_pid = lam_getpid();
 	rtf |= RTF_MPIRUN;
 /*
  * Run the application.
@@ -565,6 +592,7 @@ spawn(LIST *app, MPI_Comm comm, int cid, int *nprocs, int *numkids,
  * Set up the message.
  */
 	MPI_Comm_rank(comm, &rank);
+	LAM_ZERO_ME(nhead);
 	nhead.nh_type = 0;
 	nhead.nh_flags = DINT4MSG;
 	nhead.nh_msg = (char *) worldout;
