@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: tcreatekey.c,v 1.1.2.1 2002/10/09 19:49:13 brbarret Exp $
+ * $Id: tcreatekey.c,v 1.3 2002/12/11 19:15:14 jsquyres Exp $
  *
  *	Function:	- create a datatype attribute key
  *	Accepts:	- copy function
