@@ -10,14 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Software for Humanity
- *	NJN
+ * $HEADER$
  *
- *	This program is freely distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- *	$Id: tdarray.c,v 1.3.2.1 2002/10/09 19:49:13 brbarret Exp $
+ * $Id: tdarray.c,v 1.5 2002/12/11 19:15:14 jsquyres Exp $
  *
  *	Function:	- create distributed array derived datatype
  *	Accepts:	- size of process group
@@ -102,12 +97,12 @@ MPI_Type_create_darray(int size, int rank, int ndims, int *gsizes,
  */
     if (size <= 0 || rank < 0 || ndims <= 0 || newtype == 0) {
 	return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_ARG, EINVAL)));
     }
 
     if (oldtype == MPI_DATATYPE_NULL) {
 	return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_TYPE, 0)));
+			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_TYPE, EINVAL)));
     }
 
     for (i = 0, pi = 1; i < ndims; ++i) {
@@ -120,7 +115,7 @@ MPI_Type_create_darray(int size, int rank, int ndims, int *gsizes,
 			     && distribs[i] != MPI_DISTRIBUTE_NONE))) {
 
 	    return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	if (distribs[i] == MPI_DISTRIBUTE_BLOCK
@@ -128,12 +123,12 @@ MPI_Type_create_darray(int size, int rank, int ndims, int *gsizes,
 
 	    if (dargs[i] * psizes[i] < gsizes[i]) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITCREATEDARRAY,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	    }
 	} else if (distribs[i] == MPI_DISTRIBUTE_NONE) {
 	    if (psizes[i] != 1) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITCREATEDARRAY,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	    }
 	}
 
@@ -142,7 +137,7 @@ MPI_Type_create_darray(int size, int rank, int ndims, int *gsizes,
 
     if (pi != size) {
 	return(lam_errfunc(MPI_COMM_WORLD, BLKMPITCREATEDARRAY,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
     }
 /*
  * Calculate position in Cartesian grid as MPI would (row-major
@@ -153,7 +148,8 @@ MPI_Type_create_darray(int size, int rank, int ndims, int *gsizes,
 
     if (coords == 0 || offsets == 0) {
 	return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_OTHER, 0)));
+			   BLKMPITCREATEDARRAY, 
+			   lam_mkerr(MPI_ERR_OTHER, EINVAL)));
     }
 
     for (s = size, r = rank, i = 0; i < ndims; i++) {
@@ -267,7 +263,7 @@ MPI_Type_create_darray(int size, int rank, int ndims, int *gsizes,
  * Invalid order.
  */
 	return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPITCREATEDARRAY, lam_mkerr(MPI_ERR_ARG, EINVAL)));
     }
 
     for (ub = extent, i = 0; i < ndims; i++) {
