@@ -1,47 +1,30 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1993 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: rflat.c,v 6.4 1999/06/16 05:54:53 jsquyres Exp $
+ *	$Id: rflat.c,v 6.8 2002/12/14 04:03:58 jsquyres Exp $
  * 
  *	Function:	- runtime access to the flat server
  */
 
+#include <lam_config.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 #include <args.h>
+#include <etc_misc.h>
 #include <events.h>
 #include <flatreq.h>
 #include <ksignal.h>
@@ -51,9 +34,9 @@
 #include <terror.h>
 #include <typical.h>
 
+
 /*
  *	Function:	- load a buffer into a node and assign it a tag
-
  *	Accepts:	- server's node ID
  *			- load buffer length
  *			- allocated buffer length
@@ -61,14 +44,7 @@
  *			- buffer ptr
  */
 int
-rflat(nodeid, buffer, ldlength, malength, tag)
-
-int4			nodeid;
-char			*buffer;
-int4			ldlength;
-int4			malength;
-int4			tag;
-
+rflat(int4 nodeid, char *buffer, int4 ldlength, int4 malength, int4 tag)
 {
 	struct flreq	*request;	/* flatd request structure */
 	struct flreply	*reply;		/* flatd reply structure */
@@ -77,13 +53,14 @@ int4			tag;
 
 	if (malength == 0) return(0);
 
+	LAM_ZERO_ME(nhead);
 	request = (struct flreq *) nhead.nh_data;
 	reply = (struct flreply *) nhead.nh_data;
 
 	request->flq_src_node =
 		((nodeid == LOCAL) || (getrtype(nodeid) & NT_CAST)) ?
 		nodeid : getnodeid();
-	request->flq_src_event = -getpid();
+	request->flq_src_event = -lam_getpid();
 	request->flq_req = FLQLOAD;
 	request->flq_ldlength = ldlength;
 	request->flq_malength = malength;
@@ -121,7 +98,7 @@ int4			tag;
 		request->flq_src_node =
 			((nodeid == LOCAL) || (getrtype(nodeid) & NT_CAST)) ?
 				nodeid : getnodeid();
-		request->flq_src_event = -getpid();
+		request->flq_src_event = -lam_getpid();
 		request->flq_req = FLQLOAD;
 		request->flq_ldlength = ldlength;
 		request->flq_malength = malength;
@@ -168,8 +145,9 @@ int4			tag;
 /*
  *	Function:	- load argument and environment vectors
  *			  into a node and assign them a tag
+ *			  the argument vector must be non-null
  *	Accepts:	- server's node ID
- *			- argument vector
+ *			- argument vector 
  *			- process environment
  *			- size of loaded environment (out)
  *			- tag (inout)
@@ -177,75 +155,58 @@ int4			tag;
 int
 rflatv(int4 nodeid, char **argv, struct penv *env, int4 *envsize, int4 *tag)
 {
+    char		*data = 0;
     char		*args;
     char		*envs;
     int			argsize;
     int			packsize;
-    void		*p;
 
-    if ((argv == 0) && (env->pe_envv == 0)) {
-	*envsize = 0;
-	return 0;
+    if (argv == 0) {
+	errno = LAM_ERR_INVAL;
+	return(LAMERROR);
     }
 
     if (*tag == 0)
-	*tag = (int4) getpid();
+	*tag = (int4) lam_getpid();
 
-    if (argv) {
-	args = argvglue(argv, 0xa, MAXNMSGLEN/2);
-	if (args == 0)
-	    return(LAMERROR);
-	argsize = strlen(args);
+    args = argvglue(argv, 0xa, 0);
+    if (args == 0)
+	return(LAMERROR);
 
-	if (env->pe_envv) {
-	    envs = argvglue(env->pe_envv, 0xa, MAXNMSGLEN/2);
-	    if (envs == 0) {
-		free(args);
-		return(LAMERROR);
-	    }
+    argsize = strlen(args);
 
-	    *envsize = (int4) strlen(envs);
-	    packsize = argsize + *envsize + 1;
-
-	    p = realloc(args, packsize);
-	    if (p == 0) {
-		free(args);
-		free(envs);
-		return(LAMERROR);
-	    }
-
-	    args = p;
-	    memcpy(args + argsize, envs, *envsize + 1);
-	    free(envs);
-	} else {
-	    *envsize = 0;
-	    packsize = argsize + 1;
-	}
-
-	if (rflat(nodeid, args, (int4) packsize, (int4) packsize, *tag)) {
-	    rflclean(nodeid, *tag);
+    if (env->pe_envv) {
+	envs = argvglue(env->pe_envv, 0xa, 0); 
+	if (envs == 0) {
 	    free(args);
 	    return(LAMERROR);
 	}
 
-	free(args);
-    }
-    else {
-	envs = argvglue(env->pe_envv, 0xa, MAXNMSGLEN/2);
-	if (envs == 0)
-	    return(LAMERROR);
+	*envsize = (int4) strlen(envs);
+	packsize = argsize + *envsize + 1;
 
-	*envsize = strlen(envs);
-	packsize = *envsize + 1;
-
-	if (rflat(nodeid, envs, (int4) packsize, (int4) packsize, *tag)) {
-	    rflclean(nodeid, *tag);
+	data = realloc(args, packsize);
+	if (data == 0) {
+	    free(args);
 	    free(envs);
 	    return(LAMERROR);
 	}
 
+	memcpy(data + argsize, envs, *envsize + 1);
 	free(envs);
     }
+    else {
+	*envsize = 0;
+	data = args;
+	packsize = argsize + 1;
+    }
 
+    if (rflat(nodeid, data, (int4) packsize, (int4) packsize, *tag)) {
+	rflclean(nodeid, *tag);
+	free(data);
+	return(LAMERROR);
+    }
+
+    free(data);
     return(0);
 }
